@@ -1,15 +1,14 @@
 /*
  * libbrlapi - A library providing access to braille terminals for applications.
  *
- * Copyright (C) 2006-2008 by Dave Mielke <dave@mielke.cc>
+ * Copyright (C) 2006-2011 by Dave Mielke <dave@mielke.cc>
  *
  * libbrlapi comes with ABSOLUTELY NO WARRANTY.
  *
  * This is free software, placed under the terms of the
  * GNU Lesser General Public License, as published by the Free Software
- * Foundation; either version 2.1 of the License,
- * or (at your option) any later version.
- * Please see the file COPYING-API for details.
+ * Foundation; either version 2.1 of the License, or (at your option) any
+ * later version. Please see the file LICENSE-LGPL for details.
  *
  * Web Page: http://mielke.cc/brltty/
  *
@@ -21,7 +20,7 @@
 #include <string.h>
 
 #include <tcl.h>
-#include "iso_11548-1.h"
+#include "brldots.h"
 
 #define BRLAPI_NO_DEPRECATED
 #define BRLAPI_NO_SINGLE_SESSION
