@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2010 by The BRLTTY Developers.
+ * Copyright (C) 1995-2011 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -110,12 +110,12 @@
 
 #include "log.h"
 #include "timing.h"
+#include "ascii.h"
+#include "hidkeys.h"
 #include "brltty.h"
 
 #define BRL_STATUS_FIELDS sfAlphabeticCursorCoordinates, sfAlphabeticWindowCoordinates, sfStateLetter
 #define BRL_HAVE_STATUS_CELLS
-#define BRL_HAVE_FIRMNESS
-#define BRLCONST
 #include "brl_driver.h"
 #include "brldefs-al.h"
 #include "braille.h"
@@ -214,11 +214,27 @@ BEGIN_KEY_NAME_TABLE(smartpad)
 END_KEY_NAME_TABLE
 
 BEGIN_KEY_NAME_TABLE(thumb)
-  KEY_NAME_ENTRY(AL_KEY_THUMB+0, "Thumb1"),
-  KEY_NAME_ENTRY(AL_KEY_THUMB+1, "Thumb2"),
-  KEY_NAME_ENTRY(AL_KEY_THUMB+2, "Thumb3"),
-  KEY_NAME_ENTRY(AL_KEY_THUMB+3, "Thumb4"),
-  KEY_NAME_ENTRY(AL_KEY_THUMB+4, "Thumb5"),
+  KEY_NAME_ENTRY(AL_KEY_THUMB+0, "ThumbLeft"),
+  KEY_NAME_ENTRY(AL_KEY_THUMB+1, "ThumbUp"),
+  KEY_NAME_ENTRY(AL_KEY_THUMB+2, "ThumbHome"),
+  KEY_NAME_ENTRY(AL_KEY_THUMB+3, "ThumbDown"),
+  KEY_NAME_ENTRY(AL_KEY_THUMB+4, "ThumbRight"),
+END_KEY_NAME_TABLE
+
+BEGIN_KEY_NAME_TABLE(featurepack)
+  KEY_NAME_ENTRY(AL_KEY_Dot1, "Dot1"),
+  KEY_NAME_ENTRY(AL_KEY_Dot2, "Dot2"),
+  KEY_NAME_ENTRY(AL_KEY_Dot3, "Dot3"),
+  KEY_NAME_ENTRY(AL_KEY_Dot4, "Dot4"),
+  KEY_NAME_ENTRY(AL_KEY_Dot5, "Dot5"),
+  KEY_NAME_ENTRY(AL_KEY_Dot6, "Dot6"),
+  KEY_NAME_ENTRY(AL_KEY_Dot7, "Dot7"),
+  KEY_NAME_ENTRY(AL_KEY_Dot8, "Dot8"),
+  KEY_NAME_ENTRY(AL_KEY_Control, "Control"),
+  KEY_NAME_ENTRY(AL_KEY_Windows, "Windows"),
+  KEY_NAME_ENTRY(AL_KEY_Space, "Space"),
+  KEY_NAME_ENTRY(AL_KEY_Alt, "Alt"),
+  KEY_NAME_ENTRY(AL_KEY_Enter, "Enter"),
 END_KEY_NAME_TABLE
 
 BEGIN_KEY_NAME_TABLES(abt_small)
@@ -255,6 +271,7 @@ BEGIN_KEY_NAME_TABLES(bc)
   KEY_NAME_TABLE(etouch),
   KEY_NAME_TABLE(smartpad),
   KEY_NAME_TABLE(thumb),
+  KEY_NAME_TABLE(featurepack),
   KEY_NAME_TABLE(routing1),
   KEY_NAME_TABLE(routing2),
 END_KEY_NAME_TABLES
@@ -429,6 +446,7 @@ typedef struct {
   int (*readBytes) (unsigned char *buffer, int length, int wait);
   int (*writeBytes) (const unsigned char *buffer, int length, unsigned int *delay);
   int (*getFeatureReport) (unsigned char report, unsigned char *buffer, int length);
+  int (*setFeatureReport) (unsigned char report, const unsigned char *data, int length);
 } InputOutputOperations;
 static const InputOutputOperations *io;
 
@@ -448,7 +466,6 @@ typedef enum {
   STATUS_RIGHT
 } StatusType;
 
-static TranslationTable outputTable;
 static unsigned char *previousText = NULL;
 static unsigned char *previousStatus = NULL;
 
@@ -489,14 +506,14 @@ reallocateBuffers (BrailleDisplay *brl) {
     if (reallocateBuffer(&previousStatus, brl->statusColumns*brl->statusRows))
       return 1;
 
-  LogPrint(LOG_ERR, "cannot allocate braille buffers");
+  logMessage(LOG_ERR, "cannot allocate braille buffers");
   return 0;
 }
 
 static int
 setDefaultConfiguration (BrailleDisplay *brl) {
-  LogPrint(LOG_INFO, "detected Alva %s: %d columns, %d status cells",
-           model->name, model->columns, model->statusCells);
+  logMessage(LOG_INFO, "detected Alva %s: %d columns, %d status cells",
+             model->name, model->columns, model->statusCells);
 
   brl->textColumns = model->columns;
   brl->textRows = 1;
@@ -547,13 +564,13 @@ updateConfiguration (BrailleDisplay *brl, int autodetecting, int textColumns, in
   }
 
   if (statusColumns != brl->statusColumns) {
-    LogPrint(LOG_INFO, "status cell count changed to %d", statusColumns);
+    logMessage(LOG_INFO, "status cell count changed to %d", statusColumns);
     brl->statusColumns = statusColumns;
     changed = 1;
   }
 
   if (textColumns != brl->textColumns) {
-    LogPrint(LOG_INFO, "text column count changed to %d", textColumns);
+    logMessage(LOG_INFO, "text column count changed to %d", textColumns);
     brl->textColumns = textColumns;
     if (!autodetecting) brl->resizeRequired = 1;
     changed = 1;
@@ -577,19 +594,19 @@ updateConfiguration (BrailleDisplay *brl, int autodetecting, int textColumns, in
 #define MAXIMUM_PACKET_SIZE PACKET_SIZE(0XFF)
 #define PACKET_BYTE(packet, index) ((packet)[PACKET_SIZE((index)) - 1])
 
-static const unsigned char BRL_ID[] = {0X1B, 'I', 'D', '='};
+static const unsigned char BRL_ID[] = {ESC, 'I', 'D', '='};
 #define BRL_ID_LENGTH (sizeof(BRL_ID))
 #define BRL_ID_SIZE (BRL_ID_LENGTH + 1)
 
 static int
 writeFunction1 (BrailleDisplay *brl, unsigned char code) {
-  unsigned char bytes[] = {0X1B, 'F', 'U', 'N', code, '\r'};
+  unsigned char bytes[] = {ESC, 'F', 'U', 'N', code, CR};
   return writeBytes(bytes, sizeof(bytes), &brl->writeDelay);
 }
 
 static int
 writeParameter1 (BrailleDisplay *brl, unsigned char parameter, unsigned char setting) {
-  unsigned char bytes[] = {0X1B, 'P', 'A', 3, 0, parameter, setting, '\r'};
+  unsigned char bytes[] = {ESC, 'P', 'A', 3, 0, parameter, setting, CR};
   return writeBytes(bytes, sizeof(bytes), &brl->writeDelay);
 }
 
@@ -604,6 +621,12 @@ updateConfiguration1 (BrailleDisplay *brl, int autodetecting, const unsigned cha
   return updateConfiguration(brl, autodetecting, textColumns, statusColumns, STATUS_FIRST);
 }
 
+static int
+setFirmness1 (BrailleDisplay *brl, BrailleFirmness setting) {
+  return writeParameter1(brl, 3,
+                         setting * 4 / BRL_FIRMNESS_MAXIMUM);
+}
+
 static int
 identifyModel1 (BrailleDisplay *brl, unsigned char identifier) {
   /* Find out which model we are connected to... */
@@ -616,7 +639,7 @@ identifyModel1 (BrailleDisplay *brl, unsigned char identifier) {
   if (model->name) {
     if (setDefaultConfiguration(brl)) {
       if (model->flags & MOD_FLAG_CONFIGURABLE) {
-        BRLSYMBOL.firmness = brl_firmness;
+        brl->setFirmness = setFirmness1;
 
         if (!writeFunction1(brl, 0X07)) return 0;
         while (io->awaitInput(200)) {
@@ -633,14 +656,12 @@ identifyModel1 (BrailleDisplay *brl, unsigned char identifier) {
         }
 
         if (!writeFunction1(brl, 0X0B)) return 0;
-      } else {
-        BRLSYMBOL.firmness = NULL; 
       }
 
       return 1;
     }
   } else {
-    LogPrint(LOG_ERR, "detected unknown Alva model with ID %02X (hex)", identifier);
+    logMessage(LOG_ERR, "detected unknown Alva model with ID %02X (hex)", identifier);
   }
 
   return 0;
@@ -784,7 +805,7 @@ readCommand1 (BrailleDisplay *brl) {
           continue;
         }
 
-        break;;
+        break;
 
       case 0X72: /* primary (lower) routing keys */
         if (key <= 0X5F) {			/* make */
@@ -831,8 +852,8 @@ readCommand1 (BrailleDisplay *brl) {
               if (frontKeys & progKey) {
                 unsigned char newSetting = frontKeys & ~progKey;
 
-                LogPrint(LOG_DEBUG, "Reconfiguring front keys: %02X -> %02X",
-                         frontKeys, newSetting);
+                logMessage(LOG_DEBUG, "Reconfiguring front keys: %02X -> %02X",
+                           frontKeys, newSetting);
                 writeParameter1(brl, 6, newSetting);
               }
             }
@@ -910,23 +931,17 @@ readCommand1 (BrailleDisplay *brl) {
 
 static int
 writeBraille1 (BrailleDisplay *brl, const unsigned char *cells, int start, int count) {
-  static const unsigned char header[] = {'\r', 0X1B, 'B'};	/* escape code to display braille */
-  static const unsigned char trailer[] = {'\r'};		/* to send after the braille sequence */
+  static const unsigned char header[] = {CR, ESC, 'B'};	/* escape code to display braille */
+  static const unsigned char trailer[] = {CR};		/* to send after the braille sequence */
 
   unsigned char packet[sizeof(header) + 2 + count + sizeof(trailer)];
   unsigned char *byte = packet;
 
-  memcpy(byte, header, sizeof(header));
-  byte += sizeof(header);
-
+  byte = mempcpy(byte, header, sizeof(header));
   *byte++ = start;
   *byte++ = count;
-
-  memcpy(byte, cells, count);
-  byte += count;
-
-  memcpy(byte, trailer, sizeof(trailer));
-  byte += sizeof(trailer);
+  byte = mempcpy(byte, cells, count);
+  byte = mempcpy(byte, trailer, sizeof(trailer));
 
   return writeBytes(packet, byte-packet, &brl->writeDelay);
 }
@@ -939,9 +954,18 @@ static const ProtocolOperations protocol1Operations = {
 
 static uint32_t firmwareVersion2;
 static unsigned char splitOffset2;
+static HidKeyboardPacket hidKeyboardPacket2;
 
 static void
 initializeVariables2 (void) {
+  initializeHidKeyboardPacket(&hidKeyboardPacket2);
+}
+
+static int
+interpretKeyboardEvent2 (BrailleDisplay *brl, const unsigned char *packet) {
+  const void *newPacket = packet;
+  processHidKeyboardPacket(&hidKeyboardPacket2, newPacket);
+  return EOF;
 }
 
 static int
@@ -985,6 +1009,12 @@ interpretKeyEvent2 (BrailleDisplay *brl, unsigned char group, unsigned char key)
       secondary = 1;
       goto doKey;
 
+    case 0X78: /* feature pack key */
+      base = AL_KEY_FEATUREPACK;
+      count = AL_KEYS_FEATUREPACK;
+      secondary = 0;
+      goto doKey;
+
     doKey:
       if (secondary) {
         if ((key / count) == 1) {
@@ -1021,7 +1051,7 @@ interpretKeyEvent2 (BrailleDisplay *brl, unsigned char group, unsigned char key)
       break;
   }
 
-  LogPrint(LOG_WARNING, "unknown key: group=%02X key=%02X", group, key);
+  logMessage(LOG_WARNING, "unknown key: group=%02X key=%02X", group, key);
   return EOF;
 }
 
@@ -1045,7 +1075,7 @@ readPacket2s (unsigned char *packet, int size) {
 
     if (offset == 0) {
       switch (byte) {
-        case 0X1B:
+        case ESC:
           length = 2;
           break;
 
@@ -1069,6 +1099,8 @@ readPacket2s (unsigned char *packet, int size) {
         case 0X4B: /* K */ length =  4; break;
         case 0X54: /* T */ length =  4; break;
         case 0X56: /* V */ length = 13; break;
+        case 0X68: /* h */ length = 10; break;
+        case 0X72: /* r */ length = 3; break;
       }
     }
 
@@ -1086,14 +1118,20 @@ readPacket2s (unsigned char *packet, int size) {
 }
 
 static int
-getAttributes2s (unsigned char item, unsigned char *packet, int size) {
-  unsigned char request[] = {0X1B, item, 0X3F};
+tellDevice2s (unsigned char command, unsigned char operand) {
+  unsigned char packet[] = {ESC, command, operand};
 
-  if (writeBytes(request, sizeof(request), NULL)) {
+  logOutputPacket(packet, sizeof(packet));
+  return writeBytes(packet, sizeof(packet), NULL);
+}
+
+static int
+askDevice2s (unsigned char command, unsigned char *response, int size) {
+  if (tellDevice2s(command, 0X3F)) {
     while (io->awaitInput(200)) {
-      int length = protocol->readPacket(packet, size);
+      int length = protocol->readPacket(response, size);
       if (length <= 0) break;
-      if ((packet[0] == 0X1B) && (packet[1] == item)) return 1;
+      if ((response[0] == ESC) && (response[1] == command)) return 1;
     }
   }
 
@@ -1102,14 +1140,14 @@ getAttributes2s (unsigned char item, unsigned char *packet, int size) {
 
 static int
 updateConfiguration2s (BrailleDisplay *brl, int autodetecting, const unsigned char *packet) {
-  unsigned char buffer[0X20];
+  unsigned char response[0X20];
 
-  if (getAttributes2s(0X45, buffer, sizeof(buffer))) {
-    unsigned char textColumns = buffer[2];
+  if (askDevice2s(0X45, response, sizeof(response))) {
+    unsigned char textColumns = response[2];
 
-    if (getAttributes2s(0X54, buffer, sizeof(buffer))) {
-      unsigned char statusColumns = buffer[2];
-      unsigned char statusSide = buffer[3];
+    if (askDevice2s(0X54, response, sizeof(response))) {
+      unsigned char statusColumns = response[2];
+      unsigned char statusSide = response[3];
 
       if (updateConfiguration(brl, autodetecting, textColumns, statusColumns,
                               (statusSide == 'R')? STATUS_RIGHT: STATUS_LEFT)) {
@@ -1129,21 +1167,20 @@ identifyModel2s (BrailleDisplay *brl, unsigned char identifier) {
     NULL
   };
 
-  unsigned char packet[0X20];
+  unsigned char response[0X20];
   const ModelEntry *const *modelEntry = models;
 
   while ((model = *modelEntry++)) {
     if (model->identifier == identifier) {
-      BRLSYMBOL.firmness = NULL;
-
       firmwareVersion2 = 0;
-      if (getAttributes2s(0X56, packet, sizeof(packet))) {
-        firmwareVersion2 |= (packet[4] << 16);
-        firmwareVersion2 |= (packet[5] <<  8);
-        firmwareVersion2 |= (packet[6] <<  0);
+      if (askDevice2s(0X56, response, sizeof(response))) {
+        firmwareVersion2 |= (response[4] << 16);
+        firmwareVersion2 |= (response[5] <<  8);
+        firmwareVersion2 |= (response[6] <<  0);
 
         if (setDefaultConfiguration(brl)) {
           if (updateConfiguration2s(brl, 1, NULL)) {
+            tellDevice2s(0X72, 1);
             return 1;
           }
         }
@@ -1153,7 +1190,7 @@ identifyModel2s (BrailleDisplay *brl, unsigned char identifier) {
     }
   }
 
-  LogPrint(LOG_ERR, "detected unknown Alva model with ID %02X (hex)", identifier);
+  logMessage(LOG_ERR, "detected unknown Alva model with ID %02X (hex)", identifier);
   return 0;
 }
 
@@ -1162,10 +1199,10 @@ detectModel2s (BrailleDisplay *brl) {
   int probes = 0;
 
   do {
-    unsigned char packet[0X20];
+    unsigned char response[0X20];
 
-    if (getAttributes2s(0X3F, packet, sizeof(packet))) {
-      if (identifyModel2s(brl, packet[2])) {
+    if (askDevice2s(0X3F, response, sizeof(response))) {
+      if (identifyModel2s(brl, response[2])) {
         return 1;
       }
     } else if (errno != EAGAIN) {
@@ -1186,7 +1223,7 @@ readCommand2s (BrailleDisplay *brl) {
     if (length < 0) return BRL_CMD_RESTARTBRL;
 
     switch (packet[0]) {
-      case 0X1B:
+      case ESC:
         switch (packet[1]) {
           case 0X4B: /* K */ {
             int command = interpretKeyEvent2(brl, packet[2], packet[3]);
@@ -1194,6 +1231,12 @@ readCommand2s (BrailleDisplay *brl) {
             continue;
           }
 
+          case 0X68: /* h */ {
+            int command = interpretKeyboardEvent2(brl, &packet[2]);
+            if (command != EOF) return command;
+            continue;
+          }
+
           default:
             break;
         }
@@ -1212,13 +1255,11 @@ writeBraille2s (BrailleDisplay *brl, const unsigned char *cells, int start, int
   unsigned char packet[4 + count];
   unsigned char *byte = packet;
 
-  *byte++ = 0X1B;
+  *byte++ = ESC;
   *byte++ = 0X42;
   *byte++ = start;
   *byte++ = count;
-
-  memcpy(byte, cells, count);
-  byte += count;
+  byte = mempcpy(byte, cells, count);
 
   return writeBytes(packet, byte-packet, &brl->writeDelay);
 }
@@ -1249,6 +1290,10 @@ readPacket2u (unsigned char *packet, int size) {
 
     if (offset == 0) {
       switch (byte) {
+        case 0X01:
+          length = 9;
+          break;
+
         case 0X04:
           length = 3;
           break;
@@ -1305,8 +1350,6 @@ updateConfiguration2u (BrailleDisplay *brl, int autodetecting, const unsigned ch
 
 static int
 detectModel2u (BrailleDisplay *brl) {
-  BRLSYMBOL.firmness = NULL;
-
   {
     unsigned char buffer[0X20];
     int length = io->getFeatureReport(0X09, buffer, sizeof(buffer));
@@ -1317,6 +1360,26 @@ detectModel2u (BrailleDisplay *brl) {
     if (length >= 8) firmwareVersion2 |= (buffer[7] <<  0);
   }
 
+  {
+    int updated = 0;
+    unsigned char buffer[0X20];
+    int length = io->getFeatureReport(0X06, buffer, sizeof(buffer));
+
+    if (length >= 2) {
+      unsigned char *old = &buffer[1];
+      unsigned char new = *old | 0X20;
+
+      if (new != *old) {
+        *old = new;
+        updated = 1;
+      }
+    }
+
+    if (updated) {
+      io->setFeatureReport(0X06, buffer, length);
+    }
+  }
+
   if (setDefaultConfiguration(brl))
     if (updateConfiguration2u(brl, 1, NULL))
       return 1;
@@ -1334,6 +1397,12 @@ readCommand2u (BrailleDisplay *brl) {
     if (length < 0) return BRL_CMD_RESTARTBRL;
 
     switch (packet[0]) {
+      case 0X01: {
+        int command = interpretKeyboardEvent2(brl, &packet[1]);
+        if (command != EOF) return command;
+        continue;
+      }
+
       case 0X04: {
         int command = interpretKeyEvent2(brl, packet[2], packet[1]);
         if (command != EOF) return command;
@@ -1358,9 +1427,7 @@ writeBraille2u (BrailleDisplay *brl, const unsigned char *cells, int start, int
     *byte++ = 0X02;
     *byte++ = start;
     *byte++ = length;
-
-    memcpy(byte, cells, length);
-    byte += length;
+    byte = mempcpy(byte, cells, length);
 
     if (!writeBytes(packet, byte-packet, &brl->writeDelay)) return 0;
     cells += length;
@@ -1428,10 +1495,16 @@ getSerialFeatureReport (unsigned char report, unsigned char *buffer, int length)
   return -1;
 }
 
+static int
+setSerialFeatureReport (unsigned char report, const unsigned char *data, int length) {
+  errno = ENOSYS;
+  return -1;
+}
+
 static const InputOutputOperations serialOperations = {
   openSerialPort, closeSerialPort,
   awaitSerialInput, readSerialBytes, writeSerialBytes,
-  getSerialFeatureReport
+  getSerialFeatureReport, setSerialFeatureReport
 };
 
 #include "io_usb.h"
@@ -1526,10 +1599,15 @@ getUsbFeatureReport (unsigned char report, unsigned char *buffer, int length) {
   return usbHidGetFeature(usbChannel->device, usbChannel->definition.interface, report, buffer, length, 1000);
 }
 
+static int
+setUsbFeatureReport (unsigned char report, const unsigned char *data, int length) {
+  return usbHidSetFeature(usbChannel->device, usbChannel->definition.interface, report, data, length, 1000);
+}
+
 static const InputOutputOperations usbOperations = {
   openUsbPort, closeUsbPort,
   awaitUsbInput, readUsbBytes, writeUsbBytes,
-  getUsbFeatureReport
+  getUsbFeatureReport, setUsbFeatureReport
 };
 
 #include "io_bluetooth.h"
@@ -1572,9 +1650,9 @@ writeBluetoothBytes (const unsigned char *buffer, int length, unsigned int *dela
   int count = bthWriteData(bluetoothConnection, buffer, length);
   if (count != length) {
     if (count == -1) {
-      LogError("Alva Bluetooth write");
+      logSystemError("Alva Bluetooth write");
     } else {
-      LogPrint(LOG_WARNING, "trunccated bluetooth write: %d < %d", count, length);
+      logMessage(LOG_WARNING, "trunccated bluetooth write: %d < %d", count, length);
     }
   }
   return count;
@@ -1586,10 +1664,16 @@ getBluetoothFeatureReport (unsigned char report, unsigned char *buffer, int leng
   return -1;
 }
 
+static int
+setBluetoothFeatureReport (unsigned char report, const unsigned char *data, int length) {
+  errno = ENOSYS;
+  return -1;
+}
+
 static const InputOutputOperations bluetoothOperations = {
   openBluetoothPort, closeBluetoothPort,
   awaitBluetoothInput, readBluetoothBytes, writeBluetoothBytes,
-  getBluetoothFeatureReport
+  getBluetoothFeatureReport, setBluetoothFeatureReport
 };
 
 int
@@ -1599,11 +1683,6 @@ AL_writeData( unsigned char *data, int len ) {
 
 static int
 brl_construct (BrailleDisplay *brl, char **parameters, const char *device) {
-  {
-    static const DotsTable dots = {0X01, 0X02, 0X04, 0X08, 0X10, 0X20, 0X40, 0X80};
-    makeOutputTable(dots, outputTable);
-  }
-
   if (isSerialDevice(&device)) {
     io = &serialOperations;
   } else if (isUsbDevice(&device)) {
@@ -1620,6 +1699,8 @@ brl_construct (BrailleDisplay *brl, char **parameters, const char *device) {
     protocol->initializeVariables();
 
     if (protocol->detectModel(brl)) {
+      makeOutputTable(dotsTable_ISO11548_1);
+
       memset(&textRewriteTime, 0, sizeof(textRewriteTime));
       return 1;
     }
@@ -1647,8 +1728,8 @@ brl_destruct (BrailleDisplay *brl) {
 
 static int
 brl_writeWindow (BrailleDisplay *brl, const wchar_t *text) {
-  int from = 0;
-  int to = brl->textColumns * brl->textRows;
+  unsigned int from;
+  unsigned int to;
 
   if (textRewriteInterval) {
     struct timeval now;
@@ -1658,39 +1739,35 @@ brl_writeWindow (BrailleDisplay *brl, const wchar_t *text) {
   }
 
   if (textRewriteRequired) {
+    from = 0;
+    to = brl->textColumns;
+    memcpy(previousText, brl->buffer, brl->textColumns);
+  } else if (!cellsHaveChanged(previousText, brl->buffer, brl->textColumns, &from, &to)) {
+    return 1;
+  }
+
+  {
+    size_t count = to - from;
+    unsigned char cells[count];
+
+    translateOutputCells(cells, &brl->buffer[from], count);
+    if (!protocol->writeBraille(brl, cells, textOffset+from, count)) return 0;
     textRewriteRequired = 0;
-  } else {
-    while ((from < to) && (brl->buffer[from] == previousText[from])) from++;
-    while ((--to > from) && (brl->buffer[to] == previousText[to]));
-    to++;
   }
 
-  if (from < to) {
-    unsigned char cells[to - from - 1];
-
-    {
-      int index;
-      for (index=from; index<to; index++)
-        cells[index - from] = outputTable[(previousText[index] = brl->buffer[index])];
-    }
-
-    if (!protocol->writeBraille(brl, cells, textOffset+from, to-from)) return 0;
-  }
   return 1;
 }
 
 static int
 brl_writeStatus (BrailleDisplay *brl, const unsigned char *status) {
-  if (statusRewriteRequired || (memcmp(status, previousStatus, brl->statusColumns) != 0)) {
-    unsigned char cells[brl->statusColumns];
+  size_t cellCount = brl->statusColumns;
 
-    {
-      int i;
-      for (i=0; i<brl->statusColumns; ++i)
-        cells[i] = outputTable[(previousStatus[i] = status[i])];
-    }
+  if (cellsHaveChanged(previousStatus, status, cellCount, NULL, NULL) || statusRewriteRequired) {
+    unsigned char cells[cellCount];
 
-    if (!protocol->writeBraille(brl, cells, statusOffset, brl->statusColumns)) return 0;
+    translateOutputCells(cells, status, cellCount);
+
+    if (!protocol->writeBraille(brl, cells, statusOffset, cellCount)) return 0;
     statusRewriteRequired = 0;
   }
 
@@ -1698,12 +1775,6 @@ brl_writeStatus (BrailleDisplay *brl, const unsigned char *status) {
 }
 
 static int
-brl_readCommand (BrailleDisplay *brl, BRL_DriverCommandContext context) {
+brl_readCommand (BrailleDisplay *brl, KeyTableCommandContext context) {
   return protocol->readCommand(brl);
 }
-
-static void
-brl_firmness (BrailleDisplay *brl, BrailleFirmness setting) {
-  writeParameter1(brl, 3,
-                 setting * 4 / BRL_FIRMNESS_MAXIMUM);
-}
