@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2009 by The BRLTTY Developers.
+ * Copyright (C) 1995-2011 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -30,7 +30,9 @@
 #include <stdio.h>
 #include <string.h>
 
-#include "misc.h"
+#include "log.h"
+#include "parse.h"
+#include "timing.h"
 #include "message.h"
 
 typedef enum {
@@ -52,8 +54,6 @@ typedef enum {
 
 SerialDevice *BL_serialDevice = NULL;		/* file descriptor for Braille display */
 
-static TranslationTable outputTable;	/* dot mapping table (output) */
-static TranslationTable inputTable;	/* mapping for reversed display */
 static unsigned char *prevdata = NULL;	/* previously received data */
 static unsigned char *rawdata = NULL;	/* writebrl() buffer for raw Braille data */
 static int blitesz;	/* set to 18 or 40 */
@@ -277,9 +277,9 @@ write_prebrl (void) {
 static int
 brl_construct (BrailleDisplay *brl, char **parameters, const char *device)
 {
-  static const int good_baudrates[] =
+  static const unsigned int good_baudrates[] =
     {300,600,1200,2400,4800,9600,19200,38400, 0};
-  int baudrate;
+  unsigned int baudrate;
   /* Init string for Model detection */
 
   if (!*parameters[PARM_BAUDRATE] ||
@@ -289,112 +289,121 @@ brl_construct (BrailleDisplay *brl, char **parameters, const char *device)
 
   if (*parameters[PARM_KBEMU])
     if (!validateYesNo(&kbemu, parameters[PARM_KBEMU]))
-      LogPrint(LOG_WARNING, "%s: %s", "invalid keyboard emulation setting", parameters[PARM_KBEMU]);
+      logMessage(LOG_WARNING, "%s: %s", "invalid keyboard emulation setting", parameters[PARM_KBEMU]);
   kbemu = !!kbemu;
 
-  {
-    static const DotsTable dots = {0X01, 0X02, 0X04, 0X08, 0X10, 0X20, 0X40, 0X80};
-    makeOutputTable(dots, outputTable);
-    reverseTranslationTable(outputTable, inputTable);
-  }
-
   if (!isSerialDevice(&device)) {
     unsupportedDevice(device);
     return 0;
   }
 
-  if ((qbase = malloc(QSZ))) {
-    /* Open the Braille display device for random access */
-    LogPrint(LOG_DEBUG, "Opening serial port: %s", device);
-    if ((BL_serialDevice = serialOpenDevice(device))) {
-      /* activate new settings */
-      if (serialRestartDevice(BL_serialDevice, baudrate)) {
-        qflush();
-        write_prebrl();
-        if (await_ack()) {
-          LogPrint(LOG_DEBUG, "Got response.");
+  logMessage(LOG_DEBUG, "Opening serial port: %s", device);
+  if ((BL_serialDevice = serialOpenDevice(device))) {
+    if (serialRestartDevice(BL_serialDevice, baudrate)) {
+      if (serialSetFlowControl(BL_serialDevice, SERIAL_FLOW_HARDWARE)) {
+        if ((qbase = malloc(QSZ))) {
+          qflush();
+          write_prebrl();
 
-          /* Next, let's detect the BLT-Model (18, 40, M20, M40). */
-          barcmds = &bar2cmds;
-          {
-            unsigned char cells[18];
-            memset(cells, 0, sizeof(cells));
-            serialWriteData(BL_serialDevice, cells, sizeof(cells));
-            waiting_ack = 1;
-            approximateDelay(400);
-            qfill();
-            if (waiting_ack) {
-              /* no response, so it must be BLT40 */
-              blitesz = 40;
-              brl->helpPage = 1;
-            } else {
-              blitesz = sizeof(cells);
-              brl->helpPage = 0;
-            }
-          }
+          if (await_ack()) {
+            logMessage(LOG_DEBUG, "Got response.");
 
-          {
-            static const unsigned char request[] = {0X05, 0X57};			/* code to send before Braille */
-            approximateDelay(200);
-            qflush();
-            serialWriteData(BL_serialDevice, request, sizeof(request));
-            waiting_ack = 0;
-            approximateDelay(200);
-            qfill();
-            if (qlen) {
-              char response[qlen + 1];
-              int length = 0;
-              do {
-                unsigned char byte = qbase[qoff % QSZ];
-                qoff = (qoff + 1) % QSZ, --qlen;
-                if (!byte) break;
-                response[length++] = byte;
-              } while (qlen);
-              response[length] = 0;
-              LogPrint(LOG_INFO, "Braille Lite identity: %s", response);
+            /* Next, let's detect the BLT-Model (18, 40, M20, M40). */
+            barcmds = &bar2cmds;
+            {
+              unsigned char cells[18];
 
-              if ((response[0] == 'X') &&
-                  (response[1] == ' ') &&
-                  (response[2] == 'B')) {
-                blitesz = atoi(&response[3]);
-                if (blitesz <= 20) barcmds = &bar1cmds;
+              memset(cells, 0, sizeof(cells));
+              serialWriteData(BL_serialDevice, cells, sizeof(cells));
+              waiting_ack = 1;
+              approximateDelay(400);
+              qfill();
+
+              if (waiting_ack) {
+                /* no response, so it must be BLT40 */
+                blitesz = 40;
+                brl->keyBindings = "40_m20_m40";
+              } else {
+                blitesz = sizeof(cells);
+                brl->keyBindings = "18";
               }
             }
-          }
 
-          LogPrint(LOG_NOTICE, "Braille Lite %d detected.", blitesz);
-          brl->textColumns = blitesz;	/* initialise size of display - */
-          brl->textRows = 1;		/* Braille Lites are single line displays */
+            {
+              static const unsigned char request[] = {0X05, 0X57};			/* code to send before Braille */
 
-          /* Allocate space for buffers */
-          if ((prevdata = malloc(brl->textColumns))) {
-            memset(prevdata, 0, brl->textColumns);
+              approximateDelay(200);
+              qflush();
+              serialWriteData(BL_serialDevice, request, sizeof(request));
+              waiting_ack = 0;
+              approximateDelay(200);
+              qfill();
 
-            if ((rawdata = malloc(brl->textColumns))) {
-              if (serialSetFlowControl(BL_serialDevice, SERIAL_FLOW_HARDWARE)) return 1;
+              if (qlen) {
+                char response[qlen + 1];
+                int length = 0;
 
-              free(rawdata);
-              rawdata = NULL;
+                do {
+                  unsigned char byte = qbase[qoff % QSZ];
+
+                  qoff = (qoff + 1) % QSZ, --qlen;
+                  if (!byte) break;
+                  response[length++] = byte;
+                } while (qlen);
+
+                response[length] = 0;
+                logMessage(LOG_INFO, "Braille Lite identity: %s", response);
+
+                if ((response[0] == 'X') &&
+                    (response[1] == ' ') &&
+                    (response[2] == 'B')) {
+                  blitesz = atoi(&response[3]);
+                  if (blitesz <= 20) barcmds = &bar1cmds;
+                }
+              }
+            }
+
+            logMessage(LOG_NOTICE, "Braille Lite %d detected.", blitesz);
+            brl->textColumns = blitesz;	/* initialise size of display - */
+            brl->textRows = 1;		/* Braille Lites are single line displays */
+
+            makeOutputTable(dotsTable_ISO11548_1);
+            makeInputTable();
+
+            /* Allocate space for buffers */
+            if ((prevdata = malloc(brl->textColumns))) {
+              memset(prevdata, 0, brl->textColumns);
+
+              if ((rawdata = malloc(brl->textColumns))) {
+                return 1;
+
+              //free(rawdata);
+              //rawdata = NULL;
+              } else {
+                logMallocError();
+              }
+
+              free(prevdata);
+              prevdata = NULL;
             } else {
-              LogPrint(LOG_ERR, "Cannot allocate rawdata.");
+              logMallocError();
             }
-            free(prevdata);
-            prevdata = NULL;
           } else {
-            LogPrint(LOG_ERR, "Cannot allocate prevdata.");
+            logMessage(LOG_DEBUG, "BrailleLite not responding.");
           }
+
+          free(qbase);
+          qbase = NULL;
         } else {
-          LogPrint(LOG_DEBUG, "BrailleLite not responding.");
+          logMallocError();
         }
       }
-      serialCloseDevice(BL_serialDevice);
-      BL_serialDevice = NULL;
     }
-    free(qbase);
-    qbase = NULL;
-  } else {
-    LogPrint(LOG_ERR, "Cannot allocate qbase.");
+
+    serialCloseDevice(BL_serialDevice);
+    BL_serialDevice = NULL;
   }
+
   return 0;
 }
 
@@ -447,21 +456,19 @@ brl_writeWindow (BrailleDisplay * brl, const wchar_t *text)
     }
 
   /* Next we must handle display reversal: */
-  if (reverse_kbd)
-    for (i = 0; i < blitesz; i++)
-      rawdata[i] = inputTable[brl->buffer[blitesz - 1 - i]];
-  else
-    memcpy (rawdata, brl->buffer, blitesz);
+  if (reverse_kbd) {
+    for (i=0; i<blitesz; i+=1) {
+      rawdata[i] = translateInputCell(brl->buffer[blitesz - 1 - i]);
+    }
+  } else {
+    memcpy(rawdata, brl->buffer, blitesz);
+  }
 
   /* Only refresh display if the data has changed: */
-  if (memcmp (rawdata, prevdata, blitesz))
+  if (cellsHaveChanged(prevdata, rawdata, blitesz, NULL, NULL))
     {
-      /* Save new Braille data: */
-      memcpy (prevdata, rawdata, blitesz);
-
       /* Dot mapping from standard to BrailleLite: */
-      for (i = 0; i < blitesz; i++)
-	rawdata[i] = outputTable[rawdata[i]];
+      translateOutputCells(rawdata, rawdata, blitesz);
 
       /* First we process any pending keystrokes, just in case any of them
        * are ^e ...
@@ -484,7 +491,7 @@ brl_writeWindow (BrailleDisplay * brl, const wchar_t *text)
 
 
 static int
-brl_readCommand (BrailleDisplay *brl, BRL_DriverCommandContext context)
+brl_readCommand (BrailleDisplay *brl, KeyTableCommandContext context)
 {
   static enum {
     ST_NORMAL,	/* transparent */
@@ -554,7 +561,7 @@ brl_readCommand (BrailleDisplay *brl, BRL_DriverCommandContext context)
 	    return key.cmd;
 
 	  /* kbemu could be on, then go on */
-	  if (kbemu && context == BRL_CTX_SCREEN)
+	  if (kbemu && context == KTB_CTX_DEFAULT)
 	    break;
 
 	  /* if it's a dangerous command it should have been chorded */
@@ -602,7 +609,7 @@ brl_readCommand (BrailleDisplay *brl, BRL_DriverCommandContext context)
 	    return BRL_CMD_NOOP;
 	  case ' ':		/* practical exception for */
 	    /* If keyboard mode off, space bar == BRL_CMD_HOME */
-	    if (!kbemu || context != BRL_CTX_SCREEN)
+	    if (!kbemu || context != KTB_CTX_DEFAULT)
 	      return BRL_CMD_HOME;
 	  }
 
