@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2010 by The BRLTTY Developers.
+ * Copyright (C) 1995-2011 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -57,7 +57,6 @@ extern t_eubrl_io eubrl_usbIos, eubrl_serialIos;
 extern t_eubrl_io eubrl_bluetoothIos, eubrl_ethernetIos;
 
 
-static TranslationTable outputTable;
 static t_eubrl_io *iop = NULL;
 static t_eubrl_protocol *protocolp = NULL;
 
@@ -67,22 +66,19 @@ brl_construct (BrailleDisplay *brl, char **parameters, const char *device)
   protocolp = NULL;
   brl->textColumns = 0;
   iop = NULL;
-  {
-    static const DotsTable dots = {0X01, 0X02, 0X04, 0X08, 0X10, 0X20, 0X40, 0X80};
-    makeOutputTable(dots, outputTable);
-  }
+  makeOutputTable(dotsTable_ISO11548_1);
   if (parameters[PARAM_PROTOCOLTYPE])
     {
       unsigned int choice = 0;
       char* choices[3];
-      LogPrint(LOG_DEBUG, "Detecting param %s", parameters[PARAM_PROTOCOLTYPE]);
+      logMessage(LOG_DEBUG, "Detecting param %s", parameters[PARAM_PROTOCOLTYPE]);
       choices[0] = "clio";
       choices[1] = "esysiris";
       choices[2] = NULL;
       if (!validateChoice(&choice, parameters[PARAM_PROTOCOLTYPE], (const char **)choices))
 	{
-	  LogPrint(LOG_ERR, "%s: unknown protocol type.", 
-		   parameters[PARAM_PROTOCOLTYPE]);
+	  logMessage(LOG_ERR, "%s: unknown protocol type.", 
+		     parameters[PARAM_PROTOCOLTYPE]);
 	  return 0;
 	}
       else if (choice == 0)
@@ -91,7 +87,7 @@ brl_construct (BrailleDisplay *brl, char **parameters, const char *device)
 	protocolp = &esysirisProtocol;
       if (protocolp == NULL)
 	{
-	  LogPrint(LOG_ERR, "eu: Undefined NULL protocol subsystem.");
+	  logMessage(LOG_ERR, "eu: Undefined NULL protocol subsystem.");
 	  return (0);
 	}
     }
@@ -139,28 +135,28 @@ brl_construct (BrailleDisplay *brl, char **parameters, const char *device)
     }
   if (iop->init(brl, parameters, device) == 0)
     {
-      LogPrint(LOG_DEBUG, "eu: Failed to initialize IO subsystem.");
+      logMessage(LOG_DEBUG, "eu: Failed to initialize IO subsystem.");
       return (0);
     }
 
   if (!protocolp) /* Autodetecting */
     { 
       protocolp = &esysirisProtocol;
-      LogPrint(LOG_INFO, "eu: Starting auto-detection process...");
+      logMessage(LOG_INFO, "eu: Starting auto-detection process...");
       if (protocolp->init(brl, iop) == 0)
 	{
-	  LogPrint(LOG_INFO, "eu: Esysiris detection failed.");
+	  logMessage(LOG_INFO, "eu: Esysiris detection failed.");
 	  iop->close(brl);
 	  approximateDelay(700);
 	  if (iop->init(brl, parameters, device) == 0)
 	    {
-	      LogPrint(LOG_ERR, "Failed to initialize IO for second autodetection.");
+	      logMessage(LOG_ERR, "Failed to initialize IO for second autodetection.");
 	      return (0);
 	    }
 	  protocolp = &clioProtocol;
 	  if (protocolp->init(brl, iop) == 0)
 	    {
-	      LogPrint(LOG_ERR, "eu: Autodetection failed.");
+	      logMessage(LOG_ERR, "eu: Autodetection failed.");
 	      iop->close(brl);
 	      return (0);
 	    }
@@ -169,17 +165,17 @@ brl_construct (BrailleDisplay *brl, char **parameters, const char *device)
   else
     {
       if (protocolp->protocolType == CLIO_PROTOCOL)
-	LogPrint(LOG_INFO, "Initializing clio protocol.");
+	logMessage(LOG_INFO, "Initializing clio protocol.");
       else
-	LogPrint(LOG_INFO, "Initializing EsysIris protocol.");
+	logMessage(LOG_INFO, "Initializing EsysIris protocol.");
       if (protocolp->init(brl, iop) == 0)
 	{
-	  LogPrint(LOG_ERR, "eu: Unable to connect to Braille display.");
+	  logMessage(LOG_ERR, "eu: Unable to connect to Braille display.");
 	  iop->close(brl);
 	  return (0);
 	}      
     }
-  LogPrint(LOG_INFO, "EuroBraille driver initialized: %d display length connected", brl->textColumns);
+  logMessage(LOG_INFO, "EuroBraille driver initialized: %d display length connected", brl->textColumns);
   return (1);
 }
 
@@ -233,7 +229,7 @@ brl_readKey (BrailleDisplay *brl)
 }
 
 static int
-brl_keyToCommand (BrailleDisplay *brl, BRL_DriverCommandContext context, int key) 
+brl_keyToCommand (BrailleDisplay *brl, KeyTableCommandContext context, int key) 
 {
   if (protocolp)
     return protocolp->keyToCommand(brl, key, context);
@@ -253,7 +249,7 @@ brl_writeWindow (BrailleDisplay *brl, const wchar_t *text)
 }
 
 static int
-brl_readCommand (BrailleDisplay *brl, BRL_DriverCommandContext context) 
+brl_readCommand (BrailleDisplay *brl, KeyTableCommandContext context) 
 {
   if (protocolp)
     return protocolp->readCommand(brl, context);
