@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2009 by The BRLTTY Developers.
+ * Copyright (C) 1995-2011 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -23,18 +23,17 @@
 
 #include "prologue.h"
 
-#include "misc.h"
+#include <stdio.h>
+#include <errno.h>
+
+#include "log.h"
+#include "timing.h"
 #include "message.h"
+#include "ascii.h"
 #include "eu_protocol.h"
 #include "eu_keys.h"
 
 /* Communication codes */
-# define SOH	0x01
-# define EOT	0x04
-# define ACK	0x06
-# define DLE	0x10
-# define NAK	0x15
-
 # define PRT_E_PAR 0x01		/* parity error */
 # define PRT_E_NUM 0x02		/* frame numver error */
 # define PRT_E_ING 0x03		/* length error */
@@ -42,10 +41,19 @@
 # define PRT_E_DON 0x05		/* data error */
 # define PRT_E_SYN 0x06		/* syntax error */
 
-/* Codes which need to be escaped */
-static const unsigned char needsEscape[0x100] = {
-  [SOH] = 1, [EOT] = 1, [DLE] = 1, [ACK] = 1, [NAK] = 1
-};
+static int
+needsEscape(unsigned char code) {
+  switch (code) {
+    case SOH:
+    case EOT:
+    case DLE:
+    case ACK:
+    case NAK:
+      return 1;
+  }
+
+  return 0;
+}
 
 # define	READ_BUFFER_LENGTH 1024
 
@@ -80,7 +88,7 @@ struct		s_clioModelType
 {
   enum clioModelType	type;
   char		modelCode[3];
-  char		modelDesc[30];
+  const char	*modelDesc;
 }	t_clioModelType;
 
 /** Static Local Variables */
@@ -91,7 +99,8 @@ static t_eubrl_io*	iop = NULL; /* I/O methods */
 static unsigned char	brlFirmwareVersion[21];
 static int		routingMode = BRL_BLK_ROUTE;
 static int refreshDisplay = 0;
-static struct s_clioModelType		clioModels[] =
+static int previousPacketNumber;
+static const struct s_clioModelType		clioModels[] =
   {
     {UNKNOWN, "", ""},
     {CE2, "CE2", "Clio-EuroBraille 20"},
@@ -337,7 +346,7 @@ int     clio_init(BrailleDisplay *brl, t_eubrl_io *io)
   brlCols = 0;
   if (!io)
     {
-      LogPrint(LOG_ERR, "eu: Clio : Invalid IO Subsystem driver.");
+      logMessage(LOG_ERR, "eu: Clio : Invalid IO Subsystem driver.");
       return (-1);
     }
   memset(brlFirmwareVersion, 0, 21);
@@ -346,14 +355,17 @@ int     clio_init(BrailleDisplay *brl, t_eubrl_io *io)
     {
       clio_reset(brl);      
       approximateDelay(500);
-      clio_readCommand(brl, BRL_CTX_SCREEN);
+      clio_readCommand(brl, KTB_CTX_DEFAULT);
     }
   if (brlCols > 0)
     { /* Succesfully identified hardware. */
       brl->textRows = 1;
       brl->textColumns = brlCols;
-      LogPrint(LOG_INFO, "eu: %s connected.",
-	       clioModels[brlModel].modelDesc);
+
+      previousPacketNumber = -1;
+
+      logMessage(LOG_INFO, "eu: %s connected.",
+	         clioModels[brlModel].modelDesc);
       return (1);
     }
   return (0);
@@ -363,10 +375,10 @@ int     clio_reset(BrailleDisplay *brl)
 {
   static const unsigned char packet[] = {0X02, 'S', 'I'};
 
-  LogPrint(LOG_INFO, "eu Clio hardware reset requested");
+  logMessage(LOG_INFO, "eu Clio hardware reset requested");
   if (clio_writePacket(brl, packet, sizeof(packet)) == -1)
     {
-      LogPrint(LOG_WARNING, "Clio: Failed to send ident request.");
+      logMessage(LOG_WARNING, "Clio: Failed to send ident request.");
       return -1;
     }
   return 1;
@@ -397,12 +409,12 @@ unsigned int	clio_readKey(BrailleDisplay *brl)
   return res;
 }
 
-int	clio_readCommand(BrailleDisplay *brl, BRL_DriverCommandContext ctx)
+int	clio_readCommand(BrailleDisplay *brl, KeyTableCommandContext ctx)
 {
   return clio_keyToCommand(brl, clio_readKey(brl), ctx);
 }
 
-int	clio_keyToCommand(BrailleDisplay *brl, unsigned int key, BRL_DriverCommandContext ctx)
+int	clio_keyToCommand(BrailleDisplay *brl, unsigned int key, KeyTableCommandContext ctx)
 {
   unsigned int res = EOF;
 
@@ -424,18 +436,17 @@ int	clio_keyToCommand(BrailleDisplay *brl, unsigned int key, BRL_DriverCommandCo
  
 void     clio_writeWindow(BrailleDisplay *brl)
 {
-  static char previousBrailleWindow[80];
+  static unsigned char previousBrailleWindow[80];
   int displaySize = brl->textColumns * brl->textRows;
   unsigned char buf[displaySize + 3];
 
   if ( displaySize > sizeof(previousBrailleWindow) ) {
-    LogPrint(LOG_WARNING, "[eu] Discarding too large braille window" );
+    logMessage(LOG_WARNING, "[eu] Discarding too large braille window" );
     return;
   }
-  if (!memcmp(previousBrailleWindow, brl->buffer, displaySize) && !refreshDisplay)
+  if (!cellsHaveChanged(previousBrailleWindow, brl->buffer, displaySize, NULL, NULL) && !refreshDisplay)
     return;
   refreshDisplay = 0;
-  memcpy(previousBrailleWindow, brl->buffer, displaySize);
   buf[0] = (unsigned char)(displaySize + 2);
   buf[1] = 'D';
   buf[2] = 'P';
@@ -451,7 +462,7 @@ void     clio_writeVisual(BrailleDisplay *brl, const wchar_t *text)
   int i;
 
   if ( displaySize > sizeof(previousVisualDisplay) ) {
-    LogPrint(LOG_WARNING, "[eu] Discarding too large visual display" );
+    logMessage(LOG_WARNING, "[eu] Discarding too large visual display" );
     return;
   }
 
@@ -476,105 +487,132 @@ int	clio_hasLcdSupport(BrailleDisplay *brl)
 
 ssize_t	clio_readPacket(BrailleDisplay *brl, void *packet, size_t size)
 {
-  static char 		buffer[READ_BUFFER_LENGTH];
-  static int		pos = 0;
-  static char		prevPktNbr = 0;
-  int		parity = 0;
-  int		ret, i, j, start, end, framelen = 0, otherchars;
-  char		*tmpres = NULL;
-  
-  if (!iop || !packet || size < 3)
-    return (-1);
-  ret = iop->read(brl, buffer + pos, READ_BUFFER_LENGTH - pos);
-  if (ret < 0)
-    return (-1);
-  for (i = 0, start = -1, end = -1, framelen = 0, otherchars = 0;
-       i < pos + ret && (start == -1 || end == -1); 
-       i++)
-    {
-      if (buffer[i] == SOH && start == -1) /* packet start detection */
-	start = i;
-      /* Packet end detection */
-      if (start != -1 && end == -1 && buffer[i] == EOT &&
-	  (buffer[i - 1] != DLE || 
-	   (buffer[i - 1] == DLE && buffer[i - 2] == DLE)))
-	end = i;
-      /* Frame length count */
-      if (start != -1 || end != -1)
-	framelen++;
-      if ((start == -1 && end == -1) || (start != -1 && end != -1))
-	otherchars++;
-    }
-  if (end != -1)
-    otherchars--;
-  pos += ret;
-  /* Skipping trailing chars if no packet has been read */
-  if (start == -1 && end == -1)
-    {
-      pos -= otherchars;
-      return 0;
-    }
-  /* If we found beginning of the packet but not the end */
-  if (end == -1)
-    return 0;
+  unsigned char buffer[size + 4];
+  int offset = 0;
+  int escape = 0;
 
-  /* ignoring packets received twice **/
-  if ((needsEscape[((unsigned char)buffer[end - 1])] != 1
-       && buffer[end - 2] == prevPktNbr)
-      || (needsEscape[((unsigned char)buffer[end - 1])] == 1 
-	  && buffer[end - 3] == prevPktNbr))
+  while (1)
     {
-      memmove(buffer, buffer + end + 1, pos - framelen);
-      pos -= framelen + otherchars;
-      return 0;
-    }
-  /* Updating pprevPktNbr */
-  if (needsEscape[((unsigned char)buffer[end - 1])] != 1)
-    prevPktNbr = buffer[end - 2];
-  else
-    prevPktNbr = buffer[end - 3];
+      int started = offset > 0;
+      int escaped = 0;
+      unsigned char byte;
+      ssize_t result = iop->read(brl, &byte, 1, (started || escape));
 
-  if ((tmpres = malloc(size + 1)) == NULL)
-    {
-      LogPrint(LOG_ERR, "clio: Failed to allocate memory.");
-      return (-1);
-    }
-  if (start != -1 && end != -1
-      && size >= framelen - 2)
-    {
-      /* parity calculation and preparing resultin buffer */
-      for (parity = 0, i = start + 1, j = 0; 
-	   i < end - 1 && j < size; 
-	   i++)
-	{
-	  if (buffer[i] != DLE || buffer[i - 1] == DLE)
-	    {
-	      tmpres[j] = buffer[i];
-	      j++;
-	      parity ^= buffer[i];
-	    }
-	}
-      /* Parity check */
-      if (parity != buffer[end - 1])
-	{
-	  sendbyte(brl, NAK);
-	  sendbyte(brl, PRT_E_PAR);
-	  prevPktNbr = 0;
-	  pos = 0;
-	  free(tmpres);
-	  return (0);
-	}
+      if (!result)
+        {
+          errno = EAGAIN;
+          result = -1;
+        }
+
+      if (result == -1)
+        {
+          if (started) logPartialPacket(buffer, offset);
+          return (errno == EAGAIN)? 0: -1;
+        }
+
+      if (escape)
+        {
+          escape = 0;
+          escaped = 1;
+        }
+      else if (byte == DLE)
+        {
+          escape = 1;
+          continue;
+        }
+
+      if (!escaped)
+        {
+          switch (byte)
+            {
+            case SOH:
+              if (started)
+                {
+                  logShortPacket(buffer, offset);
+                  offset = 1;
+                  continue;
+                }
+              goto addByte;
+
+            case EOT:
+              break;
+
+            default:
+              if (needsEscape(byte))
+                {
+                  if (started) logShortPacket(buffer, offset);
+                  offset = 0;
+                  continue;
+                }
+              break;
+            }
+        }
+
+      if (!started)
+        {
+          logIgnoredByte(byte);
+          continue;
+        }
+
+    addByte:
+      if (offset < sizeof(buffer))
+        {
+          buffer[offset] = byte;
+        }
       else
-	{
-	  memcpy(packet, tmpres, j - 1);
-	  memmove(buffer, buffer + end + 1, pos - framelen);
-	  pos -= framelen + otherchars;
-	  sendbyte(brl, ACK);
-	  free(tmpres);
-	  return (1); /* success */
-	}
+        {
+          if (offset == sizeof(buffer)) logTruncatedPacket(buffer, offset);
+          logDiscardedByte(byte);
+        }
+      offset += 1;
+
+      if (!escaped && (byte == EOT))
+        {
+          if (offset > sizeof(buffer))
+            {
+              offset = 0;
+              continue;
+            }
+
+          logInputPacket(buffer, offset);
+          offset -= 1; /* remove EOT */
+
+          {
+            unsigned char parity = 0;
+
+            {
+              int i;
+
+              for (i=1; i<offset; i+=1)
+                {
+                  parity ^= buffer[i];
+                }
+            }
+
+            if (parity)
+              {
+                sendbyte(brl, NAK);
+                sendbyte(brl, PRT_E_PAR);
+
+                offset = 0;
+                continue;
+              }
+          }
+
+          offset -= 1; /* remove parity */
+          sendbyte(brl, ACK);
+
+          if (buffer[--offset] == previousPacketNumber)
+            {
+              offset = 0;
+              continue;
+            }
+          previousPacketNumber = buffer[offset];
+
+          memcpy(packet, &buffer[1], offset-1);
+          return 1;
+        }
     }
-  return (0);
 }
 
 ssize_t	clio_writePacket(BrailleDisplay *brl, const void *packet, size_t size)
@@ -590,7 +628,7 @@ ssize_t	clio_writePacket(BrailleDisplay *brl, const void *packet, size_t size)
   *q++ = SOH;
   while (size--)
     {
-	if (needsEscape[*p]) *q++ = DLE;
+	if (needsEscape(*p)) *q++ = DLE;
         *q++ = *p;
 	parity ^= *p++;
      }
@@ -598,10 +636,11 @@ ssize_t	clio_writePacket(BrailleDisplay *brl, const void *packet, size_t size)
    parity ^= pktNbr;
    if (++pktNbr >= 256)
      pktNbr = 128;
-   if (needsEscape[parity]) *q++ = DLE;
+   if (needsEscape(parity)) *q++ = DLE;
    *q++ = parity;
    *q++ = EOT;
    packetSize = q - buf;
    updateWriteDelay(brl, packetSize);
+   logOutputPacket(buf, packetSize);
    return iop->write(brl, buf, packetSize);
 }
