@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2009 by The BRLTTY Developers.
+ * Copyright (C) 1995-2011 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -35,7 +35,8 @@
 #include	<sys/ioctl.h>
 
 #include	"message.h"
-#include	"misc.h"
+#include	"log.h"
+#include	"ascii.h"
 
 
 #define		BRL_HAVE_PACKET_IO	1
@@ -168,10 +169,6 @@ static t_alias	brl_key[] = {
 #define BRLROWS		1
 #define MAX_STCELLS	5	/* hiest number of status cells */
 
-/* This is the brltty braille mapping standard to Eurobraille's mapping table.
- */
-static TranslationTable outputTable;
-
 /* Global variables */
 
 static int		chars_per_sec;
@@ -190,14 +187,6 @@ static int	control = 0;
 static int	alt = 0;
 
 
-/* Communication codes */
-
-#define SOH     0x01
-#define EOT     0x04
-#define ACK     0x06
-#define DLE     0x10
-#define NACK    0x15
-
 #define DIM_INBUFSZ 256
 
 static int readbrlkey(BrailleDisplay *brl, char key_context);
@@ -235,7 +224,7 @@ static int WriteToBrlDisplay (BrailleDisplay *brl, int len, const char *data)
 	   case EOT:
 	   case ACK:
 	   case DLE:
-	   case NACK:
+	   case NAK:
 	     *p++ = DLE;
 	  /* no break */
 	   default:
@@ -276,12 +265,9 @@ static int brl_reset(BrailleDisplay *brl)
 
 static int brl_construct (BrailleDisplay *brl, char **parameters, const char *device)
 {
-  static const DotsTable dots = {0X01, 0X02, 0X04, 0X08, 0X10, 0X20, 0X40, 0X80};
-  makeOutputTable(dots, outputTable);
-
   if ((gio_fd = open("/dev/iris", O_RDWR)) == -1)
     {
-      LogPrint(LOG_INFO, "Cannot open Iris-GIO device.");
+      logMessage(LOG_INFO, "Cannot open Iris-GIO device.");
       return 0;
     }
   
@@ -290,7 +276,7 @@ static int brl_construct (BrailleDisplay *brl, char **parameters, const char *de
   */
   if (ioctl(gio_fd, IOCTL_SETBIT_FIRSTBYTE, IO_D0) == -1)
     {
-      LogPrint(LOG_INFO, "Cannot send ioctl to device.");
+      logMessage(LOG_INFO, "Cannot send ioctl to device.");
       return 0;
     }
 
@@ -300,7 +286,7 @@ static int brl_construct (BrailleDisplay *brl, char **parameters, const char *de
   usleep(8500);
   if (ioctl(gio_fd, IOCTL_SETBIT_FIRSTBYTE, 0x00) == -1)
     {
-      LogPrint(LOG_INFO, "Cannot clear device bits.");
+      logMessage(LOG_INFO, "Cannot clear device bits.");
       return 0;
     }
   /*
@@ -325,7 +311,6 @@ static int brl_construct (BrailleDisplay *brl, char **parameters, const char *de
    serialRestartDevice(serialDevice, BAUDRATE);
 
   /* Set model params... */
-   brl->helpPage = 0;
    brl->textRows = BRLROWS;
    while (!NbCols)
      {
@@ -335,11 +320,14 @@ static int brl_construct (BrailleDisplay *brl, char **parameters, const char *de
 	while (!NbCols)
 	  {
 	     drainBrailleOutput (brl, 100);
-	     brl_readCommand (brl, BRL_CTX_SCREEN);       /* to get the answer */
+	     brl_readCommand (brl, KTB_CTX_DEFAULT);       /* to get the answer */
 	     if (++i >= 10)
 	       break;
 	  }
      }
+
+   makeOutputTable(dotsTable_ISO11548_1);
+
    ReWrite = 1;  /* To write whole display at first time */
    ReWrite_LCD = 1;
    return 1;
@@ -374,10 +362,10 @@ static void brl_destruct (BrailleDisplay *brl)
    if (gio_fd == -1)
      return;
    if (ioctl(gio_fd, IOCTL_SETBIT_FIRSTBYTE, IO_D1) == -1)
-     LogPrint(LOG_INFO, "Cannot turn off braille power.");
+     logMessage(LOG_INFO, "Cannot turn off braille power.");
    usleep(8500);
    if (ioctl(gio_fd, IOCTL_SETBIT_FIRSTBYTE, 0x00) == -1)
-     LogPrint(LOG_INFO, "Cannot send ioctl().");
+     logMessage(LOG_INFO, "Cannot send ioctl().");
    close(gio_fd);
    gio_fd = -1;
 }
@@ -423,10 +411,9 @@ static int brl_writeWindow (BrailleDisplay *brl, const wchar_t *text)
      }
    if (!ReWrite)
      /* We update the display only if it has changed */
-     if (memcmp(brl->buffer, prevdata, NbCols) != 0)
+     if (cellsHaveChanged(prevdata, brl->buffer, NbCols, NULL, NULL))
        {
 	 ReWrite = 1;
-	 memcpy(prevdata, brl->buffer, NbCols);
        }
    if (ReWrite)
      {
@@ -439,7 +426,7 @@ static int brl_writeWindow (BrailleDisplay *brl, const wchar_t *text)
 	 for (j = 0; j < 8; j++)
 	   *p++ = 0;
        for (j = NbCols - 1; j >= 0; j--)
-	 *p++ = outputTable[brl->buffer[j]];
+	 *p++ = translateOutputCell(brl->buffer[j]);
        WriteToBrlDisplay (brl, p - OutBuf, OutBuf);
        ReWrite = 0;
      }
@@ -494,7 +481,7 @@ static int brl_readKey(BrailleDisplay *brl)
   return (int)readbrlkey(brl, CTX_KEYCODES);
 }
 
-static int brl_keyToCommand(BrailleDisplay *brl, BRL_DriverCommandContext key_context, int code)
+static int brl_keyToCommand(BrailleDisplay *brl, KeyTableCommandContext key_context, int code)
 {
   if (code & 0x00010000)
     {
@@ -519,7 +506,7 @@ static int brl_keyToCommand(BrailleDisplay *brl, BRL_DriverCommandContext key_co
   return (EOF);
 }
 
-static int brl_readCommand(BrailleDisplay *brl, BRL_DriverCommandContext key_context)
+static int brl_readCommand(BrailleDisplay *brl, KeyTableCommandContext key_context)
 {
   unsigned int	key;
 
@@ -820,7 +807,7 @@ static int readbrlkey(BrailleDisplay *brl, char key_context)
       else
 	switch (c)
 	  {
-	  case NACK:
+	  case NAK:
 	    ErrFlag = 1;
 	    /* no break */
 	  case ACK:
@@ -862,7 +849,7 @@ static int readbrlkey(BrailleDisplay *brl, char key_context)
 	case 'V':
 	  /* it's an identification string */
 	  memcpy(IdentString, buf + p + 1, 40);
-	  LogPrint(LOG_INFO, "BIOS detected: %s", IdentString);
+	  logMessage(LOG_INFO, "BIOS detected: %s", IdentString);
 	  if (buf[p + 1] == 's' || buf[p + 1] == 'S')
 	    NbCols = 32;
 	  else
