@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2009 by The BRLTTY Developers.
+ * Copyright (C) 1995-2011 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -33,9 +33,6 @@
  * Edit as necessary for your system.
  */
 
-#define cSTX 0X02
-#define cETX 0X03
-
 #define BRLCOLSMAX   80 
 
 /* maximum number of status cells */
