@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2010 by The BRLTTY Developers.
+ * Copyright (C) 1995-2011 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -90,9 +90,8 @@ typedef struct {
 static const InputOutputOperations *io;
 
 #define SERIAL_BAUD 9600
-static const int charactersPerSecond = SERIAL_BAUD / 10;
+static const unsigned int charactersPerSecond = SERIAL_BAUD / 10;
 
-static TranslationTable outputTable;
 static unsigned char textCells[40];
 
 static int
@@ -307,9 +306,7 @@ writeCells_330 (BrailleDisplay *brl) {
   unsigned char packet[sizeof(header) + 2 + (brl->textColumns * 2)];
   unsigned char *byte = packet;
 
-  memcpy(byte, header, sizeof(header));
-  byte += sizeof(header);
-
+  byte = mempcpy(byte, header, sizeof(header));
   *byte++ = brl->textColumns * 2;
   *byte++ = 0;
 
@@ -317,7 +314,7 @@ writeCells_330 (BrailleDisplay *brl) {
     int i;
     for (i=0; i<brl->textColumns; i+=1) {
       *byte++ = 0;
-      *byte++ = outputTable[textCells[i]];
+      *byte++ = translateOutputCell(textCells[i]);
     }
   }
 
@@ -368,14 +365,13 @@ writeCells_331 (BrailleDisplay *brl) {
   unsigned char packet[sizeof(header) + (brl->textColumns * 2)];
   unsigned char *byte = packet;
 
-  memcpy(byte, header, sizeof(header));
-  byte += sizeof(header);
+  byte = mempcpy(byte, header, sizeof(header));
 
   {
     int i;
     for (i=0; i<brl->textColumns; i+=1) {
       *byte++ = 0;
-      *byte++ = outputTable[textCells[i]];
+      *byte++ = translateOutputCell(textCells[i]);
     }
   }
 
@@ -501,15 +497,9 @@ writeCells_332 (BrailleDisplay *brl) {
   unsigned char packet[sizeof(header) + 1 + brl->textColumns];
   unsigned char *byte = packet;
 
-  memcpy(byte, header, sizeof(header));
-  byte += sizeof(header);
-
+  byte = mempcpy(byte, header, sizeof(header));
   *byte++ = brl->textColumns;
-
-  {
-    int i;
-    for (i=0; i<brl->textColumns; i+=1) *byte++ = outputTable[textCells[i]];
-  }
+  byte = translateOutputCells(byte, textCells, brl->textColumns);
 
   return writeBytes(brl, packet, byte-packet);
 }
@@ -591,9 +581,9 @@ static int
 openUsbPort (const char *device) {
   static const SerialParameters serial = {
     .baud = SERIAL_BAUD,
-    .flow = SERIAL_FLOW_NONE,
-    .data = 8,
-    .stop = 1,
+    .flowControl = SERIAL_FLOW_NONE,
+    .dataBits = 8,
+    .stopBits = 1,
     .parity = SERIAL_PARITY_NONE
   };
 
@@ -690,9 +680,9 @@ writeBluetoothBytes (const unsigned char *buffer, int length) {
   int count = bthWriteData(bluetoothConnection, buffer, length);
   if (count != length) {
     if (count == -1) {
-      LogError("Bluetooth write");
+      logSystemError("Bluetooth write");
     } else {
-      LogPrint(LOG_WARNING, "Trunccated bluetooth write: %d < %d", count, length);
+      logMessage(LOG_WARNING, "Trunccated bluetooth write: %d < %d", count, length);
     }
   }
   return count;
@@ -706,11 +696,6 @@ static const InputOutputOperations bluetoothOperations = {
 
 static int
 brl_construct (BrailleDisplay *brl, char **parameters, const char *device) {
-  {
-    static const DotsTable dots = {0X01, 0X02, 0X04, 0X08, 0X10, 0X20, 0X40, 0X80};
-    makeOutputTable(dots, outputTable);
-  }
-  
   if (isSerialDevice(&device)) {
     io = &serialOperations;
   } else if (isUsbDevice(&device)) {
@@ -728,18 +713,19 @@ brl_construct (BrailleDisplay *brl, char **parameters, const char *device) {
     while ((protocol = *protocolAddress++)) {
       InputPacket response;
 
-      LogPrint(LOG_DEBUG, "trying protocol %s", protocol->name);
+      logMessage(LOG_DEBUG, "trying protocol %s", protocol->name);
       if (protocol->probeDisplay(brl, &response)) {
-        LogPrint(LOG_DEBUG, "Seika Protocol: %s", protocol->name);
-        LogPrint(LOG_DEBUG, "Seika Model: %u", response.fields.identity.model);
-        LogPrint(LOG_DEBUG, "Seika Size: %u", response.fields.identity.size);
+        logMessage(LOG_DEBUG, "Seika Protocol: %s", protocol->name);
+        logMessage(LOG_DEBUG, "Seika Model: %u", response.fields.identity.model);
+        logMessage(LOG_DEBUG, "Seika Size: %u", response.fields.identity.size);
 
         brl->textColumns = response.fields.identity.size;
         brl->textRows = 1;
 
+        makeOutputTable(dotsTable_ISO11548_1);
+  
         {
           const KeyTableDefinition *ktd = &KEY_TABLE_DEFINITION(all);
-
           brl->keyBindings = ktd->bindings;
           brl->keyNameTables = ktd->names;
         }
@@ -762,8 +748,7 @@ brl_destruct (BrailleDisplay *brl) {
 
 static int
 brl_writeWindow (BrailleDisplay *brl, const wchar_t *text) {
-  if (memcmp(textCells, brl->buffer, brl->textColumns) != 0) {
-    memcpy(textCells, brl->buffer, brl->textColumns);
+  if (cellsHaveChanged(textCells, brl->buffer, brl->textColumns, NULL, NULL)) {
     if (!protocol->writeCells(brl)) return 0;
   }
 
@@ -771,7 +756,7 @@ brl_writeWindow (BrailleDisplay *brl, const wchar_t *text) {
 }
 
 static int
-brl_readCommand (BrailleDisplay *brl, BRL_DriverCommandContext context) {
+brl_readCommand (BrailleDisplay *brl, KeyTableCommandContext context) {
   InputPacket packet;
   size_t length;
 
