@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2009 by The BRLTTY Developers.
+ * Copyright (C) 1995-2011 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -82,13 +82,20 @@
 #include <stdio.h>
 #include <string.h>
 
-#include "misc.h"
+#include "log.h"
+#include "timing.h"
 #include "message.h"
 
 #include "brl_driver.h"
 #include "braille.h"
 #include "io_serial.h"
 
+#ifdef __MSDOS__
+#ifdef HIGHBAUD
+#undef HIGHBAUD
+#endif /* HIGHBAUD */
+#endif /* __MSDOS__ */
+
 /* Braille display parameters that do not change */
 #define BRLROWS 1		/* only one row on braille display */
 
@@ -133,9 +140,6 @@ static int repeat_list[] =
  BRL_BLK_PASSKEY+BRL_KEY_CURSOR_DOWN,
  BRL_CMD_CSRTRK, 0};
 
-/* This defines the mapping between brltty and Navigator's dots coding. */
-static TranslationTable outputTable;
-
 /* Stabilization delay after changing baud rate */
 #define BAUD_DELAY (100)
 
@@ -284,8 +288,8 @@ static struct inbytedesc pb_key_desc[PB_KEY_LEN] =
 /* Global variables */
 
 static SerialDevice *serialDevice;              /* file descriptor for comm port */
-static int serialBaud;
-static int charactersPerSecond;
+static unsigned int serialBaud;
+static unsigned int charactersPerSecond;
 static unsigned char *rawdata,	/* translated data to send to display */
                      *prevdata, /* previous data sent */
                      *dispbuf;
@@ -298,7 +302,9 @@ static unsigned char has_sw,    /* flag: has routing keys or not */
                      sw_bcnt;   /* bytes of sensor switch information */
 static char disp_ver[Q_VER_LENGTH]; /* version of the hardware */
 
-static enum { NAV20_40=0, NAV80, PB40, PB65_80} displayType;
+typedef enum {NAV20_40, NAV80, PB40, PB65_81} DisplayType;
+static DisplayType displayType;
+static const char *keyBindings[] = {"nav20_nav40", "nav80", "pb40", "pb65_pb81"};
 
 static int
 myread(void *buf, unsigned len)
@@ -316,19 +322,19 @@ QueryDisplay(unsigned char *reply)
     if (serialAwaitInput(serialDevice, 100)) {
       if ((count = myread(reply, Q_REPLY_LENGTH)) != -1) {
         if ((count == Q_REPLY_LENGTH) && (memcmp(reply, Q_HEADER, Q_HEADER_LENGTH) == 0)) {
-          LogBytes(LOG_DEBUG, "TSI Reply", reply, count);
+          logBytes(LOG_DEBUG, "TSI Reply", reply, count);
           return 1;
         } else {
-          LogBytes(LOG_WARNING, "Unexpected Response", reply, count);
+          logUnexpectedPacket(reply, count);
         }
       } else {
-        LogError("Read");
+        logSystemError("Read");
       }
     }
   } else if (count != -1) {
-    LogPrint(LOG_ERR, "Short write: %d < %d", count, DIM_BRL_QUERY);
+    logMessage(LOG_ERR, "Short write: %d < %d", count, DIM_BRL_QUERY);
   } else {
-    LogError("Write");
+    logSystemError("Write");
   }
   return 0;
 }
@@ -352,11 +358,6 @@ brl_construct (BrailleDisplay *brl, char **parameters, const char *device)
   unsigned char reply[Q_REPLY_LENGTH];
   int speed;
 
-  {
-    static const DotsTable dots = {0X01, 0X02, 0X04, 0X08, 0X10, 0X20, 0X40, 0X80};
-    makeOutputTable(dots, outputTable);
-  }
-
   if (!isSerialDevice(&device)) {
     unsupportedDevice(device);
     return 0;
@@ -367,13 +368,13 @@ brl_construct (BrailleDisplay *brl, char **parameters, const char *device)
   /* Open the Braille display device for random access */
   if (!(serialDevice = serialOpenDevice(device))) goto failure;
   /* Try to detect display by sending query */
-  LogPrint(LOG_DEBUG,"Sending query at 9600bps");
+  logMessage(LOG_DEBUG,"Sending query at 9600bps");
   if(!serialRestartDevice(serialDevice, serialBaud=9600)) goto failure;
   if(!QueryDisplay(reply)){
 #ifdef HIGHBAUD
     /* Then send the query at 19200bps, in case a PB was left ON
        at that speed */
-    LogPrint(LOG_DEBUG,"Sending query at 19200bps");
+    logMessage(LOG_DEBUG,"Sending query at 19200bps");
     if(!serialSetBaud(serialDevice, serialBaud=19200)) goto failure;
     if(!QueryDisplay(reply)) goto failure;
 #endif /* HIGHBAUD */
@@ -381,8 +382,8 @@ brl_construct (BrailleDisplay *brl, char **parameters, const char *device)
 
   memcpy (disp_ver, &reply[Q_OFFSET_VER], Q_VER_LENGTH);
   ncells = reply[Q_OFFSET_COLS];
-  LogPrint(LOG_INFO,"Display replied: %d cells, version %c%c%c%c", ncells,
-	   disp_ver[0], disp_ver[1], disp_ver[2], disp_ver[3]);
+  logMessage(LOG_INFO,"Display replied: %d cells, version %c%c%c%c", ncells,
+	     disp_ver[0], disp_ver[1], disp_ver[2], disp_ver[3]);
 
   brl_cols = ncells;
   sw_lastkey = brl_cols-1;
@@ -394,7 +395,7 @@ brl_construct (BrailleDisplay *brl, char **parameters, const char *device)
     /* nav 20 */
     displayType = NAV20_40;
     has_sw = 0;
-    LogPrint(LOG_INFO, "Detected Navigator 20");
+    logMessage(LOG_INFO, "Detected Navigator 20");
     break;
   case 40:
     if(disp_ver[1] > '3'){
@@ -404,13 +405,13 @@ brl_construct (BrailleDisplay *brl, char **parameters, const char *device)
       sw_bcnt = SW_CNT40;
       sw_lastkey = 39;
       speed = 2;
-      LogPrint(LOG_INFO, "Detected PowerBraille 40");
+      logMessage(LOG_INFO, "Detected PowerBraille 40");
     }else{
       /* nav 40 */
       has_sw = 0;
       displayType = NAV20_40;
       slow_update = 1;
-      LogPrint(LOG_INFO, "Detected Navigator 40");
+      logMessage(LOG_INFO, "Detected Navigator 40");
     }
     break;
   case 80:
@@ -420,33 +421,33 @@ brl_construct (BrailleDisplay *brl, char **parameters, const char *device)
     sw_bcnt = SW_CNT80;
     sw_lastkey = 79;
     slow_update = 2;
-    LogPrint(LOG_INFO, "Detected Navigator 80");
+    logMessage(LOG_INFO, "Detected Navigator 80");
     break;
   case 65:
     /* pb65 */
-    displayType = PB65_80;
+    displayType = PB65_81;
     has_sw = 1;
     sw_bcnt = SW_CNT81;
     sw_lastkey = 64;
     speed = 2;
     slow_update = 2;
-    LogPrint(LOG_INFO, "Detected PowerBraille 65");
+    logMessage(LOG_INFO, "Detected PowerBraille 65");
     break;
   case 81:
     /* pb80 */
-    displayType = PB65_80;
+    displayType = PB65_81;
     has_sw = 1;
     sw_bcnt = SW_CNT81;
     sw_lastkey = 80;
     speed = 2;
     slow_update = 2;
-    LogPrint(LOG_INFO, "Detected PowerBraille 80");
+    logMessage(LOG_INFO, "Detected PowerBraille 80");
     break;
   default:
-    LogPrint(LOG_ERR,"Unrecognized braille display");
+    logMessage(LOG_ERR,"Unrecognized braille display");
     goto failure;
   };
-  brl->helpPage = displayType;
+  brl->keyBindings = keyBindings[displayType];
 
   no_multiple_updates = 0;
 #ifdef FORCE_DRAIN_AFTER_SEND
@@ -467,21 +468,21 @@ brl_construct (BrailleDisplay *brl, char **parameters, const char *device)
     serialDrainOutput(serialDevice);
     approximateDelay(BAUD_DELAY);
     if(!serialSetBaud(serialDevice, serialBaud=19200)) goto failure;
-    LogPrint(LOG_DEBUG,"Switched to 19200bps. Checking if display followed.");
+    logMessage(LOG_DEBUG,"Switched to 19200bps. Checking if display followed.");
     if(QueryDisplay(reply))
-      LogPrint(LOG_DEBUG,"Display responded at 19200bps.");
+      logMessage(LOG_DEBUG,"Display responded at 19200bps.");
     else{
-      LogPrint(LOG_INFO,"Display did not respond at 19200bps, "
-	       "falling back to 9600bps.");
+      logMessage(LOG_INFO,"Display did not respond at 19200bps, "
+	         "falling back to 9600bps.");
       if(!serialSetBaud(serialDevice, serialBaud=9600)) goto failure;
       serialDrainOutput(serialDevice);
       approximateDelay(BAUD_DELAY); /* just to be safe */
       if(QueryDisplay(reply)) {
-	LogPrint(LOG_INFO,"Found display again at 9600bps.");
-	LogPrint(LOG_INFO, "Must be a TSI emulator.");
+	logMessage(LOG_INFO,"Found display again at 9600bps.");
+	logMessage(LOG_INFO, "Must be a TSI emulator.");
         fullFreshenEvery = 1;
       }else{
-	LogPrint(LOG_ERR,"Display lost after baud switching");
+	logMessage(LOG_ERR,"Display lost after baud switching");
 	goto failure;
       }
     }
@@ -503,6 +504,8 @@ brl_construct (BrailleDisplay *brl, char **parameters, const char *device)
   brl->textColumns = brl_cols;		/* initialise size of display */
   brl->textRows = BRLROWS;		/* always 1 */
 
+  makeOutputTable(dotsTable_ISO11548_1);
+
   /* Allocate space for buffers */
   dispbuf = malloc(ncells);
   prevdata = malloc(ncells);
@@ -555,7 +558,7 @@ brl_destruct (BrailleDisplay *brl)
 static void 
 display (BrailleDisplay *brl, 
          const unsigned char *pattern, 
-	 unsigned start, unsigned stop)
+	 unsigned int from, unsigned int to)
 /* display a given dot pattern. We process only part of the pattern, from
    byte (cell) start to byte stop. That pattern should be shown at position 
    start on the display. */
@@ -563,13 +566,13 @@ display (BrailleDisplay *brl,
   int i, length;
 
   /* Assumes BRLROWS == 1 */
-  length = stop - start + 1;
+  length = to - from;
 
   rawdata[BRL_SEND_LENGTH] = 2 * length;
-  rawdata[BRL_SEND_OFFSET] = start;
+  rawdata[BRL_SEND_OFFSET] = from;
 
   for (i = 0; i < length; i++)
-    rawdata[DIM_BRL_SEND + 2 * i + 1] = outputTable[pattern[start + i]];
+    rawdata[DIM_BRL_SEND + 2 * i + 1] = translateOutputCell(pattern[from + i]);
 
   /* Some displays apparently don't like rapid updating. Most or all apprently
      don't do flow control. If we update the display too often and too fast,
@@ -613,7 +616,7 @@ static void
 display_all (BrailleDisplay *brl,
              unsigned char *pattern)
 {
-  display (brl, pattern, 0, ncells - 1);
+  display (brl, pattern, 0, ncells);
 }
 
 
@@ -633,16 +636,11 @@ brl_writeWindow (BrailleDisplay *brl, const wchar_t *text)
     memcpy(prevdata, dispbuf, ncells);
     display_all (brl, dispbuf);
   }else if(no_multiple_updates){
-    int start, stop;
+    unsigned int from, to;
     
-    for(start=0; start<ncells; start++)
-      if(dispbuf[start] != prevdata[start]) break;
-    if(start == ncells) return 1;
-    for(stop = ncells-1; stop > start; stop--)
-      if(dispbuf[stop] != prevdata[stop]) break;
-    
-    memcpy(prevdata+start, dispbuf+start, stop-start+1);
-    display (brl, dispbuf, start, stop);
+    if (cellsHaveChanged(prevdata, dispbuf, ncells, &from, &to)) {
+      display (brl, dispbuf, from, to);
+    }
   }else{
     int base = 0, i = 0, collecting = 0, simil = 0;
     
@@ -652,7 +650,7 @@ brl_writeWindow (BrailleDisplay *brl, const wchar_t *text)
 	  simil++;
 	  if (collecting && 2 * simil > DIM_BRL_SEND)
 	    {
-	      display (brl, dispbuf, base, i - simil);
+	      display (brl, dispbuf, base, i - simil + 1);
 	      base = i;
 	      collecting = 0;
 	      simil = 0;
@@ -670,7 +668,7 @@ brl_writeWindow (BrailleDisplay *brl, const wchar_t *text)
 	}
     
     if (collecting)
-      display (brl, dispbuf, base, i - simil - 1);
+      display (brl, dispbuf, base, i - simil );
   }
 return 1;
 }
@@ -767,7 +765,7 @@ cut_cursor (BrailleDisplay *brl)
       display_all (brl, prevdata);
       prevdata[pos] = oldchar;
 
-      while ((key = brl_readCommand (brl, BRL_CTX_SCREEN)) == EOF) approximateDelay(1); /* just yield */
+      while ((key = brl_readCommand (brl, KTB_CTX_DEFAULT)) == EOF) approximateDelay(1); /* just yield */
       if((key &BRL_MSK_BLK) == BRL_BLK_CUTBEGIN)
 	  res = BRL_BLK_CUTBEGIN + pos;
       else if((key &BRL_MSK_BLK) == BRL_BLK_CUTAPPEND)
@@ -855,7 +853,7 @@ cut_cursor (BrailleDisplay *brl)
 #define MAXREAD 10
 
 static int 
-brl_readCommand (BrailleDisplay *brl, BRL_DriverCommandContext context)
+brl_readCommand (BrailleDisplay *brl, KeyTableCommandContext context)
 {
   /* static bit vector recording currently pressed sensor switches (for
      repetition detection) */
@@ -915,7 +913,7 @@ brl_readCommand (BrailleDisplay *brl, BRL_DriverCommandContext context)
       if((pings>=PING_MAXNQUERY && ping_due))
 	return BRL_CMD_RESTARTBRL;
       else if(ping_due){
-	LogPrint(LOG_DEBUG,"Display idle: sending query");
+	logMessage(LOG_DEBUG,"Display idle: sending query");
 	serialDrainOutput(serialDevice);
 	approximateDelay(2*SEND_DELAY);
 	serialWriteData (serialDevice, BRL_QUERY, DIM_BRL_QUERY);
@@ -987,7 +985,7 @@ brl_readCommand (BrailleDisplay *brl, BRL_DriverCommandContext context)
     return (EOF);
   }else if(packtype == K_QUERYREP){
     /* flush the last 10bytes of the reply. */
-    LogPrint(LOG_DEBUG,"Got reply to idle ping");
+    logMessage(LOG_DEBUG,"Got reply to idle ping");
     myread(buf, Q_REPLY_LENGTH - Q_HEADER_LENGTH);
     return(EOF);
   }else if(packtype == K_NAVKEY || packtype == K_PBKEY){
@@ -1148,7 +1146,7 @@ brl_readCommand (BrailleDisplay *brl, BRL_DriverCommandContext context)
 
     KEYAND(KEY_CNCV) KEY (KEY_BROUND, BRL_CMD_HOME);
     KEYAND(KEY_CNCV | KEY_CUP) KEY(KEY_BROUND | KEY_CUP, BRL_CMD_BACK);
-    KEY (KEY_CROUND, (context == BRL_CTX_PREFS) ? BRL_CMD_MENU_PREV_SETTING
+    KEY (KEY_CROUND, (context == KTB_CTX_MENU) ? BRL_CMD_MENU_PREV_SETTING
 	 : BRL_CMD_CSRTRK);
 
     KEYAND(KEY_BUT1 | KEY_BAR1) KEY (KEY_BLEFT | KEY_BUP, BRL_CMD_TOP_LEFT);
@@ -1175,9 +1173,9 @@ brl_readCommand (BrailleDisplay *brl, BRL_DriverCommandContext context)
   /* keyboard cursor keys simulation */
     KEY (KEY_CLEFT, BRL_BLK_PASSKEY+BRL_KEY_CURSOR_LEFT);
     KEY (KEY_CRIGHT, BRL_BLK_PASSKEY+BRL_KEY_CURSOR_RIGHT);
-    KEY (KEY_CUP, (context == BRL_CTX_PREFS && displayType == PB40)
+    KEY (KEY_CUP, (context == KTB_CTX_MENU && displayType == PB40)
 	 ? BRL_CMD_MENU_PREV_SETTING : BRL_BLK_PASSKEY+BRL_KEY_CURSOR_UP);
-    KEY (KEY_CDOWN, (context == BRL_CTX_PREFS && displayType == PB40)
+    KEY (KEY_CDOWN, (context == KTB_CTX_MENU && displayType == PB40)
 	 ? BRL_CMD_MENU_NEXT_SETTING : BRL_BLK_PASSKEY+BRL_KEY_CURSOR_DOWN);
 
   /* special modes */
