@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2010 by The BRLTTY Developers.
+ * Copyright (C) 1995-2011 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -26,3 +26,19 @@
 #define VBLPTSTROBE 0x40
 #define VBLPTCLOCK 0x20
 #define VBLPTDATA 0x80
+
+#define KEY_UP 0x1
+#define KEY_LEFT 0x2
+#define KEY_RIGHT 0x4
+#define KEY_DOWN 0x8
+#define KEY_ATTRIBUTES 0x10
+#define KEY_CURSOR 0x20
+#define KEY_HOME 0x40
+#define KEY_MENU 0x80
+
+typedef struct {
+  unsigned char bigbuttons;
+  char routingkey : 7;
+  char keypressed : 1;
+} vbButtons;
+
