@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2009 by The BRLTTY Developers.
+ * Copyright (C) 1995-2011 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -56,11 +56,15 @@
 
 #ifdef __MINGW32__
 #define close(fd) CloseHandle((HANDLE)(fd))
-#define LogSocketError(msg) LogWindowsSocketError(msg)
+#define LogSocketError(msg) logWindowsSocketError(msg)
 #else /* __MINGW32__ */
-#define LogSocketError(msg) LogError(msg)
+#define LogSocketError(msg) logSystemError(msg)
 #endif /* __MINGW32__ */
 
+#include "log.h"
+#include "device.h"
+#include "parse.h"
+#include "timing.h"
 #include "misc.h"
 #include "charset.h"
 #include "cmd.h"
@@ -102,7 +106,7 @@ static int statusColumns;
 static int statusRows;
 static int statusCount;
 static unsigned char *statusCells = NULL;
-static unsigned char genericCells[BRL_genericStatusCellCount];
+static unsigned char genericCells[GSC_COUNT];
 
 typedef struct {
 #ifdef AF_LOCAL
@@ -199,7 +203,7 @@ acceptSocketConnection (
 
           {
             char *address = formatSocketAddress(localAddress);
-            LogPrint(LOG_NOTICE, "listening on: %s", address);
+            logMessage(LOG_NOTICE, "listening on: %s", address);
             free(address);
           }
 
@@ -221,7 +225,7 @@ acceptSocketConnection (
                 break;
 
               case 0:
-                LogPrint(LOG_DEBUG, "no connection yet, still waiting (%d).", attempts);
+                logMessage(LOG_DEBUG, "no connection yet, still waiting (%d).", attempts);
                 continue;
 
               default: {
@@ -229,7 +233,7 @@ acceptSocketConnection (
 
                 if ((serverSocket = accept(queueSocket, remoteAddress, remoteSize)) != -1) {
                   char *address = formatSocketAddress(remoteAddress);
-                  LogPrint(LOG_NOTICE, "client is: %s", address);
+                  logMessage(LOG_NOTICE, "client is: %s", address);
                   free(address);
                 } else {
                   LogSocketError("accept");
@@ -266,7 +270,7 @@ requestConnection (
 
   {
     char *address = formatSocketAddress(remoteAddress);
-    LogPrint(LOG_DEBUG, "connecting to: %s", address);
+    logMessage(LOG_DEBUG, "connecting to: %s", address);
     free(address);
   }
 
@@ -274,14 +278,14 @@ requestConnection (
     if (connect(clientSocket, remoteAddress, remoteSize) != -1) {
       {
         char *address = formatSocketAddress(remoteAddress);
-        LogPrint(LOG_NOTICE, "connected to: %s", address);
+        logMessage(LOG_NOTICE, "connected to: %s", address);
         free(address);
       }
 
       operations = &socketOperationsEntry;
       return clientSocket;
     } else {
-      LogPrint(LOG_WARNING, "connect error: %s", strerror(errno));
+      logMessage(LOG_WARNING, "connect error: %s", strerror(errno));
     }
 
     close(clientSocket);
@@ -316,7 +320,7 @@ setLocalAddress (const char *string, struct sockaddr_un *address) {
     strncpy(address->sun_path, string, sizeof(address->sun_path)-1);
   } else {
     ok = 0;
-    LogPrint(LOG_WARNING, "Local socket path too long: %s", string);
+    logMessage(LOG_WARNING, "Local socket path too long: %s", string);
   }
 
   return ok;
@@ -331,7 +335,7 @@ static void
 unbindLocalAddress (const struct sockaddr *address) {
   const struct sockaddr_un *localAddress = (const struct sockaddr_un *)address;
   if (unlink(localAddress->sun_path) == -1) {
-    LogError("unlink");
+    logSystemError("unlink");
   }
 }
 
@@ -359,7 +363,7 @@ readNamedPipe (int descriptor, void *buffer, int size) {
     DWORD available;
 
     if (!PeekNamedPipe((HANDLE)descriptor, NULL, 0, NULL, &available, NULL)) {
-      LogWindowsError("PeekNamedPipe");
+      logWindowsSystemError("PeekNamedPipe");
       return 0;
     }
 
@@ -378,10 +382,10 @@ readNamedPipe (int descriptor, void *buffer, int size) {
 
     if (!ReadFile((HANDLE)descriptor, buffer, size, &received, &overl)) {
       if (GetLastError() != ERROR_IO_PENDING) {
-        LogWindowsError("ReadPipe");
+        logWindowsSystemError("ReadPipe");
         received = 0;
       } else if (!GetOverlappedResult((HANDLE)descriptor, &overl, &received, TRUE)) {
-        LogWindowsError("GetOverlappedResult");
+        logWindowsSystemError("GetOverlappedResult");
         received = 0;
       }
     }
@@ -406,7 +410,7 @@ acceptNamedPipeConnection (const char *path) {
                                 PIPE_ACCESS_DUPLEX | FILE_FLAG_OVERLAPPED,
                                 PIPE_TYPE_BYTE | PIPE_READMODE_BYTE,
                                 1, 0, 0, 0, NULL)) == INVALID_HANDLE_VALUE) {
-    LogWindowsError("CreateNamedPipe");
+    logWindowsSystemError("CreateNamedPipe");
     return -1;
   }
 
@@ -417,9 +421,9 @@ acceptNamedPipeConnection (const char *path) {
         while ((res = WaitForSingleObject(overl.hEvent, 10000)) != WAIT_OBJECT_0) {
           if (res == WAIT_TIMEOUT) {
             ++attempts;
-            LogPrint(LOG_DEBUG, "no connection yet, still waiting (%d).", attempts);
+            logMessage(LOG_DEBUG, "no connection yet, still waiting (%d).", attempts);
           } else {
-            LogWindowsError("ConnectNamedPipe");
+            logWindowsSystemError("ConnectNamedPipe");
             CloseHandle(h);
             h = (HANDLE) -1;
             break;
@@ -430,7 +434,7 @@ acceptNamedPipeConnection (const char *path) {
         break;
 
       default:
-        LogWindowsError("ConnectNamedPipe");
+        logWindowsSystemError("ConnectNamedPipe");
         CloseHandle(h);
         h = (HANDLE) -1;
         break;
@@ -450,7 +454,7 @@ requestNamedPipeConnection (const char *path) {
                       GENERIC_READ|GENERIC_WRITE,
                       FILE_SHARE_READ|FILE_SHARE_WRITE,
                       NULL, OPEN_EXISTING, 0, NULL)) == INVALID_HANDLE_VALUE) {
-    LogWindowsError("Connect to named pipe");
+    logWindowsSystemError("Connect to named pipe");
     return -1;
   }
 
@@ -480,7 +484,7 @@ setInetAddress (const char *string, struct sockaddr_in *address) {
       memcpy(&address->sin_addr, host->h_addr, sizeof(address->sin_addr));
     } else {
       ok = 0;
-      LogPrint(LOG_WARNING, "Unknown host name: %s", hostName);
+      logMessage(LOG_WARNING, "Unknown host name: %s", hostName);
     }
   } else {
     address->sin_addr.s_addr = INADDR_ANY;
@@ -493,7 +497,7 @@ setInetAddress (const char *string, struct sockaddr_in *address) {
         address->sin_port = htons(port);
       } else {
         ok = 0;
-        LogPrint(LOG_WARNING, "Invalid port number: %s", portNumber);
+        logMessage(LOG_WARNING, "Invalid port number: %s", portNumber);
       }
     } else {
       const struct servent *service = getservbyname(portNumber, "tcp");
@@ -501,7 +505,7 @@ setInetAddress (const char *string, struct sockaddr_in *address) {
         address->sin_port = service->s_port;
       } else {
         ok = 0;
-        LogPrint(LOG_WARNING, "Unknown service: %s", portNumber);
+        logMessage(LOG_WARNING, "Unknown service: %s", portNumber);
       }
     }
   } else {
@@ -857,26 +861,26 @@ dimensionsChanged (BrailleDisplay *brl) {
               if ((word = nextWord())) {
                 if (isInteger(&rows2, word) && (rows2 > 0)) {
                 } else {
-                  LogPrint(LOG_WARNING, "invalid status row count: %s", word);
+                  logMessage(LOG_WARNING, "invalid status row count: %s", word);
                   ok = 0;
                 }
               }
             } else {
-              LogPrint(LOG_WARNING, "invalid status column count: %s", word);
+              logMessage(LOG_WARNING, "invalid status column count: %s", word);
               ok = 0;
             }
           }
         } else {
-          LogPrint(LOG_WARNING, "invalid text row count: %s", word);
+          logMessage(LOG_WARNING, "invalid text row count: %s", word);
           ok = 0;
         }
       }
     } else {
-      LogPrint(LOG_WARNING, "invalid text column count: %s", word);
+      logMessage(LOG_WARNING, "invalid text column count: %s", word);
       ok = 0;
     }
   } else {
-    LogPrint(LOG_WARNING, "missing text column count");
+    logMessage(LOG_WARNING, "missing text column count");
   }
 
   if (ok) {
@@ -908,13 +912,12 @@ dimensionsChanged (BrailleDisplay *brl) {
           if (statusCells) free(statusCells);
           statusCells = status;
           memset(statusCells, 0, count2);
-          memset(genericCells, 0, BRL_genericStatusCellCount);
+          memset(genericCells, 0, GSC_COUNT);
 
           brl->textColumns = brailleColumns;
           brl->textRows = brailleRows;
           brl->statusColumns = statusColumns;
           brl->statusRows = statusRows;
-          brl->helpPage = 0;
           return 1;
         }
 
@@ -984,7 +987,7 @@ brl_construct (BrailleDisplay *brl, char **parameters, const char *device) {
   } else {
     static WSADATA wsadata;
     if (WSAStartup(MAKEWORD(1, 1), &wsadata)) {
-      LogWindowsError("socket library start");
+      logWindowsSystemError("socket library start");
       goto failed;
     }
   }
@@ -1004,7 +1007,7 @@ brl_construct (BrailleDisplay *brl, char **parameters, const char *device) {
       if (line) free(line);
       if ((line = readCommandLine())) {
         const char *word;
-        LogPrint(LOG_DEBUG, "command received: %s", line);
+        logMessage(LOG_DEBUG, "command received: %s", line);
 
         if ((word = strtok(line, inputDelimiters))) {
           if (testWord(word, "cells")) {
@@ -1015,7 +1018,7 @@ brl_construct (BrailleDisplay *brl, char **parameters, const char *device) {
           } else if (testWord(word, "quit")) {
             break;
           } else {
-            LogPrint(LOG_WARNING, "unexpected command: %s", word);
+            logMessage(LOG_WARNING, "unexpected command: %s", word);
           }
         }
       } else {
@@ -1087,13 +1090,11 @@ brl_writeWindow (BrailleDisplay *brl, const wchar_t *text) {
     }
   }
 
-  if (memcmp(brl->buffer, brailleCells, brailleCount) != 0) {
+  if (cellsHaveChanged(brailleCells, brl->buffer, brailleCount, NULL, NULL)) {
     writeString("Braille \"");
     writeDots(brl->buffer, brailleCount);
     writeString("\"");
     writeLine();
-
-    memcpy(brailleCells, brl->buffer, brailleCount);
   }
 
   return 1;
@@ -1101,52 +1102,52 @@ brl_writeWindow (BrailleDisplay *brl, const wchar_t *text) {
 
 static int
 brl_writeStatus (BrailleDisplay *brl, const unsigned char *status) {
-  int generic = status[BRL_firstStatusCell] == BRL_STATUS_CELLS_GENERIC;
+  int generic = status[GSC_FIRST] == GSC_MARKER;
   unsigned char *cells;
   int count;
 
   if (generic) {
     cells = genericCells;
-    count = BRL_genericStatusCellCount;
+    count = GSC_COUNT;
   } else {
     cells = statusCells;
     count = statusCount;
   }
 
-  if (memcmp(status, cells, count) != 0) {
+  if (cellsHaveChanged(cells, status, count, NULL, NULL)) {
     if (generic) {
-      int all = cells[BRL_firstStatusCell] != BRL_STATUS_CELLS_GENERIC;
+      int all = cells[GSC_FIRST] != GSC_MARKER;
       int i;
 
       for (i=1; i<count; ++i) {
         unsigned char value = status[i];
         if (all || (value != cells[i])) {
           static const char *const names[] = {
-            [BRL_firstStatusCell] = NULL,
-            [BRL_GSC_BRLCOL] = "BRLCOL",
-            [BRL_GSC_BRLROW] = "BRLROW",
-            [BRL_GSC_CSRCOL] = "CSRCOL",
-            [BRL_GSC_CSRROW] = "CSRROW",
-            [BRL_GSC_SCRNUM] = "SCRNUM",
-            [BRL_GSC_FREEZE] = "FREEZE",
-            [BRL_GSC_DISPMD] = "DISPMD",
-            [BRL_GSC_SIXDOTS] = "SIXDOTS",
-            [BRL_GSC_SLIDEWIN] = "SLIDEWIN",
-            [BRL_GSC_SKPIDLNS] = "SKPIDLNS",
-            [BRL_GSC_SKPBLNKWINS] = "SKPBLNKWINS",
-            [BRL_GSC_CSRVIS] = "CSRVIS",
-            [BRL_GSC_CSRHIDE] = "CSRHIDE",
-            [BRL_GSC_CSRTRK] = "CSRTRK",
-            [BRL_GSC_CSRSIZE] = "CSRSIZE",
-            [BRL_GSC_CSRBLINK] = "CSRBLINK",
-            [BRL_GSC_ATTRVIS] = "ATTRVIS",
-            [BRL_GSC_ATTRBLINK] = "ATTRBLINK",
-            [BRL_GSC_CAPBLINK] = "CAPBLINK",
-            [BRL_GSC_TUNES] = "TUNES",
-            [BRL_GSC_HELP] = "HELP",
-            [BRL_GSC_INFO] = "INFO",
-            [BRL_GSC_AUTOREPEAT] = "AUTOREPEAT",
-            [BRL_GSC_AUTOSPEAK] = "AUTOSPEAK"
+            [GSC_FIRST] = NULL,
+            [gscWindowColumn] = "BRLCOL",
+            [gscWindowRow] = "BRLROW",
+            [gscCursorColumn] = "CSRCOL",
+            [gscCursorRow] = "CSRROW",
+            [gscScreenNumber] = "SCRNUM",
+            [gscFrozenScreen] = "FREEZE",
+            [gscDisplayMode] = "DISPMD",
+            [gscTextStyle] = "SIXDOTS",
+            [gscSlidingWindow] = "SLIDEWIN",
+            [gscSkipIdenticalLines] = "SKPIDLNS",
+            [gscSkipBlankWindows] = "SKPBLNKWINS",
+            [gscShowCursor] = "CSRVIS",
+            [gscHideCursor] = "CSRHIDE",
+            [gscTrackCursor] = "CSRTRK",
+            [gscCursorStyle] = "CSRSIZE",
+            [gscBlinkingCursor] = "CSRBLINK",
+            [gscShowAttributes] = "ATTRVIS",
+            [gscBlinkingAttributes] = "ATTRBLINK",
+            [gscBlinkingCapitals] = "CAPBLINK",
+            [gscAlertTunes] = "TUNES",
+            [gscHelpScreen] = "HELP",
+            [gscInfoMode] = "INFO",
+            [gscAutorepeat] = "AUTOREPEAT",
+            [gscAutospeak] = "AUTOSPEAK"
           };
           const int nameCount = ARRAY_COUNT(names);
 
@@ -1167,21 +1168,19 @@ brl_writeStatus (BrailleDisplay *brl, const unsigned char *status) {
       writeString("\"");
       writeLine();
     }
-
-    memcpy(cells, status, count);
   }
 
   return 1;
 }
 
 static int
-brl_readCommand (BrailleDisplay *brl, BRL_DriverCommandContext context) {
+brl_readCommand (BrailleDisplay *brl, KeyTableCommandContext context) {
   int command = EOF;
   char *line = readCommandLine();
 
   if (line) {
     const char *word;
-    LogPrint(LOG_DEBUG, "Command received: %s", line);
+    logMessage(LOG_DEBUG, "Command received: %s", line);
 
     if ((word = strtok(line, inputDelimiters))) {
       if (testWord(word, "cells")) {
@@ -1224,20 +1223,20 @@ brl_readCommand (BrailleDisplay *brl, BRL_DriverCommandContext context) {
                   command += number;
                   continue;
                 } else {
-                  LogPrint(LOG_WARNING, "Number out of range.");
+                  logMessage(LOG_WARNING, "Number out of range.");
                 }
               }
             }
 
-            LogPrint(LOG_WARNING, "unknown option: %s", word);
+            logMessage(LOG_WARNING, "unknown option: %s", word);
           }
 
           if (needsNumber && !numberSpecified) {
-            LogPrint(LOG_WARNING, "Number not specified.");
+            logMessage(LOG_WARNING, "Number not specified.");
             command = EOF;
           }
         } else {
-          LogPrint(LOG_WARNING, "unknown command: %s", word);
+          logMessage(LOG_WARNING, "unknown command: %s", word);
         }
       }
     }
