@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2010 by The BRLTTY Developers.
+ * Copyright (C) 1995-2011 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -211,12 +211,12 @@ typedef enum {
 static int cols,lines;
 static int input;
 static char *model = "simple";
-static const char *fontname = "-*-clearlyu-*-r-*-*-17-*-*-*-*-*-iso10646-1,-*-fixed-*-r-*-*-17-*-*-*-*-*-iso10646-1,-*-unifont-*-r-*-*-17-*-*-*-*-*-iso10646-1";
+static const char *fontname = "-*-clearlyu-*-r-*-*-17-*-*-*-*-*-iso10646-1,-*-fixed-*-r-*-*-17-*-*-*-*-*-iso10646-1,-*-unifont-*-r-*-*-17-*-*-*-*-*-iso10646-1,-*-fixed-*-*-*-*-*-*-*-*-*-*-iso10646-1,-*-fixed-*-*-*-*-*-*-*-*-*-*-iso8859-1";
 static int xtArgc = 1;
 static char *xtDefArgv[]= { "brltty", NULL };
 static char **xtArgv = xtDefArgv;
 static int regenerate;
-static void generateToplevel(void);
+static int generateToplevel(void);
 static void destroyToplevel(void);
 #if defined(USE_XAW) || defined(USE_WINDOWS)
 static unsigned char displayedWindow[WHOLESIZE];
@@ -258,7 +258,7 @@ static int modelWidth,modelHeight;
 #ifdef USE_XT
 static void KeyPressCB(Widget w, XtPointer closure, XtPointer callData)
 {
-  LogPrint(LOG_DEBUG,"keypresscb(%p)", closure);
+  logMessage(LOG_DEBUG,"keypresscb(%p)", closure);
   enqueueCommand((long) closure);
 }
 
@@ -269,12 +269,12 @@ static void keypress(Widget w, XEvent *event, String *params, Cardinal *num_para
   KeySym keysym;
 
   if (event->type != KeyPress && event->type != KeyRelease) {
-    LogPrint(LOG_ERR,"keypress is not a KeyPress");
+    logMessage(LOG_ERR,"keypress is not a KeyPress");
     return;
   }
   keysym = XtGetActionKeysym(event, &modifiers);
   modifiers |= my_modifiers;
-  LogPrint(LOG_DEBUG,"keypress(%#lx), modif(%#x)", keysym, modifiers);
+  logMessage(LOG_DEBUG,"keypress(%#lx), modif(%#x)", keysym, modifiers);
 
   /* latin1 */
   if (keysym < 0x100) keysym |= 0x1000000;
@@ -286,7 +286,7 @@ static void keypress(Widget w, XEvent *event, String *params, Cardinal *num_para
     else {
       int c = convertWcharToChar(keysym & 0xffffff);
       if (c == EOF) {
-	LogPrint(LOG_DEBUG, "non translatable unicode U+%lx", keysym & 0xffffff);
+	logMessage(LOG_DEBUG, "non translatable unicode U+%lx", keysym & 0xffffff);
 	return;
       }
       keypressed = BRL_BLK_PASSCHAR | c;
@@ -384,7 +384,7 @@ static void keypress(Widget w, XEvent *event, String *params, Cardinal *num_para
     case XK_KP_7:         keypressed = BRL_BLK_PASSCHAR | '7'; break;
     case XK_KP_8:         keypressed = BRL_BLK_PASSCHAR | '8'; break;
     case XK_KP_9:         keypressed = BRL_BLK_PASSCHAR | '9'; break;
-    default: LogPrint(LOG_DEBUG,"unsupported keysym %lx",keysym); return;
+    default: logMessage(LOG_DEBUG,"unsupported keysym %lx",keysym); return;
   }
   if (modifiers & ControlMask)
     keypressed |= BRL_FLG_CHAR_CONTROL;
@@ -398,12 +398,12 @@ static void keypress(Widget w, XEvent *event, String *params, Cardinal *num_para
     keypressed |= BRL_FLG_REPEAT_DELAY | BRL_FLG_REPEAT_INITIAL;
   else
     keypressed = BRL_CMD_NOOP;
-  LogPrint(LOG_DEBUG,"keypressed %#lx", keypressed);
+  logMessage(LOG_DEBUG,"keypressed %#lx", keypressed);
   enqueueCommand(keypressed);
   return;
 
 modif:
-  LogPrint(LOG_DEBUG,"modifier %#x", modifier);
+  logMessage(LOG_DEBUG,"modifier %#x", modifier);
   if (event->type == KeyPress)
     my_modifiers |= modifier;
   else
@@ -413,7 +413,7 @@ modif:
 static void route(Widget w, XEvent *event, String *params, Cardinal *num_params)
 {
   int index = atoi(params[0]);
-  LogPrint(LOG_DEBUG,"route(%u)", index);
+  logMessage(LOG_DEBUG,"route(%u)", index);
   enqueueCommand(BRL_BLK_ROUTE | (index&BRL_MSK_ARG));
 }
 
@@ -650,16 +650,16 @@ int CALLBACK fontEnumProc(ENUMLOGFONTEX *lpelfe, NEWTEXTMETRICEX *lpntme, DWORD
 		return 1;
 	font = CreateFont(CHRY-6, CHRX-4, 0, 0, 0, FALSE, FALSE, FALSE, DEFAULT_CHARSET, OUT_DEFAULT_PRECIS, CLIP_DEFAULT_PRECIS, DEFAULT_QUALITY, DEFAULT_PITCH, (LPCTSTR) lpelfe->elfFullName);
 	if (!font) {
-		LogWindowsError("Couldn't load font");
-		LogPrint(LOG_ERR,"font %s", lpelfe->elfFullName);
+		logWindowsSystemError("Couldn't load font");
+		logMessage(LOG_ERR,"font %s", lpelfe->elfFullName);
 		return 1;
 	}
-	LogPrint(LOG_INFO, "Using braille font `%s\'",lpelfe->elfFullName);
+	logMessage(LOG_INFO, "Using braille font `%s\'",lpelfe->elfFullName);
 	return 0;
 }
 #endif /* USE_WINDOWS */
 
-static int brl_readCommand(BrailleDisplay *brl, BRL_DriverCommandContext context)
+static int brl_readCommand(BrailleDisplay *brl, KeyTableCommandContext context)
 {
 #if defined(USE_XT)
   while (XtAppPending(app_con)) {
@@ -717,7 +717,7 @@ static void popup(Widget w, XEvent *event, String *params, Cardinal *num_params)
 }
 #endif /* USE_XM */
 
-static void generateToplevel(void)
+static int generateToplevel(void)
 {
 #ifdef USE_XT
   int argc;
@@ -787,7 +787,7 @@ static void generateToplevel(void)
     EnumFontFamiliesEx(hdc, NULL, (void*) fontEnumProc, 0, 0);
     ReleaseDC(root, hdc);
     if (!font) {
-      LogPrint(LOG_ERR,"Error while loading braille font");
+      logMessage(LOG_ERR,"Error while loading braille font");
       totlines = lines;
     } else {
       totlines = 2*lines;
@@ -809,8 +809,12 @@ static void generateToplevel(void)
     };
     if (!(RegisterClass(&wndclass)) &&
 	GetLastError() != ERROR_CLASS_ALREADY_EXISTS) {
-      LogWindowsError("RegisterClass");
-      exit(1);
+      logWindowsSystemError("RegisterClass");
+      if (font) {
+	DeleteObject(font);
+	font = NULL;
+      }
+      return 0;
     }
     modelWidth = cols*CHRX;
     if (keyModel) {
@@ -824,8 +828,12 @@ static void generateToplevel(void)
 	    "BRLTTYWClass", "BRLTTY",
 	    WS_POPUP, GetSystemMetrics(SM_CXSCREEN)-modelWidth-RIGHTMARGIN, 0,
 	    modelWidth, totlines*CHRY+modelHeight, NULL, NULL, NULL, NULL))) {
-      LogWindowsError("CreateWindow");
-      exit(1);
+      logWindowsSystemError("CreateWindow");
+      if (font) {
+	DeleteObject(font);
+	font = NULL;
+      }
+      return 0;
     }
   }
 #else /* USE_ */
@@ -849,11 +857,11 @@ static void generateToplevel(void)
 
 #ifdef USE_XAW
   if (!(fontset = XCreateFontSet(XtDisplay(toplevel), fontname, &missing_charset_list_return, &missing_charset_count_return, &def_string_return)))
-    LogPrint(LOG_ERR,"Error while loading unicode font");
+    logMessage(LOG_ERR,"Error while loading unicode font");
   if (missing_charset_count_return) {
     int i;
     for (i=0; i<missing_charset_count_return; i++)
-      LogPrint(LOG_INFO,"Could not load a unicode font for charset %s",missing_charset_list_return[i]);
+      logMessage(LOG_INFO,"Could not load a unicode font for charset %s",missing_charset_list_return[i]);
     XFreeStringList(missing_charset_list_return);
   }
 #endif /* USE_XAW */
@@ -1056,6 +1064,7 @@ static void generateToplevel(void)
 #endif /* USE_XAW || USE_WINDOWS */
   memset(displayedVisual,0,sizeof(displayedVisual));
   lastcursor = -1;
+  return 1;
 }
 
 static int brl_construct(BrailleDisplay *brl, char **parameters, const char *device)
@@ -1068,7 +1077,7 @@ static int brl_construct(BrailleDisplay *brl, char **parameters, const char *dev
     if (validateInteger(&value, parameters[PARM_LINES], &minimum, &maximum)) {
       lines=value;
     } else {
-      LogPrint(LOG_WARNING, "%s: %s", "invalid line count", parameters[PARM_LINES]);
+      logMessage(LOG_WARNING, "%s: %s", "invalid line count", parameters[PARM_LINES]);
     }
   }
 
@@ -1080,7 +1089,7 @@ static int brl_construct(BrailleDisplay *brl, char **parameters, const char *dev
     if (validateInteger(&value, parameters[PARM_COLUMNS], &minimum, &maximum)) {
       cols=value;
     } else {
-      LogPrint(LOG_WARNING, "%s: %s", "invalid column count", parameters[PARM_COLUMNS]);
+      logMessage(LOG_WARNING, "%s: %s", "invalid column count", parameters[PARM_COLUMNS]);
     }
   }
 
@@ -1089,7 +1098,7 @@ static int brl_construct(BrailleDisplay *brl, char **parameters, const char *dev
     if (validateFlag(&value, parameters[PARM_INPUT], "on", "off")) {
       input = value;
     } else {
-      LogPrint(LOG_WARNING, "%s: %s", "invalid input setting", parameters[PARM_INPUT]);
+      logMessage(LOG_WARNING, "%s: %s", "invalid input setting", parameters[PARM_INPUT]);
     }
   }
 
@@ -1121,9 +1130,7 @@ static int brl_construct(BrailleDisplay *brl, char **parameters, const char *dev
   brl->textColumns=cols;
   brl->textRows=lines;
 
-  generateToplevel();
-
-  return 1;
+  return generateToplevel();
 }
 static void destroyToplevel(void)
 {
@@ -1140,7 +1147,7 @@ static void destroyToplevel(void)
 #elif defined(USE_WINDOWS)
   DestroyMenu(menu);
   if (!DestroyWindow(toplevel))
-    LogWindowsError("DestroyWindow");
+    logWindowsSystemError("DestroyWindow");
   if (font) {
     DeleteObject(font);
     font = NULL;
@@ -1157,6 +1164,7 @@ static void brl_destruct(BrailleDisplay *brl)
 
 static int brl_writeWindow(BrailleDisplay *brl, const wchar_t *text)
 {
+  unsigned int from, to;
   wchar_t wc;
   int i;
 #ifdef USE_XM
@@ -1244,30 +1252,28 @@ static int brl_writeWindow(BrailleDisplay *brl, const wchar_t *text)
   }
 
 #if defined(USE_XAW) || defined(USE_WINDOWS)
-  if (!displayb[0] || (memcmp(brl->buffer,displayedWindow,brl->textRows*brl->textColumns) == 0)) return 1;
+  if (!cellsHaveChanged(displayedWindow,brl->buffer,brl->textRows*brl->textColumns,&from,&to) || !displayb[0]) return 1;
 
-  for (i=0;i<brl->textRows*brl->textColumns;i++)
-    if (displayedWindow[i] != brl->buffer[i]) {
-      unsigned char c = brl->buffer[i];
-      c =
-	 (!!(c&BRL_DOT1))<<0
-	|(!!(c&BRL_DOT2))<<1
-	|(!!(c&BRL_DOT3))<<2
-	|(!!(c&BRL_DOT4))<<3
-	|(!!(c&BRL_DOT5))<<4
-	|(!!(c&BRL_DOT6))<<5
-	|(!!(c&BRL_DOT7))<<6
-	|(!!(c&BRL_DOT8))<<7;
+  for (i=from;i<to;i++) {
+    unsigned char c = brl->buffer[i];
+    c =
+       (!!(c&BRL_DOT1))<<0
+      |(!!(c&BRL_DOT2))<<1
+      |(!!(c&BRL_DOT3))<<2
+      |(!!(c&BRL_DOT4))<<3
+      |(!!(c&BRL_DOT5))<<4
+      |(!!(c&BRL_DOT6))<<5
+      |(!!(c&BRL_DOT7))<<6
+      |(!!(c&BRL_DOT8))<<7;
 #ifdef USE_XAW
-      convertWcharToUtf8(UNICODE_BRAILLE_ROW | c, utf8);
+    convertWcharToUtf8(UNICODE_BRAILLE_ROW | c, utf8);
 
-      XtVaSetValues(displayb[i], XtNlabel, utf8, NULL);
+    XtVaSetValues(displayb[i], XtNlabel, utf8, NULL);
 #elif defined(USE_WINDOWS)
-      data[0] = UNICODE_BRAILLE_ROW | c;
-      data[1] = 0;
-      SetWindowTextW(displayb[i],data);
+    data[0] = UNICODE_BRAILLE_ROW | c;
+    data[1] = 0;
+    SetWindowTextW(displayb[i],data);
 #endif /* USE_WINDOWS */
-      displayedWindow[i] = brl->buffer[i];
   }
 #endif /* USE_XAW || USE_WINDOWS */
   return 1;
