@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2009 by The BRLTTY Developers.
+ * Copyright (C) 1995-2011 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -29,6 +29,10 @@
 #include <linux/vt.h>
 #include <linux/kd.h>
 
+#include "ascii.h"
+#include "log.h"
+#include "device.h"
+#include "parse.h"
 #include "misc.h"
 #include "system.h"
 #include "sys_linux.h"
@@ -72,7 +76,7 @@ static int
 allocateCharsetConverter (CharsetConverter *converter, const char *sourceCharset, const char *targetCharset) {
   if (converter->iconvHandle == ICONV_NULL) {
     if ((converter->iconvHandle = iconv_open(targetCharset, sourceCharset)) == ICONV_NULL) {
-      LogError("iconv_open");
+      logSystemError("iconv_open");
       return 0;
     }
   }
@@ -98,7 +102,7 @@ convertCharacters (
   if (errno == EILSEQ) return CONV_ILLEGAL;
   if (errno == EINVAL) return CONV_SHORT;
   if (errno == E2BIG) return CONV_OVERFLOW;
-  LogError("iconv");
+  logSystemError("iconv");
   return CONV_ERROR;
 }
 #else /* charset conversion definitions */
@@ -290,7 +294,7 @@ convertCharacter (const wchar_t *character) {
       if (result != CONV_ILLEGAL) break;
 
       if (!--length) break;
-      memcpy(buffer, buffer+1, length);
+      memmove(buffer, buffer+1, length);
     }
   }
 
@@ -329,9 +333,9 @@ static void
 closeConsole (void) {
   if (consoleDescriptor != -1) {
     if (close(consoleDescriptor) == -1) {
-      LogError("console close");
+      logSystemError("console close");
     }
-    LogPrint(LOG_DEBUG, "console closed: fd=%d", consoleDescriptor);
+    logMessage(LOG_DEBUG, "console closed: fd=%d", consoleDescriptor);
     consoleDescriptor = -1;
   }
 }
@@ -343,7 +347,7 @@ openConsole (unsigned char vt) {
   if (name) {
     int console = openCharacterDevice(name, O_RDWR|O_NOCTTY, 4, vt);
     if (console != -1) {
-      LogPrint(LOG_DEBUG, "console opened: %s: fd=%d", name, console);
+      logMessage(LOG_DEBUG, "console opened: %s: fd=%d", name, console);
       closeConsole();
       consoleDescriptor = console;
       opened = 1;
@@ -367,9 +371,9 @@ static void
 closeScreen (void) {
   if (screenDescriptor != -1) {
     if (close(screenDescriptor) == -1) {
-      LogError("screen close");
+      logSystemError("screen close");
     }
-    LogPrint(LOG_DEBUG, "screen closed: fd=%d", screenDescriptor);
+    logMessage(LOG_DEBUG, "screen closed: fd=%d", screenDescriptor);
     screenDescriptor = -1;
   }
 }
@@ -381,7 +385,7 @@ openScreen (unsigned char vt) {
   if (name) {
     int screen = openCharacterDevice(name, O_RDWR, 7, 0X80|vt);
     if (screen != -1) {
-      LogPrint(LOG_DEBUG, "screen opened: %s: fd=%d", name, screen);
+      logMessage(LOG_DEBUG, "screen opened: %s: fd=%d", name, screen);
       if (openConsole(vt)) {
         closeScreen();
         screenDescriptor = screen;
@@ -389,7 +393,7 @@ openScreen (unsigned char vt) {
         opened = 1;
       } else {
         close(screen);
-        LogPrint(LOG_DEBUG, "screen closed: fd=%d", screen);
+        logMessage(LOG_DEBUG, "screen closed: fd=%d", screen);
       }
     }
     free(name);
@@ -423,17 +427,17 @@ setScreenFontMap (int force) {
   while (1) {
     sfm.entry_ct = size;
     if (!(sfm.entries = malloc(sfm.entry_ct * sizeof(*sfm.entries)))) {
-      LogError("screen font map allocation");
+      logSystemError("screen font map allocation");
       return 0;
     }
     if (controlConsole(GIO_UNIMAP, &sfm) != -1) break;
     free(sfm.entries);
     if (errno != ENOMEM) {
-      LogError("ioctl GIO_UNIMAP");
+      logSystemError("ioctl GIO_UNIMAP");
       return 0;
     }
     if (!(size <<= 1)) {
-      LogPrint(LOG_ERR, "screen font map too big.");
+      logMessage(LOG_ERR, "screen font map too big.");
       return 0;
     }
   }
@@ -455,13 +459,13 @@ setScreenFontMap (int force) {
   screenFontMapTable = sfm.entries;
   screenFontMapCount = sfm.entry_ct;
   screenFontMapSize = size;
-  LogPrint(LOG_INFO, "Screen Font Map Size: %d", screenFontMapCount);
+  logMessage(LOG_INFO, "Screen Font Map Size: %d", screenFontMapCount);
   if (debugScreenFontMap) {
     int i;
     for (i=0; i<screenFontMapCount; ++i) {
       const struct unipair *map = &screenFontMapTable[i];
-      LogPrint(LOG_DEBUG, "sfm[%03u]: unum=%4.4X fpos=%4.4X",
-               i, map->unicode, map->fontpos);
+      logMessage(LOG_DEBUG, "sfm[%03u]: unum=%4.4X fpos=%4.4X",
+                 i, map->unicode, map->fontpos);
     }
   }
   return 1;
@@ -488,7 +492,7 @@ setVgaCharacterCount (int force) {
       vgaCharacterCount = 0;
 
       if (errno != EINVAL) {
-        LogPrint(LOG_WARNING, "ioctl KDFONTOP[GET]: %s", strerror(errno));
+        logMessage(LOG_WARNING, "ioctl KDFONTOP[GET]: %s", strerror(errno));
       }
     }
   }
@@ -508,9 +512,9 @@ setVgaCharacterCount (int force) {
     if (vgaCharacterCount == oldCount)
       return 0;
 
-  LogPrint(LOG_INFO, "VGA Character Count: %d(%s)",
-           vgaCharacterCount,
-           vgaLargeTable? "large": "small");
+  logMessage(LOG_INFO, "VGA Character Count: %d(%s)",
+             vgaCharacterCount,
+             vgaLargeTable? "large": "small");
 
   return 1;
 }
@@ -527,8 +531,8 @@ setAttributesMasks (unsigned short bit) {
                             (((bit & 0X0F00) - 0X0100) & 0X0F00);
   shiftedAttributesMask = ((~((bit & 0XF000) - 0X1000) << 1) & 0XE000) |
                           ((~((bit & 0X0F00) - 0X0100) << 1) & 0X0E00);
-  LogPrint(LOG_DEBUG, "attributes masks: font=%04X unshifted=%04X shifted=%04X",
-           fontAttributesMask, unshiftedAttributesMask, shiftedAttributesMask);
+  logMessage(LOG_DEBUG, "attributes masks: font=%04X unshifted=%04X shifted=%04X",
+             fontAttributesMask, unshiftedAttributesMask, shiftedAttributesMask);
 }
 
 #ifndef VT_GETHIFONTMASK
@@ -545,9 +549,9 @@ determineAttributesMasks (void) {
     {
       unsigned short mask;
       if (controlConsole(VT_GETHIFONTMASK, &mask) == -1) {
-        if (errno != EINVAL) LogError("ioctl[VT_GETHIFONTMASK]");
+        if (errno != EINVAL) logSystemError("ioctl[VT_GETHIFONTMASK]");
       } else if (mask & 0XFF) {
-        LogPrint(LOG_ERR, "high font mask has bit set in low-order byte: %04X", mask);
+        logMessage(LOG_ERR, "high font mask has bit set in low-order byte: %04X", mask);
       } else {
         setAttributesMasks(mask);
         return 1;
@@ -571,13 +575,13 @@ determineAttributesMasks (void) {
           setAttributesMasks((counts[0XE] > counts[0X7])? 0X0100: 0X0800);
           return 1;
         } else {
-          LogError("read");
+          logSystemError("read");
         }
       } else {
-        LogError("read");
+        logSystemError("read");
       }
     } else {
-      LogError("lseek");
+      logSystemError("lseek");
     }
   }
 
@@ -593,7 +597,7 @@ processParameters_LinuxScreen (char **parameters) {
   }
 
   if (!validateYesNo(&debugScreenFontMap, parameters[PARM_DEBUGSFM]))
-    LogPrint(LOG_WARNING, "%s: %s", "invalid screen font map debug setting", parameters[PARM_DEBUGSFM]);
+    logMessage(LOG_WARNING, "%s: %s", "invalid screen font map debug setting", parameters[PARM_DEBUGSFM]);
 
   highFontBit = 0;
   if (parameters[PARM_HFB] && *parameters[PARM_HFB]) {
@@ -605,7 +609,7 @@ processParameters_LinuxScreen (char **parameters) {
     if (validateInteger(&bit, parameters[PARM_HFB], &minimum, &maximum)) {
       highFontBit = 1 << (bit + 8);
     } else if (!validateChoice(&choice, parameters[PARM_HFB], choices)) {
-      LogPrint(LOG_WARNING, "%s: %s", "invalid high font bit", parameters[PARM_HFB]);
+      logMessage(LOG_WARNING, "%s: %s", "invalid high font bit", parameters[PARM_HFB]);
     } else if (choice) {
       static const unsigned short bits[] = {0X0800, 0X0100};
       highFontBit = bits[choice-1];
@@ -873,16 +877,16 @@ userVirtualTerminal_LinuxScreen (int number) {
 static int
 readScreenDevice (off_t offset, void *buffer, size_t size) {
   if (lseek(screenDescriptor, offset, SEEK_SET) == -1) {
-    LogError("screen seek");
+    logSystemError("screen seek");
   } else {
     ssize_t count = read(screenDescriptor, buffer, size);
     if (count == size) return 1;
 
     if (count == -1) {
-      LogError("screen read");
+      logSystemError("screen read");
     } else {
-      LogPrint(LOG_ERR, "truncated screen data: expected %u bytes, read %d",
-               (unsigned int)size, (int)count);
+      logMessage(LOG_ERR, "truncated screen data: expected %u bytes, read %d",
+                 (unsigned int)size, (int)count);
     }
   }
   return 0;
@@ -1024,7 +1028,7 @@ getConsoleDescription (ScreenDescription *description) {
   } else {
     struct vt_stat state;
     if (controlConsole(VT_GETSTATE, &state) == -1) {
-      LogError("ioctl VT_GETSTATE");
+      logSystemError("ioctl VT_GETSTATE");
       description->number = 0;
       problemText = "can't get virtual terminal number";
       return 0;
@@ -1045,12 +1049,12 @@ getConsoleDescription (ScreenDescription *description) {
   {
     int mode;
     if (controlConsole(KDGETMODE, &mode) == -1) {
-      LogError("ioctl KDGETMODE");
+      logSystemError("ioctl KDGETMODE");
     } else if (mode == KD_TEXT) {
       problemText = NULL;
       return 1;
     }
-    problemText = "screen not in text mode";
+    problemText = gettext("screen not in text mode");
     return 0;
   }
 }
@@ -1259,7 +1263,7 @@ insertUinput (ScreenKey key) {
 static int
 insertByte (char byte) {
   if (controlConsole(TIOCSTI, &byte) != -1) return 1;
-  LogError("ioctl TIOCSTI");
+  logSystemError("ioctl TIOCSTI");
   return 0;
 }
 
@@ -1280,7 +1284,7 @@ insertXlate (wchar_t character) {
 
   if (result != CONV_OK) {
     uint32_t value = character;
-    LogPrint(LOG_WARNING, "character 0X%02" PRIX32 " not insertable in xlate mode." , value);
+    logMessage(LOG_WARNING, "character 0X%02" PRIX32 " not insertable in xlate mode." , value);
     return 0;
   }
 
@@ -1297,7 +1301,7 @@ insertUnicode (wchar_t character) {
 
   {
     uint32_t value = character;
-    LogPrint(LOG_WARNING, "character 0X%02" PRIX32 " not insertable in unicode mode." , value);
+    logMessage(LOG_WARNING, "character 0X%02" PRIX32 " not insertable in unicode mode." , value);
   }
 
   return 0;
@@ -1415,14 +1419,14 @@ insertCode (ScreenKey key, int raw) {
         case SCR_KEY_CURSOR_RIGHT: code = 0X4D; break;
         default:
           if (insertUinput(key)) return 1;
-          LogPrint(LOG_WARNING, "key %04X not suported in raw keycode mode.", key);
+          logMessage(LOG_WARNING, "key %04X not suported in raw keycode mode.", key);
           return 0;
       }
 
       if (raw) {
         prefix = 0XE0;
       } else if (!(code = emul0XtScanCodeToLinuxKeyCode[code])) {
-        LogPrint(LOG_WARNING, "key %04X not suported in medium raw keycode mode.", key);
+        logMessage(LOG_WARNING, "key %04X not suported in medium raw keycode mode.", key);
         return 0;
       }
       break;
@@ -1468,9 +1472,9 @@ insertCode (ScreenKey key, int raw) {
     if (modAltRight) {
       if (raw) {
         codes[count++] = codeEmul0;
-        codes[count++] = codeAlt | bitRelease;;
+        codes[count++] = codeAlt | bitRelease;
       } else {
-        codes[count++] = emul0XtScanCodeToLinuxKeyCode[codeAlt & 0XFF] | bitRelease;;
+        codes[count++] = emul0XtScanCodeToLinuxKeyCode[codeAlt & 0XFF] | bitRelease;
       }
     }
     if (modAltLeft) codes[count++] = codeAlt | bitRelease;
@@ -1488,8 +1492,8 @@ insertCode (ScreenKey key, int raw) {
 static int
 insertTranslated (ScreenKey key, int (*insertCharacter)(wchar_t character)) {
   wchar_t buffer[2];
-  wchar_t *sequence;
-  wchar_t *end;
+  const wchar_t *sequence;
+  const wchar_t *end;
 
   setKeyModifiers(&key, 0);
 
@@ -1599,13 +1603,15 @@ insertTranslated (ScreenKey key, int (*insertCharacter)(wchar_t character)) {
         break;
       default:
 	if (insertUinput(key)) return 1;
-        LogPrint(LOG_WARNING, "key %04X not supported in xlate mode.", key);
+        logMessage(LOG_WARNING, "key %04X not supported in xlate mode.", key);
         return 0;
     }
     end = sequence + wcslen(sequence);
   } else {
-    sequence = end = buffer + ARRAY_COUNT(buffer);
-    *--sequence = key & SCR_KEY_CHAR_MASK;
+    wchar_t *character = buffer + ARRAY_COUNT(buffer);
+
+    end = character;
+    *--character = key & SCR_KEY_CHAR_MASK;
 
     if (key & SCR_KEY_ALT_LEFT) {
       int meta;
@@ -1613,20 +1619,22 @@ insertTranslated (ScreenKey key, int (*insertCharacter)(wchar_t character)) {
 
       switch (meta) {
         case K_ESCPREFIX:
-          *--sequence = 0X1B;
+          *--character = ESC;
           break;
 
         case K_METABIT:
-          if (*sequence < 0X80) {
-            *sequence |= 0X80;
+          if (*character < 0X80) {
+            *character |= 0X80;
             break;
           }
 
         default:
-          LogPrint(LOG_WARNING, "unsupported keyboard meta mode: %d", meta);
+          logMessage(LOG_WARNING, "unsupported keyboard meta mode: %d", meta);
           return 0;
       }
     }
+
+    sequence = character;
   }
 
   while (sequence != end)
@@ -1638,29 +1646,45 @@ insertTranslated (ScreenKey key, int (*insertCharacter)(wchar_t character)) {
 static int
 insertKey_LinuxScreen (ScreenKey key) {
   int ok = 0;
-  LogPrint(LOG_DEBUG, "insert key: %4.4X", key);
+  logMessage(LOG_DEBUG, "insert key: %4.4X", key);
   if (rebindConsole()) {
     int mode;
     if (controlConsole(KDGKBMODE, &mode) != -1) {
       switch (mode) {
+        {
+          int raw;
+
         case K_RAW:
-          if (insertCode(key, 1)) ok = 1;
-          break;
+          raw = 1;
+          goto doCode;
+
         case K_MEDIUMRAW:
-          if (insertCode(key, 0)) ok = 1;
+          raw = 0;
+          goto doCode;
+
+        doCode:
+          if (insertUinput(key)) {
+            ok = 1;
+          } else if (insertCode(key, raw)) {
+            ok = 1;
+          }
           break;
+        }
+
         case K_XLATE:
           if (insertTranslated(key, insertXlate)) ok = 1;
           break;
+
         case K_UNICODE:
           if (insertTranslated(key, insertUnicode)) ok = 1;
           break;
+
         default:
-          LogPrint(LOG_WARNING, "unsupported keyboard mode: %d", mode);
+          logMessage(LOG_WARNING, "unsupported keyboard mode: %d", mode);
           break;
       }
     } else {
-      LogError("ioctl KDGKBMODE");
+      logSystemError("ioctl KDGKBMODE");
     }
   }
   return ok;
@@ -1678,7 +1702,7 @@ typedef struct {
 static int
 selectCharacters (CharacterSelectionArguments *arguments) {
   if (controlConsole(TIOCLINUX, arguments) != -1) return 1;
-  if (errno != EINVAL) LogError("ioctl[TIOCLINUX]");
+  if (errno != EINVAL) logSystemError("ioctl[TIOCLINUX]");
   return 0;
 }
 
@@ -1711,7 +1735,7 @@ unhighlightRegion_LinuxScreen (void) {
 static int
 validateVt (int vt) {
   if ((vt >= 1) && (vt <= 0X3F)) return 1;
-  LogPrint(LOG_DEBUG, "virtual terminal out of range: %d", vt);
+  logMessage(LOG_DEBUG, "virtual terminal out of range: %d", vt);
   return 0;
 }
 
@@ -1727,10 +1751,10 @@ switchVirtualTerminal_LinuxScreen (int vt) {
   if (validateVt(vt)) {
     if (selectVirtualTerminal_LinuxScreen(0)) {
       if (ioctl(consoleDescriptor, VT_ACTIVATE, vt) != -1) {
-        LogPrint(LOG_DEBUG, "switched to virtual tertminal %d.", vt);
+        logMessage(LOG_DEBUG, "switched to virtual tertminal %d.", vt);
         return 1;
       } else {
-        LogError("ioctl VT_ACTIVATE");
+        logSystemError("ioctl VT_ACTIVATE");
       }
     }
   }
@@ -1745,9 +1769,9 @@ currentVirtualTerminal_LinuxScreen (void) {
 }
 
 static int
-executeCommand_LinuxScreen (int command) {
-  int blk = command & BRL_MSK_BLK;
-  int arg UNUSED = command & BRL_MSK_ARG;
+executeCommand_LinuxScreen (int *command) {
+  int blk = *command & BRL_MSK_BLK;
+  int arg UNUSED = *command & BRL_MSK_ARG;
   int cmd = blk | arg;
 
   switch (cmd) {
@@ -1765,17 +1789,17 @@ executeCommand_LinuxScreen (int command) {
             int press = !(arg & 0X80);
             arg &= 0X7F;
 
-            if (command & BRL_FLG_KBD_EMUL0) {
+            if (*command & BRL_FLG_KBD_EMUL0) {
               unsigned char code = emul0XtScanCodeToLinuxKeyCode[arg];
               if (!code) {
-		LogPrint(LOG_WARNING, "Xt emul0 scancode not supported: %02X", arg);
+		logMessage(LOG_WARNING, "Xt emul0 scancode not supported: %02X", arg);
                 return 0;
               }
               arg = code;
-	    } else if (command & BRL_FLG_KBD_EMUL1) {
+	    } else if (*command & BRL_FLG_KBD_EMUL1) {
               unsigned int code = emul1XtScanCodeToLinuxKeyCode[arg];
               if (!code) {
-		LogPrint(LOG_WARNING, "Xt emul1 scancode not supported: %02X", arg);
+		logMessage(LOG_WARNING, "Xt emul1 scancode not supported: %02X", arg);
                 return 0;
               }
               arg = code;
@@ -1788,19 +1812,19 @@ executeCommand_LinuxScreen (int command) {
           {
             int handled = 0;
 
-            if (command & BRL_FLG_KBD_RELEASE) {
+            if (*command & BRL_FLG_KBD_RELEASE) {
               at2Pressed = 0;
             } else if (arg == 0XF0) {
               at2Pressed = 0;
               handled = 1;
             }
 
-            if (command & BRL_FLG_KBD_EMUL0) {
+            if (*command & BRL_FLG_KBD_EMUL0) {
               at2Keys = at2KeysE0;
             } else if (arg == 0XE0) {
               at2Keys = at2KeysE0;
               handled = 1;
-	    } else if (command & BRL_FLG_KBD_EMUL1) {
+	    } else if (*command & BRL_FLG_KBD_EMUL1) {
               at2Keys = at2KeysE1;
             } else if (arg == 0XE1) {
               at2Keys = at2KeysE1;
