@@ -2,13 +2,14 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2008 by The BRLTTY Developers.
+ * Copyright (C) 1995-2011 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
  * This is free software, placed under the terms of the
  * GNU General Public License, as published by the Free Software
- * Foundation.  Please see the file COPYING for details.
+ * Foundation; either version 2 of the License, or (at your option) any
+ * later version. Please see the file LICENSE-GPL for details.
  *
  * Web Page: http://mielke.cc/brltty/
  *
@@ -55,7 +56,8 @@ typedef struct {
 
 #endif /* monitor definitions */
 
-#include "misc.h"
+#include "log.h"
+#include "timing.h"
 #include "queue.h"
 #include "async.h"
 
@@ -126,7 +128,7 @@ awaitOperation (MonitorEntry *monitors, int count, int timeout) {
     if ((result >= WAIT_OBJECT_0) && (result < (WAIT_OBJECT_0 + count))) return 1;
 
     if (result == WAIT_FAILED) {
-      LogWindowsError("WaitForMultipleObjects");
+      logWindowsSystemError("WaitForMultipleObjects");
     }
   } else {
     approximateDelay(timeout);
@@ -146,7 +148,7 @@ testMonitor (const MonitorEntry *monitor) {
   if (result == WAIT_OBJECT_0) return 1;
 
   if (result == WAIT_FAILED) {
-    LogWindowsError("WaitForSingleObject");
+    logWindowsSystemError("WaitForSingleObject");
   }
 
   return 0;
@@ -264,7 +266,7 @@ awaitOperation (MonitorEntry *monitors, int count, int timeout) {
   if (result > 0) return 1;
 
   if (result == -1) {
-    if (errno != EINTR) LogError("poll");
+    if (errno != EINTR) logSystemError("poll");
   }
 
   return 0;
@@ -321,7 +323,7 @@ doSelect (int setSize, fd_set *readSet, fd_set *writeSet, int timeout) {
     if (result > 0) return 1;
 
     if (result == -1) {
-      if (errno != EINTR) LogError("select");
+      if (errno != EINTR) logSystemError("select");
     }
 
     return 0;
@@ -438,9 +440,13 @@ invokeInputCallback (OperationEntry *operation) {
   if (operation->error) return 0;
   if (extension->direction.input.end) return 0;
 
-  if (count)
+  operation->finished = 0;
+  if (count) {
     memmove(extension->buffer, &extension->buffer[count],
             extension->length -= count);
+    if (extension->length > 0) operation->finished = 1;
+  }
+
   return 1;
 }
 
@@ -448,7 +454,10 @@ static int
 invokeOutputCallback (OperationEntry *operation) {
   TransferExtension *extension = operation->extension;
 
-  if (!operation->error && (extension->length < extension->size)) return 1;
+  if (!operation->error && (extension->length < extension->size)) {
+    operation->finished = 0;
+    return 1;
+  }
 
   if (extension->direction.output.callback) {
     AsyncOutputResult result;
@@ -471,7 +480,7 @@ deallocateFunctionEntry (void *item, void *data) {
 }
 
 static int
-testFunctionEntry (void *item, void *data) {
+testFunctionEntry (const void *item, const void *data) {
   const FunctionEntry *function = item;
   const FunctionKey *key = data;
   return (function->fileDescriptor == key->fileDescriptor) &&
@@ -509,7 +518,7 @@ getFunctionElement (FileDescriptor fileDescriptor, const FunctionMethods *method
       key.methods = methods;
 
       {
-        Element *element = processQueue(functions, testFunctionEntry, &key);
+        Element *element = findElement(functions, testFunctionEntry, &key);
         if (element) return element;
       }
     }
@@ -533,6 +542,8 @@ getFunctionElement (FileDescriptor fileDescriptor, const FunctionMethods *method
         }
 
         free(function);
+      } else {
+        logMallocError();
       }
     }
   }
@@ -565,9 +576,8 @@ createOperation (
     if ((functionElement = getFunctionElement(fileDescriptor, methods, 1))) {
       FunctionEntry *function = getElementItem(functionElement);
       int new = !getQueueSize(function->operations);
-      Element *operationElement;
 
-      if ((operationElement = enqueueItem(function->operations, operation))) {
+      if (enqueueItem(function->operations, operation)) {
         operation->function = function;
         operation->extension = extension;
         operation->data = data;
@@ -582,6 +592,8 @@ createOperation (
     }
 
     free(operation);
+  } else {
+    logMallocError();
   }
 
   return 0;
@@ -606,6 +618,8 @@ createTransferOperation (
     if (createOperation(fileDescriptor, methods, extension, data)) return 1;
 
     free(extension);
+  } else {
+    logMallocError();
   }
 
   return 0;
@@ -758,6 +772,8 @@ asyncAbsoluteAlarm (
       if (enqueueItem(alarms, alarm)) return 1;
 
       free(alarm);
+    } else {
+      logMallocError();
     }
   }
 
@@ -854,6 +870,7 @@ asyncWait (int duration) {
           monitorArray = NULL;
         }
       } else {
+        logMallocError();
         monitorCount = 0;
       }
     }
@@ -873,7 +890,6 @@ asyncWait (int duration) {
 
       if (!operation->finished) finishOperation(operation);
       if (function->methods->invokeCallback(operation)) {
-        operation->finished = 0;
         operation->error = 0;
       } else {
         deleteElement(operationElement);
@@ -881,7 +897,7 @@ asyncWait (int duration) {
 
       if ((operationElement = getQueueHead(function->operations))) {
         operation = getElementItem(operationElement);
-        startOperation(operation);
+        if (!operation->finished) startOperation(operation);
         requeueElement(functionElement);
       } else {
         deleteElement(functionElement);
