@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2009 by The BRLTTY Developers.
+ * Copyright (C) 1995-2011 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -23,9 +23,18 @@
 extern "C" {
 #endif /* __cplusplus */
 
-#include <bluetooth/bluetooth.h>
+#define BDA_SIZE 6
 
-extern int btConnect (bdaddr_t address, unsigned char channel);
+typedef struct BluetoothConnectionExtensionStruct BluetoothConnectionExtension;
+
+struct BluetoothConnectionStruct {
+  uint64_t address;
+  uint8_t channel;
+  BluetoothConnectionExtension *extension;
+};
+
+extern BluetoothConnectionExtension *bthConnect (uint64_t bda, uint8_t channel);
+extern void bthDisconnect (BluetoothConnectionExtension *bcx);
 
 #ifdef __cplusplus
 }
