@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2009 by The BRLTTY Developers.
+ * Copyright (C) 1995-2011 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -23,7 +23,8 @@
 #include <errno.h>
 #include <fcntl.h>
 
-#include "misc.h"
+#include "log.h"
+#include "timing.h"
 #include "async.h"
 #include "message.h"
 #include "charset.h"
@@ -31,8 +32,11 @@
 #include "drivers.h"
 #include "brl.h"
 #include "ttb.h"
+#include "ktb.h"
 #include "brl.auto.h"
 #include "cmd.h"
+#include "queue.h"
+#include "brltty.h"
 
 #define BRLSYMBOL noBraille
 #define DRIVER_NAME NoBraille
@@ -40,7 +44,6 @@
 #define DRIVER_COMMENT "no braille support"
 #define DRIVER_VERSION ""
 #define DRIVER_DEVELOPERS ""
-#define BRLHELP "/dev/null"
 #include "brl_driver.h"
 
 static int
@@ -53,7 +56,7 @@ brl_destruct (BrailleDisplay *brl) {
 }
 
 static int
-brl_readCommand (BrailleDisplay *brl, BRL_DriverCommandContext context) {
+brl_readCommand (BrailleDisplay *brl, KeyTableCommandContext context) {
   return EOF;
 }
 
@@ -106,14 +109,19 @@ initializeBrailleDisplay (BrailleDisplay *brl) {
   brl->textRows = 1;
   brl->statusColumns = 0;
   brl->statusRows = 0;
-  brl->helpPage = 0;
+
+  brl->keyBindings = "all";
+  brl->keyNameTables = NULL;
+  brl->keyTable = NULL;
+
   brl->buffer = NULL;
   brl->writeDelay = 0;
   brl->bufferResized = NULL;
-  brl->dataDirectory = NULL;
   brl->touchEnabled = 0;
   brl->highlightWindow = 0;
   brl->data = NULL;
+  brl->setFirmness = NULL;
+  brl->setSensitivity = NULL;
 }
 
 unsigned int
@@ -226,9 +234,9 @@ clearStatusCells (BrailleDisplay *brl) {
 static void
 brailleBufferResized (BrailleDisplay *brl, int infoLevel) {
   memset(brl->buffer, 0, brl->textColumns*brl->textRows);
-  LogPrint(infoLevel, "Braille Display Dimensions: %d %s, %d %s",
-           brl->textRows, (brl->textRows == 1)? "row": "rows",
-           brl->textColumns, (brl->textColumns == 1)? "column": "columns");
+  logMessage(infoLevel, "Braille Display Dimensions: %d %s, %d %s",
+             brl->textRows, (brl->textRows == 1)? "row": "rows",
+             brl->textColumns, (brl->textColumns == 1)? "column": "columns");
   if (brl->bufferResized) brl->bufferResized(infoLevel, brl->textRows, brl->textColumns);
 }
 
@@ -246,7 +254,7 @@ resizeBrailleBuffer (BrailleDisplay *brl, int infoLevel) {
         void *newAddress = malloc(newSize);
 
         if (!newAddress) {
-          LogError("malloc");
+          logMallocError();
           return 0;
         }
 
@@ -274,12 +282,237 @@ ensureBrailleBuffer (BrailleDisplay *brl, int infoLevel) {
   return 1;
 }
 
+typedef struct {
+  int command;
+} CommandQueueItem;
+
+static Queue *
+getCommandQueue (int create) {
+  static Queue *commandQueue = NULL;
+
+  if (create && !commandQueue) {
+    commandQueue = newQueue(NULL, NULL);
+  }
+
+  return commandQueue;
+}
+
 int
-readBrailleCommand (BrailleDisplay *brl, BRL_DriverCommandContext context) {
-  int command = dequeueCommand();
-  if (command == EOF) command = braille->readCommand(brl, context);
-  resizeBrailleBuffer(brl, LOG_INFO);
-  return command;
+enqueueCommand (int command) {
+  if (command != EOF) {
+    Queue *queue = getCommandQueue(1);
+
+    if (queue) {
+      CommandQueueItem *item = malloc(sizeof(CommandQueueItem));
+
+      if (item) {
+        item->command = command;
+        if (enqueueItem(queue, item)) return 1;
+
+        free(item);
+      }
+    }
+  }
+
+  return 0;
+}
+
+static int
+dequeueCommand (void) {
+  Queue *queue = getCommandQueue(0);
+
+  if (queue) {
+    CommandQueueItem *item;
+
+    while ((item = dequeueItem(queue))) {
+      int command = item->command;
+      free(item);
+
+#ifdef ENABLE_API
+      if (apiStarted)
+        if ((command = api_handleCommand(command)) == EOF)
+          continue;
+#endif /* ENABLE_API */
+
+      return command;
+    }
+  }
+
+  return EOF;
+}
+
+typedef struct {
+  unsigned char set;
+  unsigned char key;
+  unsigned press:1;
+} KeyEventQueueItem;
+
+static Queue *
+getKeyEventQueue (int create) {
+  static Queue *keyEventQueue = NULL;
+
+  if (create && !keyEventQueue) {
+    keyEventQueue = newQueue(NULL, NULL);
+  }
+
+  return keyEventQueue;
+}
+
+int
+enqueueKeyEvent (unsigned char set, unsigned char key, int press) {
+  Queue *queue = getKeyEventQueue(1);
+
+  if (queue) {
+    KeyEventQueueItem *item = malloc(sizeof(KeyEventQueueItem));
+
+    if (item) {
+      item->set = set;
+      item->key = key;
+      item->press = press;
+      if (enqueueItem(queue, item)) return 1;
+
+      free(item);
+    }
+  }
+
+  return 0;
+}
+
+static int
+dequeueKeyEvent (unsigned char *set, unsigned char *key, int *press) {
+  Queue *queue = getKeyEventQueue(0);
+
+  if (queue) {
+    KeyEventQueueItem *item;
+
+    while ((item = dequeueItem(queue))) {
+#ifdef ENABLE_API
+      if (apiStarted) {
+        if ((api_handleKeyEvent(item->set, item->key, item->press)) == EOF) {
+          free(item);
+	  continue;
+	}
+      }
+#endif /* ENABLE_API */
+
+      *set = item->set;
+      *key = item->key;
+      *press = item->press;
+      free(item);
+      return 1;
+    }
+  }
+
+  return 0;
+}
+
+int
+enqueueKey (unsigned char set, unsigned char key) {
+  if (enqueueKeyEvent(set, key, 1))
+    if (enqueueKeyEvent(set, key, 0))
+      return 1;
+
+  return 0;
+}
+
+int
+enqueueKeys (uint32_t bits, unsigned char set, unsigned char key) {
+  unsigned char stack[0X20];
+  unsigned char count = 0;
+
+  while (bits) {
+    if (bits & 0X1) {
+      if (!enqueueKeyEvent(set, key, 1)) return 0;
+      stack[count++] = key;
+    }
+
+    bits >>= 1;
+    key += 1;
+  }
+
+  while (count)
+    if (!enqueueKeyEvent(set, stack[--count], 0))
+      return 0;
+
+  return 1;
+}
+
+int
+enqueueUpdatedKeys (uint32_t new, uint32_t *old, unsigned char set, unsigned char key) {
+  uint32_t bit = 0X1;
+  unsigned char stack[0X20];
+  unsigned char count = 0;
+
+  while (*old != new) {
+    if ((new & bit) && !(*old & bit)) {
+      stack[count++] = key;
+      *old |= bit;
+    } else if (!(new & bit) && (*old & bit)) {
+      if (!enqueueKeyEvent(set, key, 0)) return 0;
+      *old &= ~bit;
+    }
+
+    key += 1;
+    bit <<= 1;
+  }
+
+  while (count)
+    if (!enqueueKeyEvent(set, stack[--count], 1))
+      return 0;
+
+  return 1;
+}
+
+int
+enqueueXtScanCode (
+  unsigned char key, unsigned char escape,
+  unsigned char set00, unsigned char setE0, unsigned char setE1
+) {
+  int command = BRL_BLK_PASSXT | key;
+  switch (escape) {
+    case 0XE0:
+      command |= BRL_FLG_KBD_EMUL0;
+      break;
+
+    case 0XE1:
+      command |= BRL_FLG_KBD_EMUL1;
+      break;
+
+    default:
+    case 0X00:
+      break;
+  }
+
+  return enqueueCommand(command);
+}
+
+int
+readBrailleCommand (BrailleDisplay *brl, KeyTableCommandContext context) {
+  {
+    int command = dequeueCommand();
+    if (command != EOF) return command;
+  }
+
+  {
+    int command = braille->readCommand(brl, context);
+    resizeBrailleBuffer(brl, LOG_INFO);
+
+    {
+      unsigned char set;
+      unsigned char key;
+      int press;
+
+      while (dequeueKeyEvent(&set, &key, &press)) {
+        if (brl->keyTable) {
+          processKeyEvent(brl->keyTable, context, set, key, press);
+        }
+      }
+    }
+
+    if (command != EOF) enqueueCommand(command);
+  }
+
+  return dequeueCommand();
 }
 
 #ifdef ENABLE_LEARN_MODE
@@ -288,14 +521,14 @@ learnMode (BrailleDisplay *brl, int poll, int timeout) {
   const char *mode = "lrn";
 
   if (!setStatusText(brl, mode)) return 0;
-  if (!message(mode, "command learn mode", MSG_NODELAY)) return 0;
+  if (!message(mode, gettext("command learn mode"), MSG_NODELAY)) return 0;
 
   hasTimedOut(0);
   do {
-    int command = readBrailleCommand(brl, BRL_CTX_SCREEN);
+    int command = readBrailleCommand(brl, KTB_CTX_DEFAULT);
     if (command != EOF) {
-      LogPrint(LOG_DEBUG, "Learn: command=%06X", command);
-      if (IS_DELAYED_COMMAND(command)) continue;
+      logMessage(LOG_DEBUG, "Learn: command=%06X", command);
+      if (BRL_DELAYED_COMMAND(command)) continue;
 
       {
         int cmd = command & BRL_MSK_CMD;
@@ -305,8 +538,9 @@ learnMode (BrailleDisplay *brl, int poll, int timeout) {
 
       {
         char buffer[0X100];
-        describeCommand(command, buffer, sizeof(buffer));
-        LogPrint(LOG_DEBUG, "Learn: %s", buffer);
+        describeCommand(command, buffer, sizeof(buffer),
+                        CDO_IncludeName | CDO_IncludeOperand);
+        logMessage(LOG_DEBUG, "Learn: %s", buffer);
         if (!message(mode, buffer, MSG_NODELAY|MSG_SILENT)) return 0;
       }
 
@@ -320,22 +554,137 @@ learnMode (BrailleDisplay *brl, int poll, int timeout) {
 }
 #endif /* ENABLE_LEARN_MODE */
 
+int
+cellsHaveChanged (
+  unsigned char *cells, const unsigned char *new, unsigned int count,
+  unsigned int *from, unsigned int *to
+) {
+  if (memcmp(cells, new, count) == 0) return 0;
+
+  {
+    unsigned int first = 0;
+
+    if (to) {
+      while (count) {
+        unsigned int last = count - 1;
+        if (cells[last] != new[last]) break;
+        count = last;
+      }
+
+      *to = count;
+    }
+
+    if (from) {
+      while (first < count) {
+        if (cells[first] != new[first]) break;
+        first += 1;
+      }
+
+      *from = first;
+    }
+
+    memcpy(cells+first, new+first, count-first);
+  }
+
+  return 1;
+}
+
+const DotsTable dotsTable_ISO11548_1 = {
+  BRL_DOT1, BRL_DOT2, BRL_DOT3, BRL_DOT4,
+  BRL_DOT5, BRL_DOT6, BRL_DOT7, BRL_DOT8
+};
+
 void
-reverseTranslationTable (TranslationTable from, TranslationTable to) {
+makeTranslationTable (const DotsTable dots, TranslationTable table) {
+  int byte;
+
+  for (byte=0; byte<TRANSLATION_TABLE_SIZE; byte+=1) {
+    unsigned char cell = 0;
+    int dot;
+
+    for (dot=0; dot<DOTS_TABLE_SIZE; dot+=1)
+      if (byte & dotsTable_ISO11548_1[dot])
+        cell |= dots[dot];
+
+    table[byte] = cell;
+  }
+}
+
+void
+reverseTranslationTable (const TranslationTable from, TranslationTable to) {
   int byte;
   memset(to, 0, sizeof(TranslationTable));
   for (byte=TRANSLATION_TABLE_SIZE-1; byte>=0; byte--) to[from[byte]] = byte;
 }
 
+static inline void *
+translateCells (
+  const TranslationTable table,
+  unsigned char *target, const unsigned char *source, size_t count
+) {
+  if (table) {
+    while (count--) *target++ = table[*source++];
+    return target;
+  }
+
+  if (target == source) return target + count;
+  return mempcpy(target, source, count);
+}
+
+static inline unsigned char
+translateCell (const TranslationTable table, unsigned char cell) {
+  return table? table[cell]: cell;
+}
+
+static TranslationTable internalOutputTable;
+static const unsigned char *outputTable;
+
 void
-makeOutputTable (const DotsTable dots, TranslationTable table) {
-  static const DotsTable internalDots = {BRL_DOT1, BRL_DOT2, BRL_DOT3, BRL_DOT4, BRL_DOT5, BRL_DOT6, BRL_DOT7, BRL_DOT8};
-  int byte, dot;
-  memset(table, 0, sizeof(TranslationTable));
-  for (byte=0; byte<TRANSLATION_TABLE_SIZE; byte++)
-    for (dot=0; dot<DOTS_TABLE_SIZE; dot++)
-      if (byte & internalDots[dot])
-        table[byte] |= dots[dot];
+setOutputTable (const TranslationTable table) {
+  outputTable = table;
+}
+
+void
+makeOutputTable (const DotsTable dots) {
+  if (memcmp(dots, dotsTable_ISO11548_1, DOTS_TABLE_SIZE) == 0) {
+    outputTable = NULL;
+  } else {
+    makeTranslationTable(dots, internalOutputTable);
+    outputTable = internalOutputTable;
+  }
+}
+
+void *
+translateOutputCells (unsigned char *target, const unsigned char *source, size_t count) {
+  return translateCells(outputTable, target, source, count);
+}
+
+unsigned char
+translateOutputCell (unsigned char cell) {
+  return translateCell(outputTable, cell);
+}
+
+static TranslationTable internalInputTable;
+static const unsigned char *inputTable;
+
+void
+makeInputTable (void) {
+  if (outputTable) {
+    reverseTranslationTable(outputTable, internalInputTable);
+    inputTable = internalInputTable;
+  } else {
+    inputTable = NULL;
+  }
+}
+
+void *
+translateInputCells (unsigned char *target, const unsigned char *source, size_t count) {
+  return translateCells(inputTable, target, source, count);
+}
+
+unsigned char
+translateInputCell (unsigned char cell) {
+  return translateCell(inputTable, cell);
 }
 
 /* Functions which support vertical and horizontal status cells. */
@@ -422,14 +771,22 @@ portraitFlag (int number, int on) {
   return dots;
 }
 
-void
+int
 setBrailleFirmness (BrailleDisplay *brl, BrailleFirmness setting) {
-  LogPrint(LOG_DEBUG, "setting braille firmness: %d", setting);
-  braille->firmness(brl, setting);
+  if (brl->setFirmness) {
+    logMessage(LOG_DEBUG, "setting braille firmness: %d", setting);
+    if (brl->setFirmness(brl, setting)) return 1;
+  }
+
+  return 0;
 }
 
-void
+int
 setBrailleSensitivity (BrailleDisplay *brl, BrailleSensitivity setting) {
-  LogPrint(LOG_DEBUG, "setting braille sensitivity: %d", setting);
-  braille->sensitivity(brl, setting);
+  if (brl->setSensitivity) {
+    logMessage(LOG_DEBUG, "setting braille sensitivity: %d", setting);
+    if (brl->setSensitivity(brl, setting)) return 1;
+  }
+
+  return 0;
 }
