@@ -1,7 +1,7 @@
 /*
  * libbrlapi - A library providing access to braille terminals for applications.
  *
- * Copyright (C) 2002-2009 by
+ * Copyright (C) 2002-2011 by
  *   Samuel Thibault <Samuel.Thibault@ens-lyon.org>
  *   Sébastien Hinderer <Sebastien.Hinderer@ens-lyon.org>
  *
@@ -19,6 +19,7 @@
 
 /* api_client.c handles connection with BrlApi */
 
+#define WIN_ERRNO_STORAGE_CLASS static
 #include "prologue.h"
 
 #include <stdio.h>
@@ -27,15 +28,17 @@
 #include <errno.h>
 #include <fcntl.h>
 #include <sys/stat.h>
+#include <locale.h>
+
 #ifndef __MINGW32__
 #include <langinfo.h>
 #endif /* __MINGW32__ */
-#include <locale.h>
 
 #ifdef __MINGW32__
 #include <ws2tcpip.h>
 #include "win_pthread.h"
-#include "misc.h"
+
+#include "win_errno.h"
 
 #define syslog(level,fmt,...) fprintf(stderr,#level ": " fmt, ## __VA_ARGS__)
 
@@ -57,7 +60,6 @@
 #include <sys/time.h>
 #endif /* HAVE_SYS_SELECT_H */
 
-#define setSocketErrno()
 #endif /* __MINGW32__ */
 
 #ifdef HAVE_ALLOCA_H
@@ -133,6 +135,23 @@ static WIN_PROC_STUB(freeaddrinfo);
 #endif /* __MINGW32__ */
 #endif /* WINDOWS */
 
+/* We need to declare these as weak external references to determine at runtime 
+ * whether libpthread is used or not. We also can't rely on the functions prototypes.
+ */
+#if defined(WINDOWS)
+
+#elif defined(__GNUC__) || defined(__sun__)
+#pragma weak pthread_key_create
+#pragma weak pthread_once
+#pragma weak pthread_getspecific
+#pragma weak pthread_setspecific
+#pragma weak sem_init
+#pragma weak sem_post
+#pragma weak sem_wait
+#pragma weak sem_destroy
+
+#endif /* weak external references */
+
 /** key presses buffer size
  *
  * key presses won't be lost provided no more than BRL_KEYBUF_SIZE key presses
@@ -195,16 +214,14 @@ static brlapi_handle_t defaultHandle;
 /* Initialize a BrlAPI handle */
 static void brlapi_initializeHandle(brlapi_handle_t *handle)
 {
-  pthread_mutexattr_t mattr;
-  pthread_mutexattr_init(&mattr);
   handle->brlx = 0;
   handle->brly = 0;
   handle->fileDescriptor = INVALID_FILE_DESCRIPTOR;
   handle->addrfamily = 0;
-  pthread_mutex_init(&handle->fileDescriptor_mutex, &mattr);
-  pthread_mutex_init(&handle->req_mutex, &mattr);
-  pthread_mutex_init(&handle->key_mutex, &mattr);
-  pthread_mutex_init(&handle->read_mutex, &mattr);
+  pthread_mutex_init(&handle->fileDescriptor_mutex, NULL);
+  pthread_mutex_init(&handle->req_mutex, NULL);
+  pthread_mutex_init(&handle->key_mutex, NULL);
+  pthread_mutex_init(&handle->read_mutex, NULL);
   handle->reading = 0;
   handle->altExpectedPacketType = 0;
   handle->altPacket = NULL;
@@ -212,7 +229,7 @@ static void brlapi_initializeHandle(brlapi_handle_t *handle)
   handle->altRes = NULL;
   handle->altSem = NULL;
   handle->state = 0;
-  pthread_mutex_init(&handle->state_mutex, &mattr);
+  pthread_mutex_init(&handle->state_mutex, NULL);
   memset(handle->keybuf, 0, sizeof(handle->keybuf));
   handle->keybuf_next = 0;
   handle->keybuf_nb = 0;
@@ -220,7 +237,7 @@ static void brlapi_initializeHandle(brlapi_handle_t *handle)
     handle->exceptionHandler.withoutHandle = brlapi_defaultExceptionHandler;
   else
     handle->exceptionHandler.withHandle = brlapi__defaultExceptionHandler;
-  pthread_mutex_init(&handle->exceptionHandler_mutex, &mattr);
+  pthread_mutex_init(&handle->exceptionHandler_mutex, NULL);
 }
 
 /* brlapi_doWaitForPacket */
@@ -249,7 +266,10 @@ static ssize_t brlapi__doWaitForPacket(brlapi_handle_t *handle, brlapi_packetTyp
   if (handle->altSem && type==handle->altExpectedPacketType) {
     /* Yes, put packet content there */
     *handle->altRes = res = brlapi_readPacketContent(handle->fileDescriptor, res, handle->altPacket, handle->altSize);
-    sem_post(handle->altSem);
+#ifndef WINDOWS
+    if (sem_post)
+#endif /* WINDOWS */
+      sem_post(handle->altSem);
     handle->altSem = NULL;
     pthread_mutex_unlock(&handle->read_mutex);
     if (res < 0) return res;
@@ -317,7 +337,12 @@ again:
   pthread_mutex_lock(&handle->read_mutex);
   if (!handle->reading) doread = handle->reading = 1;
   else {
-    if (handle->altSem) {
+    if (
+#ifndef WINDOWS
+    	!sem_init || !sem_post || !sem_wait || !sem_destroy ||
+#endif /* WINDOWS */
+	handle->altSem) {
+      /* This can't happen without threads */
       syslog(LOG_ERR,"third call to brlapi_waitForPacket !");
       brlapi_errno = BRLAPI_ERROR_ILLEGAL_INSTRUCTION;
       return -1;
@@ -342,7 +367,10 @@ again:
     pthread_mutex_lock(&handle->read_mutex);
     if (handle->altSem) {
       *handle->altRes = -3; /* no packet for him */
-      sem_post(handle->altSem);
+#ifndef WINDOWS
+      if (sem_post)
+#endif /* WINDOWS */
+        sem_post(handle->altSem);
       handle->altSem = NULL;
     }
     handle->reading = 0;
@@ -612,11 +640,12 @@ brlapi_fileDescriptor BRLAPI_STDCALL brlapi__openConnection(brlapi_handle_t *han
 
   if (tryHost(handle, settings.host)<0) {
     brlapi_error_t error = brlapi_error;
-    if (tryHost(handle, settings.host="127.0.0.1:0")<0
+    if (strrchr(settings.host, ':') != settings.host ||
+	(tryHost(handle, settings.host="127.0.0.1:0")<0
 #ifdef AF_INET6
       && tryHost(handle, settings.host="::1:0")<0
 #endif /* AF_INET6 */
-      ) {
+      )) {
       brlapi_error = error;
       goto out;
     }
@@ -1086,7 +1115,7 @@ static size_t getCharset(unsigned char *buffer)
   if (CHECKPROC("ntdll.dll", wcslen) && wide) {
 #endif /* __CYGWIN32__ */
     *p++ = strlen(WIN_WCHAR_T);
-    strcpy(p, WIN_WCHAR_T);
+    strcpy((char*) p, WIN_WCHAR_T);
     p += strlen(WIN_WCHAR_T);
   } else
 #endif /* WINDOWS */
@@ -1095,7 +1124,7 @@ static size_t getCharset(unsigned char *buffer)
 #ifdef WINDOWS
     UINT CP;
     if ((CP = GetACP() || (CP = GetOEMCP()))) {
-      len = sprintf(p+3, "%d", CP);
+      len = sprintf(p+3, "%u", CP);
       *p++ = 2 + len;
       *p++ = 'C';
       *p++ = 'P';
@@ -1626,7 +1655,7 @@ brlapi_describeKeyCode (brlapi_keyCode_t keyCode, brlapi_describedKeyCode_t *dkc
           case 0:
             FLAG(TOGGLE_ON);
             FLAG(TOGGLE_OFF);
-            FLAG(ROUTE);
+            FLAG(MOTION_ROUTE);
             break;
 
           case BRLAPI_KEY_CMD_GOTOLINE:
@@ -1861,19 +1890,6 @@ static pthread_key_t error_key;
 /* the key must be created at most once */
 static pthread_once_t error_key_once = PTHREAD_ONCE_INIT;
 
-/* We need to declare these as weak external references to determine at runtime 
- * whether libpthread is used or not. We also can't rely on the functions prototypes.
- */
-#if defined(WINDOWS)
-
-#elif defined(__GNUC__) || defined(__sun__)
-#pragma weak pthread_key_create
-#pragma weak pthread_once
-#pragma weak pthread_getspecific
-#pragma weak pthread_setspecific
-
-#endif /* weak external references */
-
 static void error_key_free(void *key)
 {
   free(key);
