@@ -2,13 +2,14 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2008 by The BRLTTY Developers.
+ * Copyright (C) 1995-2011 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
  * This is free software, placed under the terms of the
  * GNU General Public License, as published by the Free Software
- * Foundation.  Please see the file COPYING for details.
+ * Foundation; either version 2 of the License, or (at your option) any
+ * later version. Please see the file LICENSE-GPL for details.
  *
  * Web Page: http://mielke.cc/brltty/
  *
@@ -20,8 +21,10 @@
 /* Source file for range list management module */
 /* For a description of what each function does, see rangelist.h */
 
+#include <stdio.h>
+
 #include "brlapi_keyranges.h"
-#include "misc.h"
+#include "log.h"
 
 static int inKeyrange(KeyrangeList *l, KeyrangeElem e)
 {
@@ -102,7 +105,7 @@ int addKeyrange(KeyrangeElem x0, KeyrangeElem y0, KeyrangeList **l)
   KeyrangeElem min = KeyrangeElem(minFlags, minVal);
   KeyrangeElem max = KeyrangeElem(maxFlags, maxVal);
 
-  LogPrint(LOG_DEBUG, "adding range [%"PRIx32"(%"PRIx32")..%"PRIx32"(%"PRIx32")]", minVal, minFlags, maxVal, maxFlags);
+  logMessage(LOG_DEBUG, "adding range [%"PRIx32"(%"PRIx32")..%"PRIx32"(%"PRIx32")]", minVal, minFlags, maxVal, maxFlags);
 
   c = *l;
   while (c) {
@@ -143,7 +146,7 @@ int removeKeyrange(KeyrangeElem x0, KeyrangeElem y0, KeyrangeList **l)
 
   if ((l==NULL) || (*l==NULL)) return 0;
 
-  LogPrint(LOG_DEBUG, "removing range [%"PRIx32"(%"PRIx32")..%"PRIx32"(%"PRIx32")]", minVal, minFlags, maxVal, maxFlags);
+  logMessage(LOG_DEBUG, "removing range [%"PRIx32"(%"PRIx32")..%"PRIx32"(%"PRIx32")]", minVal, minFlags, maxVal, maxFlags);
 
   /* Need to intersect with every range */
   p = l; c = *p;
@@ -214,7 +217,6 @@ int removeKeyrange(KeyrangeElem x0, KeyrangeElem y0, KeyrangeList **l)
     }
     if (i<32) {
       /* don't intersect any more, keep it */
-      c = c->next;
       break;
     } else {
       /* remaining intersection, drop it */
