@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2010 by The BRLTTY Developers.
+ * Copyright (C) 1995-2011 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -53,14 +53,6 @@ extern "C" {
 #define BRL_ARG_GET(code) (BRL_CODE_GET(ARG, (code)) | (BRL_CODE_GET(EXT, (code)) << BRL_BITS_ARG))
 #define BRL_ARG_SET(value) (BRL_CODE_SET(ARG, (value)) | BRL_CODE_SET(EXT, ((value) >> BRL_BITS_ARG)))
 
-/* Argument for brl_readCommand() */
-typedef enum {
-  BRL_CTX_MENU,
-  BRL_CTX_CHORDS,
-  BRL_CTX_WAITING,
-  BRL_CTX_DEFAULT /* this one must be defined last */
-} BRL_DriverCommandContext;
-
 /* The following command codes are return values for brl_readCommand().
  * The BRL_CMD_* codes are guaranteed to be between 1 and 255
  * inclusive, with the exception of BRL_CMD_NOOP = 0.
@@ -118,22 +110,22 @@ typedef enum {
   
   /* feature activation and deactivation */
   BRL_CMD_FREEZE /* freeze/unfreeze screen image */,
-  BRL_CMD_DISPMD /* toggle display mode attributes/text */,
-  BRL_CMD_SIXDOTS /* toggle text style 6-dot/8-dot */,
-  BRL_CMD_SLIDEWIN /* toggle sliding window on/off */,
-  BRL_CMD_SKPIDLNS /* toggle skipping of lines with identical content on/off */,
-  BRL_CMD_SKPBLNKWINS /* toggle skipping of blank windows on/off */,
-  BRL_CMD_CSRVIS /* toggle cursor visibility on/off */,
-  BRL_CMD_CSRHIDE /* toggle hidden cursor on/off */,
-  BRL_CMD_CSRTRK /* toggle cursor tracking on/off */,
-  BRL_CMD_CSRSIZE /* toggle cursor style block/underline */,
-  BRL_CMD_CSRBLINK /* toggle cursor blinking on/off */,
-  BRL_CMD_ATTRVIS /* toggle attribute underlining on/off */,
-  BRL_CMD_ATTRBLINK /* toggle attribute blinking on/off */,
-  BRL_CMD_CAPBLINK /* toggle capital letter blinking on/off */,
-  BRL_CMD_TUNES /* toggle alert tunes on/off */,
-  BRL_CMD_AUTOREPEAT /* toggle autorepeat on/off */,
-  BRL_CMD_AUTOSPEAK /* toggle autospeak on/off */,
+  BRL_CMD_DISPMD /* set display mode attributes/text */,
+  BRL_CMD_SIXDOTS /* set text style 6-dot/8-dot */,
+  BRL_CMD_SLIDEWIN /* set sliding window on/off */,
+  BRL_CMD_SKPIDLNS /* set skipping of lines with identical content on/off */,
+  BRL_CMD_SKPBLNKWINS /* set skipping of blank windows on/off */,
+  BRL_CMD_CSRVIS /* set cursor visibility on/off */,
+  BRL_CMD_CSRHIDE /* set hidden cursor on/off */,
+  BRL_CMD_CSRTRK /* set cursor tracking on/off */,
+  BRL_CMD_CSRSIZE /* set cursor style block/underline */,
+  BRL_CMD_CSRBLINK /* set cursor blinking on/off */,
+  BRL_CMD_ATTRVIS /* set attribute underlining on/off */,
+  BRL_CMD_ATTRBLINK /* set attribute blinking on/off */,
+  BRL_CMD_CAPBLINK /* set capital letter blinking on/off */,
+  BRL_CMD_TUNES /* set alert tunes on/off */,
+  BRL_CMD_AUTOREPEAT /* set autorepeat on/off */,
+  BRL_CMD_AUTOSPEAK /* set autospeak on/off */,
  
   /* mode selection */
   BRL_CMD_HELP /* enter/leave help display */,
@@ -220,7 +212,7 @@ typedef enum {
 #define BRL_BLK_APNDCHARS 0X1100 /* append characters to cut buffer */
 
 /* For entering a special key. */
-#define BRL_BLK_PASSKEY 0X2000 /* input keyboard key */
+#define BRL_BLK_PASSKEY 0X2000 /* emulate special key */
 typedef enum {
   BRL_KEY_ENTER,
   BRL_KEY_TAB,
@@ -239,8 +231,8 @@ typedef enum {
   BRL_KEY_FUNCTION
 } BRL_Key;
 
-#define BRL_BLK_PASSCHAR 0X2100 /* input unicode character */
-#define BRL_BLK_PASSDOTS 0X2200 /* input braille character */
+#define BRL_BLK_PASSCHAR 0X2100 /* type unicode character */
+#define BRL_BLK_PASSDOTS 0X2200 /* type braille character */
 #define BRL_FLG_CHAR_SHIFT   0X010000 /* shift key pressed */
 #define BRL_FLG_CHAR_UPPER   0X020000 /* convert to uppercase */
 #define BRL_FLG_CHAR_CONTROL 0X040000 /* control key pressed */
@@ -276,53 +268,30 @@ typedef enum {
 #define BRL_DOT8 BRL_ARG_SET(0200) /* lower-right dot of computer braille cell */
 #define BRL_DOTC BRL_ARG_SET(0400) /* space key pressed */
 
-#define BRL_BLK_PASSAT 0X2300 /* input AT (set 2) keyboard scan code */
-#define BRL_BLK_PASSXT 0X2400 /* input XT (set 1) keyboard scan code */
-#define BRL_BLK_PASSPS2 0X2500 /* input PS/2 (set 3) keyboard scan code */
+#define BRL_BLK_PASSAT 0X2300 /* AT (set 2) keyboard scan code */
+#define BRL_BLK_PASSXT 0X2400 /* XT (set 1) keyboard scan code */
+#define BRL_BLK_PASSPS2 0X2500 /* PS/2 (set 3) keyboard scan code */
 #define BRL_FLG_KBD_RELEASE 0X010000 /* it is a release scan code */
 #define BRL_FLG_KBD_EMUL0 0X020000 /* it is an emulation 0 scan code */
 #define BRL_FLG_KBD_EMUL1 0X040000 /* it is an emulation 1 scan code */
 
 #define BRL_BLK_CONTEXT 0X2600 /* switch to command context */
 
-/*
- * Please comment all BRL_GSC_* definitions. They are
- * used during automatic help file generation.
- */
-#define BRL_STATUS_CELLS_GENERIC 0XFF /* must be in BRL_firstStatusCell */
 typedef enum {
-  BRL_firstStatusCell = 0,
+  BRL_FIRMNESS_MINIMUM,
+  BRL_FIRMNESS_LOW,
+  BRL_FIRMNESS_MEDIUM,
+  BRL_FIRMNESS_HIGH,
+  BRL_FIRMNESS_MAXIMUM
+} BrailleFirmness;
 
-  /* numbers */
-  BRL_GSC_BRLCOL /* screen column where left of braille window is */,
-  BRL_GSC_BRLROW /* screen row where top of braille window is */,
-  BRL_GSC_CSRCOL /* screen column where cursor is */,
-  BRL_GSC_CSRROW /* screen row where cursor is */,
-  BRL_GSC_SCRNUM /* virtual screen number */,
-
-  /* flags */
-  BRL_GSC_FREEZE /* frozen screen */,
-  BRL_GSC_DISPMD /* attributes display */,
-  BRL_GSC_SIXDOTS /* six-dot braille */,
-  BRL_GSC_SLIDEWIN /* sliding window */,
-  BRL_GSC_SKPIDLNS /* skip identical lines */,
-  BRL_GSC_SKPBLNKWINS /* skip blank windows */,
-  BRL_GSC_CSRVIS /* visible cursor */,
-  BRL_GSC_CSRHIDE /* hidden cursor */,
-  BRL_GSC_CSRTRK /* cursor tracking */,
-  BRL_GSC_CSRSIZE /* block cursor */,
-  BRL_GSC_CSRBLINK /* blinking cursor */,
-  BRL_GSC_ATTRVIS /* visible attributes underline */,
-  BRL_GSC_ATTRBLINK /* blinking attributes underline */,
-  BRL_GSC_CAPBLINK /* blinking capital letters */,
-  BRL_GSC_TUNES /* alert tunes */,
-  BRL_GSC_HELP /* help mode */,
-  BRL_GSC_INFO /* info mode */,
-  BRL_GSC_AUTOREPEAT /* autorepeat */,
-  BRL_GSC_AUTOSPEAK /* autospeak */,
-
-  BRL_genericStatusCellCount
-} BRL_GenericStatusCell;
+typedef enum {
+  BRL_SENSITIVITY_MINIMUM,
+  BRL_SENSITIVITY_LOW,
+  BRL_SENSITIVITY_MEDIUM,
+  BRL_SENSITIVITY_HIGH,
+  BRL_SENSITIVITY_MAXIMUM
+} BrailleSensitivity;
 
 #ifdef __cplusplus
 }
