@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2010 by The BRLTTY Developers.
+ * Copyright (C) 1995-2011 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -59,8 +59,8 @@ ret convert##name (from f) { \
     to t; \
     to *tp = &t; \
     size_t ts = sizeof(t); \
-    if (iconv(iconv##name, (void *)&fp, &fs, (void *)&tp, &ts) != -1) return t; \
-    LogPrint(LOG_DEBUG, "iconv (" #from " -> " #to ") error: %s", strerror(errno)); \
+    if (iconv(iconv##name, (void *)&fp, &fs, (void *)&tp, &ts) != (size_t)-1) return t; \
+    logMessage(LOG_DEBUG, "iconv (" #from " -> " #to ") error: %s", strerror(errno)); \
   } \
   return eof; \
 }
@@ -82,7 +82,7 @@ convertCharToWchar (char c) {
   int result = MultiByteToWideChar(CHARSET_WINDOWS_CODEPAGE, MB_ERR_INVALID_CHARS,
                                    &c, 1,  &wc, 1);
   if (result) return wc;
-  LogWindowsError("MultiByteToWideChar[" STRINGIFY(CHARSET_WINDOWS_CODEPAGE) "]");
+  logWindowsSystemError("MultiByteToWideChar[" STRINGIFY(CHARSET_WINDOWS_CODEPAGE) "]");
   return WEOF;
 }
 
@@ -93,7 +93,7 @@ convertWcharToChar (wchar_t wc) {
                                    &wc, 1, &c, 1,
                                    NULL, NULL);
   if (result) return c;
-  LogWindowsError("WideCharToMultiByte[" STRINGIFY(CHARSET_WINDOWS_CODEPAGE) "]");
+  logWindowsSystemError("WideCharToMultiByte[" STRINGIFY(CHARSET_WINDOWS_CODEPAGE) "]");
   return EOF;
 }
 
@@ -306,14 +306,9 @@ convertWcharToChar (wchar_t wc) {
 }
 #endif /* conversions */
 
-void
-convertCharsToWchars (const char *c, wchar_t *wc, size_t count) {
-  while (count > 0) {
-    wint_t wi = convertCharToWchar(*c++);
-    if (wi == WEOF) wi = UNICODE_REPLACEMENT_CHARACTER;
-    *wc++ = wi;
-    count -= 1;
-  }
+size_t
+getTextLength (const char *text) {
+  return convertTextToWchars(NULL, text, 0);
 }
 
 size_t
@@ -349,7 +344,7 @@ convertWcharToUtf8 (wchar_t wc, Utf8Buffer utf8) {
 
 wint_t
 convertUtf8ToWchar (const char **utf8, size_t *utfs) {
-  wint_t wc = WEOF;
+  uint32_t character = UINT32_MAX;
   int state = 0;
 
   while (*utfs) {
@@ -357,14 +352,14 @@ convertUtf8ToWchar (const char **utf8, size_t *utfs) {
     (*utfs)--;
 
     if (!(byte & 0X80)) {
-      if (wc != WEOF) goto truncated;
-      wc = byte;
+      if (character != UINT32_MAX) goto truncated;
+      character = byte;
       break;
     }
 
     if (!(byte & 0X40)) {
-      if (wc == WEOF) break;
-      wc = (wc << 6) | (byte & 0X3F);
+      if (character == UINT32_MAX) break;
+      character = (character << 6) | (byte & 0X3F);
       if (!--state) break;
     } else {
       if (!(byte & 0X20)) {
@@ -381,32 +376,50 @@ convertUtf8ToWchar (const char **utf8, size_t *utfs) {
         state = 0;
       }
 
-      if (wc != WEOF) goto truncated;
+      if (character != UINT32_MAX) goto truncated;
 
       if (!state) {
-        wc = WEOF;
+        character = UINT32_MAX;
         break;
       }
 
-      wc = byte & ((1 << (6 - state)) - 1);
+      character = byte & ((1 << (6 - state)) - 1);
     }
   }
 
   while (*utfs) {
     if ((**utf8 & 0XC0) != 0X80) break;
     (*utf8)++, (*utfs)--;
-    wc = WEOF;
+    character = UINT32_MAX;
   }
 
-  return wc;
+  if (character == UINT32_MAX) goto error;
+  if (character > WCHAR_MAX) character = UNICODE_REPLACEMENT_CHARACTER;
+  return character;
 
 truncated:
   (*utf8)--, (*utfs)++;
+error:
   return WEOF;
 }
 
+size_t
+getUtf8Length (const char *utf8) {
+  size_t length = 0;
+
+  while (*utf8) {
+    size_t utfs = UTF8_LEN_MAX;
+    wint_t character = convertUtf8ToWchar(&utf8, &utfs);
+
+    if (character == WEOF) break;
+    length += 1;
+  }
+
+  return length;
+}
+
 void
-convertStringToWchars (const char **utf8, wchar_t **characters, size_t count) {
+convertUtf8ToWchars (const char **utf8, wchar_t **characters, size_t count) {
   while (**utf8 && (count > 1)) {
     size_t utfs = UTF8_LEN_MAX;
     wint_t character = convertUtf8ToWchar(utf8, &utfs);
@@ -505,9 +518,9 @@ setCharset (const char *name) {
     } ConvEntry;
 
     ConvEntry convTable[] = {
-      {&iconvCharToWchar, charset, wcharCharset, 0},
-      {&iconvWcharToChar, wcharCharset, charset, 0},
-      {NULL, NULL, NULL, 1}
+      {&iconvCharToWchar, charset, wcharCharset, 0, CHARSET_ICONV_NULL},
+      {&iconvWcharToChar, wcharCharset, charset, 0, CHARSET_ICONV_NULL},
+      {NULL, NULL, NULL, 1, CHARSET_ICONV_NULL}
     };
     ConvEntry *conv = convTable;
 
@@ -515,7 +528,7 @@ setCharset (const char *name) {
       if (conv->permanent && (*conv->handle != CHARSET_ICONV_NULL)) {
         conv->newHandle = CHARSET_ICONV_NULL;
       } else if ((conv->newHandle = iconv_open(conv->toCharset, conv->fromCharset)) == CHARSET_ICONV_NULL) {
-        LogError("iconv_open");
+        logSystemError("iconv_open");
         while (conv != convTable)
           if ((--conv)->newHandle != CHARSET_ICONV_NULL)
             iconv_close(conv->newHandle);
