@@ -2,13 +2,14 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2007 by The BRLTTY Developers.
+ * Copyright (C) 1995-2011 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
  * This is free software, placed under the terms of the
  * GNU General Public License, as published by the Free Software
- * Foundation.  Please see the file COPYING for details.
+ * Foundation; either version 2 of the License, or (at your option) any
+ * later version. Please see the file LICENSE-GPL for details.
  *
  * Web Page: http://mielke.cc/brltty/
  *
@@ -22,27 +23,32 @@
 extern "C" {
 #endif /* __cplusplus */
 
-#include <wchar.h>
-#include <limits.h>
-
 #include "lock.h"
 
 extern const char *setCharset (const char *name);
 extern const char *getCharset (void);
 
+extern const char *getLocaleCharset (void);
 extern const char *getWcharCharset (void);
 
-typedef char Utf8Buffer[MB_LEN_MAX+1];
+#define UTF8_SIZE(bits) (((bits) < 8)? 1: (((bits) + 3) / 5))
+#define UTF8_LEN_MAX UTF8_SIZE(32)
+typedef char Utf8Buffer[UTF8_LEN_MAX + 1];
 
-extern int convertCharToUtf8 (char c, Utf8Buffer utf8);
-extern int convertUtf8ToChar (char **utf8, size_t *utfs);
+extern size_t convertCharToUtf8 (char c, Utf8Buffer utf8);
+extern int convertUtf8ToChar (const char **utf8, size_t *utfs);
 
-extern int convertWcharToUtf8 (wchar_t wc, Utf8Buffer utf8);
-extern wint_t convertUtf8ToWchar (char **utf8, size_t *utfs);
+extern size_t convertWcharToUtf8 (wchar_t wc, Utf8Buffer utf8);
+extern wint_t convertUtf8ToWchar (const char **utf8, size_t *utfs);
+
+extern size_t getUtf8Length (const char *utf8);
+extern void convertUtf8ToWchars (const char **utf8, wchar_t **characters, size_t count);
 
 extern wint_t convertCharToWchar (char c);
 extern int convertWcharToChar (wchar_t wc);
 
+extern size_t getTextLength (const char *text);
+
 extern int lockCharset (LockOptions options);
 extern void unlockCharset (void);
 
