@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2010 by The BRLTTY Developers.
+ * Copyright (C) 1995-2011 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -19,15 +19,14 @@
 #ifndef BRLTTY_INCLUDED_CTB
 #define BRLTTY_INCLUDED_CTB
 
+#include "ctbdefs.h"
+
 #ifdef __cplusplus
 extern "C" {
 #endif /* __cplusplus */
 
 typedef struct ContractionTableStruct ContractionTable;
 
-#define CTB_NO_OFFSET -1
-#define CTB_NO_CURSOR -1
-
 extern ContractionTable *compileContractionTable (const char *fileName);
 extern void destroyContractionTable (ContractionTable *table);
 extern int contractText (
