@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2010 by The BRLTTY Developers.
+ * Copyright (C) 1995-2011 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -26,7 +26,9 @@
 
 #include "ctb.h"
 #include "ctb_internal.h"
+#include "prefs.h"
 #include "unicode.h"
+#include "ascii.h"
 #include "brldots.h"
 
 static ContractionTable *table;
@@ -234,33 +236,44 @@ selectRule (int length) {
       setAfter(currentFindLength);
 
       if (!maximumLength) {
-        typedef enum {CS_Any, CS_Lower, CS_UpperSingle, CS_UpperMultiple} CapitalizationState;
-#define STATE(c) (testCharacter((c), CTC_UpperCase)? CS_UpperSingle: testCharacter((c), CTC_LowerCase)? CS_Lower: CS_Any)
-        CapitalizationState current = STATE(before);
-        int i;
         maximumLength = currentFindLength;
 
-        for (i=0; i<currentFindLength; ++i) {
-          wchar_t character = src[i];
-          CapitalizationState next = STATE(character);
+        if (prefs.capitalizationMode != CTB_CAP_NONE) {
+          typedef enum {CS_Any, CS_Lower, CS_UpperSingle, CS_UpperMultiple} CapitalizationState;
+#define STATE(c) (testCharacter((c), CTC_UpperCase)? CS_UpperSingle: testCharacter((c), CTC_LowerCase)? CS_Lower: CS_Any)
 
-          if ((i > 0) &&
-              (((current == CS_Lower) && (next == CS_UpperSingle)) ||
-               ((current == CS_UpperMultiple) && (next == CS_Lower)))) {
-            maximumLength = i;
-            break;
-          }
+          CapitalizationState current = STATE(before);
+          int i;
 
-          if ((current > CS_Lower) && (next == CS_UpperSingle)) {
-            current = CS_UpperMultiple;
-          } else if (next != CS_Any) {
-            current = next;
-          } else if (current == CS_Any) {
-            current = CS_Lower;
+          for (i=0; i<currentFindLength; i+=1) {
+            wchar_t character = src[i];
+            CapitalizationState next = STATE(character);
+
+            if (i > 0) {
+              if (((current == CS_Lower) && (next == CS_UpperSingle)) ||
+                  ((current == CS_UpperMultiple) && (next == CS_Lower))) {
+                maximumLength = i;
+                break;
+              }
+
+              if ((prefs.capitalizationMode != CTB_CAP_SIGN) &&
+                  (next == CS_UpperSingle)) {
+                maximumLength = i;
+                break;
+              }
+            }
+
+            if ((prefs.capitalizationMode == CTB_CAP_SIGN) && (current > CS_Lower) && (next == CS_UpperSingle)) {
+              current = CS_UpperMultiple;
+            } else if (next != CS_Any) {
+              current = next;
+            } else if (current == CS_Any) {
+              current = CS_Lower;
+            }
           }
-        }
 
 #undef STATE
+        }
       }
 
       if ((currentFindLength <= maximumLength) &&
@@ -402,8 +415,17 @@ putCell (BYTE byte) {
 }
 
 static int
-putReplace (const ContractionTableRule *rule) {
-  return putCells((BYTE *)&rule->findrep[rule->findlen], rule->replen);
+putReplace (const ContractionTableRule *rule, wchar_t character) {
+  const BYTE *cells = (BYTE *)&rule->findrep[rule->findlen];
+  int count = rule->replen;
+
+  if ((prefs.capitalizationMode == CTB_CAP_DOT7) &&
+      testCharacter(character, CTC_UpperCase)) {
+    if (!putCell(*cells++ | BRL_DOT7)) return 0;
+    if (!(count -= 1)) return 1;
+  }
+
+  return putCells(cells, count);
 }
 
 static const ContractionTableRule *
@@ -419,23 +441,46 @@ getAlwaysRule (wchar_t character) {
   return NULL;
 }
 
+typedef struct {
+  const ContractionTableRule *rule;
+} SetCharacterRuleData;
+
+static int
+setCharacterRule (wchar_t character, void *data) {
+  const ContractionTableRule *rule = getAlwaysRule(character);
+
+  if (rule) {
+    SetCharacterRuleData *scr = data;
+    scr->rule = rule;
+    return 1;
+  }
+
+  return 0;
+}
+
 static int
 putCharacter (wchar_t character) {
   {
-    const ContractionTableRule *rule = getAlwaysRule(character);
-    if (rule) return putReplace(rule);
+    SetCharacterRuleData scr = {
+      .rule = NULL
+    };
+
+    if (handleBestCharacter(character, setCharacterRule, &scr)) {
+      return putReplace(scr.rule, character);
+    }
   }
 
   {
 #ifdef HAVE_WCHAR_H 
-    const wchar_t substitute = UNICODE_REPLACEMENT_CHARACTER;
+    const wchar_t replacementCharacter = UNICODE_REPLACEMENT_CHARACTER;
+    if (getCharacterWidth(character) == 0) return 1;
 #else /* HAVE_WCHAR_H */
-    const wchar_t substitute = 0X1A; /* ASCII SUB (CTRL-Z) */
+    const wchar_t replacementCharacter = SUB;
 #endif /* HAVE_WCHAR_H */
 
-    if (character != substitute) {
-      const ContractionTableRule *rule = getAlwaysRule(substitute);
-      if (rule) return putReplace(rule);
+    if (replacementCharacter != character) {
+      const ContractionTableRule *rule = getAlwaysRule(replacementCharacter);
+      if (rule) return putReplace(rule, character);
     }
   }
 
@@ -900,7 +945,7 @@ contractText (
     if ((!literal && selectRule(srcmax-src)) || selectRule(1)) {
       if (!literal &&
           ((currentOpcode == CTO_Literal) ||
-           ((cursor >= src) && (cursor < (src + currentFindLength))))) {
+           (prefs.expandCurrentWord && (cursor >= src) && (cursor < (src + currentFindLength))))) {
         literal = src + currentFindLength;
 
         if (!testCharacter(*src, CTC_Space)) {
@@ -933,19 +978,21 @@ contractText (
         }
       }
 
-      if (testCharacter(*src, CTC_UpperCase)) {
-        if (!testCharacter(before, CTC_UpperCase)) {
-          if (table->header.fields->beginCapitalSign &&
-              (src + 1 < srcmax) && testCharacter(src[1], CTC_UpperCase)) {
-            if (!putSequence(table->header.fields->beginCapitalSign)) break;
-          } else if (table->header.fields->capitalSign) {
-            if (!putSequence(table->header.fields->capitalSign)) break;
+      if (prefs.capitalizationMode == CTB_CAP_SIGN) {
+        if (testCharacter(*src, CTC_UpperCase)) {
+          if (!testCharacter(before, CTC_UpperCase)) {
+            if (table->header.fields->beginCapitalSign &&
+                (src + 1 < srcmax) && testCharacter(src[1], CTC_UpperCase)) {
+              if (!putSequence(table->header.fields->beginCapitalSign)) break;
+            } else if (table->header.fields->capitalSign) {
+              if (!putSequence(table->header.fields->capitalSign)) break;
+            }
+          }
+        } else if (testCharacter(*src, CTC_LowerCase)) {
+          if (table->header.fields->endCapitalSign && (src - 2 >= srcmin) &&
+              testCharacter(src[-1], CTC_UpperCase) && testCharacter(src[-2], CTC_UpperCase)) {
+            if (!putSequence(table->header.fields->endCapitalSign)) break;
           }
-        }
-      } else if (testCharacter(*src, CTC_LowerCase)) {
-        if (table->header.fields->endCapitalSign && (src - 2 >= srcmin) &&
-            testCharacter(src[-1], CTC_UpperCase) && testCharacter(src[-2], CTC_UpperCase)) {
-          if (!putSequence(table->header.fields->endCapitalSign)) break;
         }
       }
 
@@ -975,7 +1022,7 @@ contractText (
       if (currentRule->replen &&
           !((currentOpcode == CTO_Always) && (currentFindLength == 1))) {
         const wchar_t *srcnxt = src + currentFindLength;
-        if (!putReplace(currentRule)) goto done;
+        if (!putReplace(currentRule, *src)) goto done;
         while (++src != srcnxt) clearOffset();
       } else {
         const wchar_t *srclim = src + currentFindLength;
