@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2010 by The BRLTTY Developers.
+ * Copyright (C) 1995-2011 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -36,7 +36,7 @@ extern ssize_t bthReadData (
 );
 extern ssize_t bthWriteData (BluetoothConnection *conection, const void *buffer, size_t size);
 
-extern int isBluetoothDevice (const char **path);
+extern int isBluetoothDevice (const char **identifier);
 
 #ifdef __cplusplus
 }
