@@ -2,13 +2,14 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2008 by The BRLTTY Developers.
+ * Copyright (C) 1995-2011 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
  * This is free software, placed under the terms of the
  * GNU General Public License, as published by the Free Software
- * Foundation.  Please see the file COPYING for details.
+ * Foundation; either version 2 of the License, or (at your option) any
+ * later version. Please see the file LICENSE-GPL for details.
  *
  * Web Page: http://mielke.cc/brltty/
  *
@@ -31,7 +32,7 @@
 #endif /* HAVE_SYS_SELECT_H */
 
 #include "io_misc.h"
-#include "misc.h"
+#include "log.h"
 
 #ifdef __MSDOS__
 #include "sys_msdos.h"
@@ -63,7 +64,7 @@ awaitFileDescriptor (int fileDescriptor, int milliseconds, int output) {
 
         if (result == -1) {
           if (errno == EINTR) continue;
-          LogError("poll");
+          logSystemError("poll");
           return 0;
         }
 
@@ -90,7 +91,7 @@ awaitFileDescriptor (int fileDescriptor, int milliseconds, int output) {
 
         if (result == -1) {
           if (errno == EINTR) continue;
-          LogError("select");
+          logSystemError("select");
           return 0;
         }
 
@@ -107,8 +108,8 @@ awaitFileDescriptor (int fileDescriptor, int milliseconds, int output) {
 #endif /* __MSDOS__ */
 
     if (milliseconds > 0)
-      LogPrint(LOG_DEBUG, "timeout after %d %s",
-               milliseconds, ((milliseconds == 1)? "millisecond": "milliseconds"));
+      logMessage(LOG_DEBUG, "timeout after %d %s",
+                 milliseconds, ((milliseconds == 1)? "millisecond": "milliseconds"));
 
     errno = EAGAIN;
     return 0;
@@ -152,7 +153,7 @@ readChunk (
       if (errno == EWOULDBLOCK) goto noInput;
 #endif /* EWOULDBLOCK */
 
-      LogError("read");
+      logSystemError("read");
       return 0;
     }
 
@@ -161,7 +162,7 @@ readChunk (
     noInput:
       if ((timeout = *offset? subsequentTimeout: initialTimeout)) {
         if (awaitInput(fileDescriptor, timeout)) continue;
-        LogPrint(LOG_WARNING, "input byte missing at offset %d", (int)*offset);
+        logMessage(LOG_WARNING, "input byte missing at offset %d", (int)*offset);
 #ifdef __MSDOS__
       } else if (!tried) {
         if (awaitInput(fileDescriptor, 0)) continue;
@@ -208,7 +209,7 @@ canWrite:
 #ifdef EWOULDBLOCK
       if (errno == EWOULDBLOCK) goto noOutput;
 #endif /* EWOULDBLOCK */
-      LogError("Write");
+      logSystemError("Write");
       return count;
     }
 
@@ -244,10 +245,10 @@ changeOpenFlags (int fileDescriptor, int flagsToClear, int flagsToSet) {
     if (fcntl(fileDescriptor, F_SETFL, flags) != -1) {
       return 1;
     } else {
-      LogError("F_SETFL");
+      logSystemError("F_SETFL");
     }
   } else {
-    LogError("F_GETFL");
+    logSystemError("F_GETFL");
   }
 #else /* defined(F_GETFL) && defined(F_SETFL) */
   errno = ENOSYS;
