@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2009 by The BRLTTY Developers.
+ * Copyright (C) 1995-2011 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -19,23 +19,24 @@
 #ifndef BRLTTY_INCLUDED_KTB
 #define BRLTTY_INCLUDED_KTB
 
+#include "ktbdefs.h"
+
 #ifdef __cplusplus
 extern "C" {
 #endif /* __cplusplus */
 
-#include "keycodes.h"
-
-typedef struct KeyTableStruct KeyTable;
-
-extern KeyTable *keyTable;
-
-extern KeyTable *compileKeyTable (const char *name);
+extern KeyTable *compileKeyTable (const char *name, KEY_NAME_TABLES_REFERENCE keys);
 extern void destroyKeyTable (KeyTable *table);
 
+typedef int KeyTableListHandler (const wchar_t *line, void *data);
+extern int listKeyTable (KeyTable *table, KeyTableListHandler handleLine, void *data);
+extern int listKeyNames (KEY_NAME_TABLES_REFERENCE keys, KeyTableListHandler handleLine, void *data);
+
 extern char *ensureKeyTableExtension (const char *path);
 
-extern int getKeyCommand (KeyTable *table, const KeyCodeSet *modifiers, KeyCode code);
-extern int isKeyModifiers (KeyTable *table, const KeyCodeSet *modifiers);
+extern void resetKeyTable (KeyTable *table);
+extern KeyTableState processKeyEvent (KeyTable *table, unsigned char context, unsigned char set, unsigned char key, int press);
+extern void logKeyEvents (KeyTable *table);
 
 #ifdef __cplusplus
 }
