@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2010 by The BRLTTY Developers.
+ * Copyright (C) 1995-2011 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -33,7 +33,7 @@ getKeyContext (KeyTable *table, unsigned char context) {
 
 static inline int
 isTemporaryKeyContext (const KeyTable *table, const KeyContext *ctx) {
-  return ((ctx - table->keyContextTable) > BRL_CTX_DEFAULT) && !ctx->title;
+  return ((ctx - table->keyContextTable) > KTB_CTX_DEFAULT) && !ctx->title;
 }
 
 #ifdef __cplusplus
