@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2009 by The BRLTTY Developers.
+ * Copyright (C) 1995-2011 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -26,7 +26,7 @@ extern "C" {
 /* message.h - send a message to Braille and speech */
 
 /* Prototype: */
-extern int message (const char *mode, const char *string, short flags);
+extern int message (const char *mode, const char *text, short flags);
 
 
 /* Flags for the second argument: */
