@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2009 by The BRLTTY Developers.
+ * Copyright (C) 1995-2011 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -29,16 +29,19 @@
 
 #include <string.h>
 
-#include "misc.h"
+#include "log.h"
 #include "system.h"
 #include "drivers.h"
+#include "brltty.h"
 #include "scr.h"
-#include "scr_frozen.h"
 #include "scr_help.h"
+#include "scr_menu.h"
+#include "scr_frozen.h"
 #include "scr_real.h"
 #include "scr.auto.h"
 
 static HelpScreen helpScreen;
+static MenuScreen menuScreen;
 static FrozenScreen frozenScreen;                
 static MainScreen mainScreen;
 static BaseScreen *currentScreen = &mainScreen.base;
@@ -98,8 +101,8 @@ constructScreenDriver (char **parameters) {
     if (mainScreen.construct()) {
       return 1;
     } else {
-      LogPrint(LOG_DEBUG, "screen driver initialization failed: %s",
-               screen->definition.code);
+      logMessage(LOG_DEBUG, "screen driver initialization failed: %s",
+                 screen->definition.code);
     }
   }
 
@@ -128,19 +131,22 @@ identifyScreenDrivers (int full) {
 void
 constructSpecialScreens (void) {
   initializeHelpScreen(&helpScreen);
+  initializeMenuScreen(&menuScreen);
   initializeFrozenScreen(&frozenScreen);
 }
 
 void
 destructSpecialScreens (void) {
-  frozenScreen.destruct();
   helpScreen.destruct();
+  menuScreen.destruct();
+  frozenScreen.destruct();
 }
 
 
 typedef enum {
   SCR_HELP   = 0X01,
-  SCR_FROZEN = 0X02
+  SCR_MENU   = 0X02,
+  SCR_FROZEN = 0X04
 } ActiveScreen;
 static ActiveScreen activeScreens = 0;
 
@@ -150,8 +156,10 @@ selectScreen (void) {
     ActiveScreen which;
     BaseScreen *screen;
   } ScreenEntry;
+
   static const ScreenEntry screenEntries[] = {
     {SCR_HELP  , &helpScreen.base},
+    {SCR_MENU  , &menuScreen.base},
     {SCR_FROZEN, &frozenScreen.base},
     {0         , &mainScreen.base}
   };
@@ -159,12 +167,17 @@ selectScreen (void) {
 
   while (entry->which) {
     if (entry->which & activeScreens) break;
-    ++entry;
+    entry += 1;
   }
 
   currentScreen = entry->screen;
 }
 
+int
+haveScreen (ActiveScreen which) {
+  return (activeScreens & which) != 0;
+}
+
 static void
 activateScreen (ActiveScreen which) {
   activeScreens |= which;
@@ -193,14 +206,14 @@ validateScreenBox (const ScreenBox *box, int columns, int rows) {
             if (((box->top + box->height) <= rows))
               return 1;
 
-  LogPrint(LOG_ERR, "invalid screen area: cols=%d left=%d width=%d rows=%d top=%d height=%d",
-           columns, box->left, box->width,
-           rows, box->top, box->height);
+  logMessage(LOG_ERR, "invalid screen area: cols=%d left=%d width=%d rows=%d top=%d height=%d",
+             columns, box->left, box->width,
+             rows, box->top, box->height);
   return 0;
 }
 
 void
-setScreenCharacterText (ScreenCharacter *characters, char text, size_t count) {
+setScreenCharacterText (ScreenCharacter *characters, wchar_t text, size_t count) {
   while (count > 0) {
     characters[--count].text = text;
   }
@@ -223,20 +236,19 @@ setScreenCharacterAttributes (ScreenCharacter *characters, unsigned char attribu
 
 void
 clearScreenCharacters (ScreenCharacter *characters, size_t count) {
-  setScreenCharacterText(characters, ' ', count);
-  setScreenCharacterAttributes(characters, 0X07, count);
+  setScreenCharacterText(characters, WC_C(' '), count);
+  setScreenCharacterAttributes(characters, SCR_COLOUR_DEFAULT, count);
 }
 
 void
 setScreenMessage (const ScreenBox *box, ScreenCharacter *buffer, const char *message) {
   int count = box->width * box->height;
+  size_t length = strlen(message);
 
-  {
-    size_t length = strlen(message) - box->left;
-    if (length < 0) length = 0;
-    if (length > box->width) length = box->width;
+  clearScreenCharacters(buffer, count);
 
-    clearScreenCharacters(buffer, count);
+  if (length > box->left) {
+    if ((length -= box->left) > box->width) length = box->width;
     if (length) copyScreenCharacterText(buffer, &message[box->left], length);
   }
 }
@@ -316,10 +328,15 @@ userVirtualTerminal (int number) {
 }
 
 int
-executeScreenCommand (int command) {
+executeScreenCommand (int *command) {
   return currentScreen->executeCommand(command);
 }
 
+KeyTableCommandContext
+getScreenCommandContext (void) {
+  return currentScreen->getCommandContext();
+}
+
 
 int
 constructRoutingScreen (void) {
@@ -337,7 +354,7 @@ destructRoutingScreen (void) {
 }
 
 
-static int helpConstructed = 0;
+static int helpScreenConstructed = 0;
 
 int
 isHelpScreen (void) {
@@ -346,12 +363,12 @@ isHelpScreen (void) {
 
 int
 haveHelpScreen (void) {
-  return (activeScreens & SCR_HELP) != 0;
+  return haveScreen(SCR_HELP);
 }
 
 int
 activateHelpScreen (void) {
-  if (!helpConstructed) return 0;
+  if (!helpScreenConstructed) return 0;
   activateScreen(SCR_HELP);
   return 1;
 }
@@ -362,31 +379,58 @@ deactivateHelpScreen (void) {
 }
 
 int
-constructHelpScreen (const char *file) {
-  return helpConstructed = helpScreen.construct(file);
+constructHelpScreen (void) {
+  return (helpScreenConstructed = helpScreen.construct());
 }
 
 void
 destructHelpScreen (void) {
-  if (helpConstructed) {
+  if (helpScreenConstructed) {
     helpScreen.destruct();
-    helpConstructed = 0;
+    helpScreenConstructed = 0;
   }
 }
 
+int
+addHelpLine (const wchar_t *line) {
+  return helpScreen.addLine(line);
+}
+
+unsigned int
+getHelpLineCount (void) {
+  return helpScreen.getLineCount();
+}
+
+
+static int menuScreenConstructed = 0;
+
+int
+isMenuScreen (void) {
+  return currentScreen == &menuScreen.base;
+}
+
+int
+haveMenuScreen (void) {
+  return haveScreen(SCR_MENU);
+}
+
+int
+activateMenuScreen (void) {
+  if (!menuScreenConstructed) {
+    Menu *menu = getPreferencesMenu();
+
+    if (!menu) return 0;
+    if (!menuScreen.construct(menu)) return 0;
+    menuScreenConstructed = 1;
+  }
+
+  activateScreen(SCR_MENU);
+  return 1;
+}
+
 void
-setHelpPageNumber (short page) {
-  helpScreen.setPageNumber(page);
-}
-
-short
-getHelpPageNumber (void) {
-  return helpScreen.getPageNumber();
-}
-
-short
-getHelpPageCount (void) {
-  return helpScreen.getPageCount();
+deactivateMenuScreen (void) {
+  deactivateScreen(SCR_MENU);
 }
 
 
@@ -397,7 +441,7 @@ isFrozenScreen (void) {
 
 int
 haveFrozenScreen (void) {
-  return (activeScreens & SCR_FROZEN) != 0;
+  return haveScreen(SCR_FROZEN);
 }
 
 int
