@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2010 by The BRLTTY Developers.
+ * Copyright (C) 1995-2011 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -19,6 +19,8 @@
 #ifndef BRLTTY_INCLUDED_SCR_BASE
 #define BRLTTY_INCLUDED_SCR_BASE
 
+#include "ktbdefs.h"
+
 #ifdef __cplusplus
 extern "C" {
 #endif /* __cplusplus */
@@ -37,7 +39,8 @@ typedef struct {
   int (*selectVirtualTerminal) (int vt);
   int (*switchVirtualTerminal) (int vt);
   int (*currentVirtualTerminal) (void);
-  int (*executeCommand) (int command);
+  int (*executeCommand) (int *command);
+  KeyTableCommandContext (*getCommandContext) (void);
 } BaseScreen;
 
 extern void initializeBaseScreen (BaseScreen *);
