@@ -2,13 +2,14 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2008 by The BRLTTY Developers.
+ * Copyright (C) 1995-2011 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
  * This is free software, placed under the terms of the
  * GNU General Public License, as published by the Free Software
- * Foundation.  Please see the file COPYING for details.
+ * Foundation; either version 2 of the License, or (at your option) any
+ * later version. Please see the file LICENSE-GPL for details.
  *
  * Web Page: http://mielke.cc/brltty/
  *
@@ -20,7 +21,7 @@
 #include <stdio.h>
 #include <string.h>
 
-#include "misc.h"
+#include "log.h"
 #include "scr.h"
 #include "scr_frozen.h"
 
@@ -41,7 +42,7 @@ construct_FrozenScreen (BaseScreen *source) {
     free(screenCharacters);
     screenCharacters = NULL;
   } else {
-    LogError("malloc");
+    logMallocError();
   }
   return 0;
 }
