@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2010 by The BRLTTY Developers.
+ * Copyright (C) 1995-2011 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -50,7 +50,7 @@ clearHelpScreen (void) {
   }
 
   lineTableSize = 0;
-  lineLength = 0;
+  lineLength = 1;
   cursorRow = 0;
   cursorColumn = 0;
 }
@@ -62,7 +62,7 @@ addLine_HelpScreen (const wchar_t *line) {
     HelpLineEntry *newTable = realloc(lineTable, ARRAY_SIZE(newTable, newSize));
 
     if (!newTable) {
-      LogError("realloc");
+      logSystemError("realloc");
       return 0;
     }
 
@@ -76,7 +76,7 @@ addLine_HelpScreen (const wchar_t *line) {
     if ((hle->length = length) > lineLength) lineLength = length;
 
     if (!(hle->characters = malloc(ARRAY_SIZE(hle->characters, length)))) {
-      LogError("malloc");
+      logMallocError();
       return 0;
     }
 
@@ -87,6 +87,11 @@ addLine_HelpScreen (const wchar_t *line) {
   return 1;
 }
 
+static unsigned int 
+getLineCount_HelpScreen (void) {
+  return lineCount;
+}
+
 static int
 construct_HelpScreen (void) {
   clearHelpScreen();
@@ -205,4 +210,5 @@ initializeHelpScreen (HelpScreen *help) {
   help->construct = construct_HelpScreen;
   help->destruct = destruct_HelpScreen;
   help->addLine = addLine_HelpScreen;
+  help->getLineCount = getLineCount_HelpScreen;
 }
