@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2009 by The BRLTTY Developers.
+ * Copyright (C) 1995-2011 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -28,7 +28,8 @@
 #include <sys/time.h>
 #endif /* HAVE_SYS_SELECT_H */
 
-#include "misc.h"
+#include "log.h"
+#include "device.h"
 #include "async.h"
 #include "scr.h"
 #include "scr_real.h"
@@ -68,13 +69,13 @@ gpmOpenConnection (void) {
       gpm_zerobased = 1;
 
       if (Gpm_Open(&options, -1) == -1) {
-        LogPrint(GPM_LOG_LEVEL, "GPM open error: %s", strerror(errno));
+        logMessage(GPM_LOG_LEVEL, "GPM open error: %s", strerror(errno));
         asyncRelativeAlarm(5000, gpmResetConnection, NULL);
         gpmConnectionState = GCS_FAILED;
         return 0;
       }
 
-      LogPrint(GPM_LOG_LEVEL, "GPM opened: fd=%d con=%d", gpm_fd, gpm_consolefd);
+      logMessage(GPM_LOG_LEVEL, "GPM opened: fd=%d con=%d", gpm_fd, gpm_consolefd);
       gpmConnectionState = GCS_OPENED;
     }
 
@@ -89,7 +90,7 @@ static void
 gpmCloseConnection (int alreadyClosed) {
   if (gpmConnectionState == GCS_OPENED) {
     if (!alreadyClosed) Gpm_Close();
-    LogPrint(GPM_LOG_LEVEL, "GPM closed");
+    logMessage(GPM_LOG_LEVEL, "GPM closed");
   }
   gpmConnectionState = GCS_CLOSED;
 }
@@ -110,7 +111,7 @@ highlightRegion_RealScreen (int left, int right, int top, int bottom) {
       if (Gpm_DrawPointer(left, top, console) != -1) return 1;
 
       if (errno != EINVAL) {
-        LogPrint(GPM_LOG_LEVEL, "Gpm_DrawPointer error: %s", strerror(errno));
+        logMessage(GPM_LOG_LEVEL, "Gpm_DrawPointer error: %s", strerror(errno));
         gpmCloseConnection(0);
         return 0;
       }
@@ -145,18 +146,18 @@ getPointer_RealScreen (int *column, int *row) {
 
         if (result == -1) {
           if (errno == EINTR) continue;
-          LogError("select");
+          logSystemError("select");
           break;
         }
 
         if (!FD_ISSET(gpm_fd, &mask)) {
-          LogPrint(GPM_LOG_LEVEL, "GPM file descriptor not set: %d", gpm_fd);
+          logMessage(GPM_LOG_LEVEL, "GPM file descriptor not set: %d", gpm_fd);
           break;
         }
 
         if ((result = Gpm_GetEvent(&event)) == -1) {
           if (errno == EINTR) continue;
-          LogError("Gpm_GetEvent");
+          logSystemError("Gpm_GetEvent");
           break;
         }
 
