@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2010 by The BRLTTY Developers.
+ * Copyright (C) 1995-2011 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -175,7 +175,7 @@ struct SerialDeviceStruct {
 };
 
 typedef struct {
-  int baud;
+  unsigned int baud;
   SerialSpeed speed;
 } BaudEntry;
 static const BaudEntry baudTable[] = {
@@ -380,7 +380,7 @@ static const BaudEntry baudTable[] = {
 };
 
 static const BaudEntry *
-getBaudEntry (int baud) {
+getBaudEntry (unsigned int baud) {
   const BaudEntry *entry = baudTable;
   while (entry->baud) {
     if (baud == entry->baud) return entry;
@@ -458,51 +458,52 @@ serialSetSpeed (SerialDevice *serial, SerialSpeed speed) {
     if (cfsetispeed(&serial->pendingAttributes, speed) != -1) {
       return 1;
     } else {
-      LogError("cfsetispeed");
+      logSystemError("cfsetispeed");
     }
   } else {
-    LogError("cfsetospeed");
+    logSystemError("cfsetospeed");
   }
   return 0;
 #endif /* set speed */
 }
 
 int
-serialSetBaud (SerialDevice *serial, int baud) {
+serialSetBaud (SerialDevice *serial, unsigned int baud) {
   const BaudEntry *entry = getBaudEntry(baud);
   if (entry) {
     if (serialSetSpeed(serial, entry->speed)) {
       return 1;
     }
   } else {
-    LogPrint(LOG_WARNING, "unsupported serial baud: %d", baud);
+    logMessage(LOG_WARNING, "unsupported serial baud: %d", baud);
   }
   return 0;
 }
 
 int
-serialValidateBaud (int *baud, const char *description, const char *word, const int *choices) {
-  if (!*word || isInteger(baud, word)) {
+serialValidateBaud (unsigned int *baud, const char *description, const char *word, const unsigned int *choices) {
+  if (!*word || isUnsignedInteger(baud, word)) {
     const BaudEntry *entry = getBaudEntry(*baud);
+
     if (entry) {
       if (!choices) return 1;
 
       while (*choices) {
         if (*baud == *choices) return 1;
-        ++choices;
+        choices += 1;
       }
 
-      LogPrint(LOG_ERR, "Unsupported %s: %d", description, *baud);
+      logMessage(LOG_ERR, "unsupported %s: %u", description, *baud);
       return 0;
     }
   }
 
-  LogPrint(LOG_ERR, "Invalid %s: %d", description, *baud);
+  logMessage(LOG_ERR, "invalid %s: %u", description, *baud);
   return 0;
 }
 
 int
-serialSetDataBits (SerialDevice *serial, int bits) {
+serialSetDataBits (SerialDevice *serial, unsigned int bits) {
 #if defined(__MINGW32__) || defined(__MSDOS__)
   if ((bits < 5) || (bits > 8)) {
 #else /* UNIX */
@@ -526,7 +527,7 @@ serialSetDataBits (SerialDevice *serial, int bits) {
 
     default:
 #endif /* test data bits */
-      LogPrint(LOG_WARNING, "unsupported serial data bit count: %d", bits);
+      logMessage(LOG_WARNING, "unsupported serial data bit count: %d", bits);
       return 0;
   }
 
@@ -542,7 +543,7 @@ serialSetDataBits (SerialDevice *serial, int bits) {
 }
 
 int
-serialSetStopBits (SerialDevice *serial, int bits) {
+serialSetStopBits (SerialDevice *serial, unsigned int bits) {
 #if defined(__MINGW32__)
   if (bits == 1) {
     serial->pendingAttributes.StopBits = ONESTOPBIT;
@@ -562,7 +563,7 @@ serialSetStopBits (SerialDevice *serial, int bits) {
     serial->pendingAttributes.c_cflag |= CSTOPB;
 #endif /* set stop bits */
   } else {
-    LogPrint(LOG_WARNING, "unsupported serial stop bit count: %d", bits);
+    logMessage(LOG_WARNING, "unsupported serial stop bit count: %d", bits);
     return 0;
   }
   return 1;
@@ -644,7 +645,7 @@ serialSetParity (SerialDevice *serial, SerialParity parity) {
   return 1;
 
 unsupportedParity:
-  LogPrint(LOG_WARNING, "unsupported serial parity: %d", parity);
+  logMessage(LOG_WARNING, "unsupported serial parity: %d", parity);
   return 0;
 }
 
@@ -672,7 +673,7 @@ serialStartFlowControlThread (SerialDevice *serial) {
     pthread_attr_setdetachstate(&attributes, PTHREAD_CREATE_DETACHED);
 
     if (pthread_create(&thread, &attributes, serial->currentFlowControlProc, serial)) {
-      LogError("pthread_create");
+      logSystemError("pthread_create");
       return 0;
     }
 
@@ -801,71 +802,101 @@ serialSetFlowControl (SerialDevice *serial, SerialFlowControl flow) {
 #endif /* HAVE_POSIX_THREADS */
 
   if (!flow) return 1;
-  LogPrint(LOG_WARNING, "unsupported serial flow control: 0X%02X", flow);
+  logMessage(LOG_WARNING, "unsupported serial flow control: 0X%02X", flow);
   return 0;
 }
 
 int
-serialGetCharacterBits (SerialDevice *serial) {
-  int bits = 2; /* 1 start bit + 1 stop bit */
+serialSetParameters (SerialDevice *serial, const SerialParameters *parameters) {
+  if (!serialSetBaud(serial, parameters->baud)) return 0;
+  if (!serialSetDataBits(serial, parameters->dataBits)) return 0;
+  if (!serialSetStopBits(serial, parameters->stopBits)) return 0;
+  if (!serialSetParity(serial, parameters->parity)) return 0;
+  if (!serialSetFlowControl(serial, parameters->flowControl)) return 0;
+  return 1;
+}
 
+unsigned int
+serialGetCharacterSize (const SerialParameters *parameters) {
+  return 1 + parameters->dataBits + parameters->stopBits +
+         ((parameters->parity != SERIAL_PARITY_NONE)? 1: 0);
+}
+
+unsigned int
+serialGetDataBits (SerialDevice *serial) {
 #if defined(__MINGW32__)
-  bits += serial->pendingAttributes.ByteSize;
-  if (serial->pendingAttributes.fParity && (serial->pendingAttributes.Parity != NOPARITY)) ++bits;
-  if (serial->pendingAttributes.StopBits == TWOSTOPBITS) {
-    ++bits;
-  } else if (serial->pendingAttributes.StopBits != ONESTOPBIT) {
-    LogPrint(LOG_WARNING, "unsupported serial stop bits value: %X", serial->pendingAttributes.StopBits);
-  }
+  return serial->pendingAttributes.ByteSize;
 #elif defined(__MSDOS__)
-  bits += serial->pendingAttributes.bios.fields.bits + 5;
-  if (serial->pendingAttributes.bios.fields.parity) ++bits;
-  if (serial->pendingAttributes.bios.fields.stop) ++bits;
+  return serial->pendingAttributes.bios.fields.bits + 5;
 #else /* UNIX */
-  {
-    tcflag_t size = serial->pendingAttributes.c_cflag & CSIZE;
-    switch (size) {
-  #ifdef CS5
-      case CS5: bits += 5; break;
-  #endif /* CS5 */
+  tcflag_t size = serial->pendingAttributes.c_cflag & CSIZE;
 
-  #ifdef CS6
-      case CS6: bits += 6; break;
-  #endif /* CS6 */
+  switch (size) {
+#ifdef CS5
+    case CS5: return 5;
+#endif /* CS5 */
 
-  #ifdef CS7
-      case CS7: bits += 7; break;
-  #endif /* CS7 */
+#ifdef CS6
+    case CS6: return 6;
+#endif /* CS6 */
 
-  #ifdef CS8
-      case CS8: bits += 8; break;
-  #endif /* CS8 */
+#ifdef CS7
+    case CS7: return 7;
+#endif /* CS7 */
 
-      default:
-        LogPrint(LOG_WARNING, "unsupported serial data bits value: %lX", (unsigned long) size);
-        break;
-    }
+#ifdef CS8
+    case CS8: return 8;
+#endif /* CS8 */
+
+    default:
+      logMessage(LOG_WARNING, "unsupported serial data bits value: %lX", (unsigned long) size);
+      return 0;
   }
+#endif /* data bits */
+}
 
-  if (serial->pendingAttributes.c_cflag & PARENB) ++bits;
-  if (serial->pendingAttributes.c_cflag & CSTOPB) ++bits;
-#endif /* increment bits */
+unsigned int
+serialGetParityBits (SerialDevice *serial) {
+#if defined(__MINGW32__)
+  return (serial->pendingAttributes.fParity && (serial->pendingAttributes.Parity != NOPARITY))? 1: 0;
+#elif defined(__MSDOS__)
+  return serial->pendingAttributes.bios.fields.parity? 1: 0;
+#else /* UNIX */
+  return (serial->pendingAttributes.c_cflag & PARENB)? 1: 0;
+#endif /* parity bits */
+}
 
-  return bits;
+unsigned int
+serialGetStopBits (SerialDevice *serial) {
+#if defined(__MINGW32__)
+  if (serial->pendingAttributes.StopBits == ONESTOPBIT) return 1;
+  if (serial->pendingAttributes.StopBits == TWOSTOPBITS) return 2;
+  logMessage(LOG_WARNING, "unsupported serial stop bits value: %X", serial->pendingAttributes.StopBits);
+  return 0;
+#elif defined(__MSDOS__)
+  return serial->pendingAttributes.bios.fields.stop? 2: 1;
+#else /* UNIX */
+  return (serial->pendingAttributes.c_cflag & CSTOPB)? 2: 1;
+#endif /* stop bits */
+}
+
+unsigned int
+serialGetCharacterBits (SerialDevice *serial) {
+  return 1 + serialGetDataBits(serial) + serialGetParityBits(serial) + serialGetStopBits(serial);
 }
 
 int
 serialDiscardInput (SerialDevice *serial) {
 #if defined(__MINGW32__)
   if (PurgeComm(serial->fileHandle, PURGE_RXCLEAR)) return 1;
-  LogWindowsError("PurgeComm");
+  logWindowsSystemError("PurgeComm");
 #elif defined(__MSDOS__)
-  LogPrint(LOG_DEBUG, "function not supported: %s", __func__);
+  logMessage(LOG_DEBUG, "function not supported: %s", __func__);
   return 1;
 #else /* UNIX */
   if (tcflush(serial->fileDescriptor, TCIFLUSH) != -1) return 1;
   if (errno == EINVAL) return 1;
-  LogError("TCIFLUSH");
+  logSystemError("TCIFLUSH");
 #endif /* discard input */
   return 0;
 }
@@ -874,14 +905,14 @@ int
 serialDiscardOutput (SerialDevice *serial) {
 #if defined(__MINGW32__)
   if (PurgeComm(serial->fileHandle, PURGE_TXCLEAR)) return 1;
-  LogWindowsError("PurgeComm");
+  logWindowsSystemError("PurgeComm");
 #elif defined(__MSDOS__)
-  LogPrint(LOG_DEBUG, "function not supported: %s", __func__);
+  logMessage(LOG_DEBUG, "function not supported: %s", __func__);
   return 1;
 #else /* UNIX */
   if (tcflush(serial->fileDescriptor, TCOFLUSH) != -1) return 1;
   if (errno == EINVAL) return 1;
-  LogError("TCOFLUSH");
+  logSystemError("TCOFLUSH");
 #endif /* discard output */
   return 0;
 }
@@ -890,7 +921,7 @@ int
 serialFlushOutput (SerialDevice *serial) {
   if (serial->stream) {
     if (fflush(serial->stream) == EOF) {
-      LogError("fflush");
+      logSystemError("fflush");
       return 0;
     }
   }
@@ -903,15 +934,15 @@ serialDrainOutput (SerialDevice *serial) {
 
 #if defined(__MINGW32__)
   if (FlushFileBuffers(serial->fileHandle)) return 1;
-  LogWindowsError("FlushFileBuffers");
+  logWindowsSystemError("FlushFileBuffers");
 #elif defined(__MSDOS__)
-  LogPrint(LOG_DEBUG, "function not supported: %s", __func__);
+  logMessage(LOG_DEBUG, "function not supported: %s", __func__);
   return 1;
 #else /* UNIX */
   do {
     if (tcdrain(serial->fileDescriptor) != -1) return 1;
   } while (errno == EINTR);
-  LogError("tcdrain");
+  logSystemError("tcdrain");
 #endif /* drain output */
   return 0;
 }
@@ -931,7 +962,7 @@ serialReadAttributes (SerialDevice *serial) {
 #if defined(__MINGW32__)
   serial->currentAttributes.DCBlength = sizeof(serial->currentAttributes);
   if (GetCommState(serial->fileHandle, &serial->currentAttributes)) return 1;
-  LogWindowsError("GetCommState");
+  logWindowsSystemError("GetCommState");
 #elif defined(__MSDOS__)
   int interruptsWereEnabled = disable();
   unsigned char lcr = serialReadPort(serial, SERIAL_PORT_LCR);
@@ -959,7 +990,7 @@ serialReadAttributes (SerialDevice *serial) {
   return 1;
 #else /* UNIX */
   if (tcgetattr(serial->fileDescriptor, &serial->currentAttributes) != -1) return 1;
-  LogError("tcgetattr");
+  logSystemError("tcgetattr");
 #endif /* read attributes */
   return 0;
 }
@@ -971,14 +1002,14 @@ serialWriteAttributes (SerialDevice *serial, const SerialAttributes *attributes)
 
 #if defined(__MINGW32__)
     if (!SetCommState(serial->fileHandle, (SerialAttributes *)attributes)) {
-      LogWindowsError("SetCommState");
+      logWindowsSystemError("SetCommState");
       return 0;
     }
 #elif defined(__MSDOS__)
     {
       if (attributes->speed.biosBPS <= 7) {
         if (bioscom(0, attributes->bios.byte, serial->port) & 0X0700) {
-          LogPrint(LOG_ERR, "serialWriteAttributes failed");
+          logMessage(LOG_ERR, "serialWriteAttributes failed");
           return 0;
         }
       } else {
@@ -998,7 +1029,7 @@ serialWriteAttributes (SerialDevice *serial, const SerialAttributes *attributes)
     }
 #else /* UNIX */
     if (tcsetattr(serial->fileDescriptor, TCSANOW, attributes) == -1) {
-      LogError("tcsetattr");
+      logSystemError("tcsetattr");
       return 0;
     }
 #endif /* write attributes */
@@ -1040,13 +1071,13 @@ serialAwaitInput (SerialDevice *serial, int timeout) {
     char c;
 
     if (!(SetCommTimeouts(serial->fileHandle, &timeouts))) {
-      LogWindowsError("SetCommTimeouts serialAwaitInput");
+      logWindowsSystemError("SetCommTimeouts serialAwaitInput");
       setSystemErrno();
       return 0;
     }
 
     if (!ReadFile(serial->fileHandle, &c, 1, &bytesRead, NULL)) {
-      LogWindowsError("ReadFile");
+      logWindowsSystemError("ReadFile");
       setSystemErrno();
       return 0;
     }
@@ -1082,13 +1113,13 @@ serialReadChunk (
     bytesRead = 1;
   } else {
     if (!(SetCommTimeouts(serial->fileHandle, &timeouts))) {
-      LogWindowsError("SetCommTimeouts serialReadChunk1");
+      logWindowsSystemError("SetCommTimeouts serialReadChunk1");
       setSystemErrno();
       return 0;
     }
 
     if (!ReadFile(serial->fileHandle, buffer+*offset, count, &bytesRead, NULL)) {
-      LogWindowsError("ReadFile");
+      logWindowsSystemError("ReadFile");
       setSystemErrno();
       return 0;
     }
@@ -1103,7 +1134,7 @@ serialReadChunk (
   *offset += bytesRead;
   timeouts.ReadTotalTimeoutConstant = subsequentTimeout;
   if (!(SetCommTimeouts(serial->fileHandle, &timeouts))) {
-    LogWindowsError("SetCommTimeouts serialReadChunk2");
+    logWindowsSystemError("SetCommTimeouts serialReadChunk2");
     setSystemErrno();
     return 0;
   }
@@ -1119,7 +1150,7 @@ serialReadChunk (
   }
 
   if (!count) return 1;
-  LogWindowsError("ReadFile");
+  logWindowsSystemError("ReadFile");
   setSystemErrno();
   return 0;
 #else /* __MINGW32__ */
@@ -1156,7 +1187,7 @@ serialWriteData (
     DWORD bytesWritten;
 
     if (!(SetCommTimeouts(serial->fileHandle, &timeouts))) {
-      LogWindowsError("SetCommTimeouts serialWriteData");
+      logWindowsSystemError("SetCommTimeouts serialWriteData");
       setSystemErrno();
       return -1;
     }
@@ -1168,10 +1199,10 @@ serialWriteData (
     }
 
     if (!left) return size;
-    LogWindowsError("WriteFile");
+    logWindowsSystemError("WriteFile");
 #else /* __MINGW32__ */
     if (writeData(serial->fileDescriptor, data, size) != -1) return size;
-    LogError("serial write");
+    logSystemError("serial write");
 #endif /* __MINGW32__ */
   }
   return -1;
@@ -1182,12 +1213,12 @@ serialGetLines (SerialDevice *serial, SerialLines *lines) {
 #if defined(__MINGW32__)
   DCB dcb;
   if (!GetCommModemStatus(serial->fileHandle, &serial->linesState)) {
-    LogWindowsError("GetCommModemStatus");
+    logWindowsSystemError("GetCommModemStatus");
     return 0;
   }
   dcb.DCBlength = sizeof(dcb);
   if (!GetCommState(serial->fileHandle, &dcb)) {
-    LogWindowsError("GetCommState");
+    logWindowsSystemError("GetCommState");
     return 0;
   }
   if (dcb.fRtsControl == RTS_CONTROL_ENABLE)
@@ -1198,7 +1229,7 @@ serialGetLines (SerialDevice *serial, SerialLines *lines) {
   serial->linesState = serialReadPort(serial, SERIAL_PORT_MSR) & 0XF0;
 #elif defined(TIOCMGET)
   if (ioctl(serial->fileDescriptor, TIOCMGET, &serial->linesState) == -1) {
-    LogError("TIOCMGET");
+    logSystemError("TIOCMGET");
     return 0;
   }
 #else /* get lines */
@@ -1227,9 +1258,9 @@ serialSetLines (SerialDevice *serial, SerialLines high, SerialLines low) {
       dcb.fDtrControl = DTR_CONTROL_ENABLE;
 
     if (SetCommState(serial->fileHandle, &dcb)) return 1;
-    LogWindowsError("SetCommState");
+    logWindowsSystemError("SetCommState");
   } else {
-    LogWindowsError("GetCommState");
+    logWindowsSystemError("GetCommState");
   }
 #elif defined(__MSDOS__)
   int interruptsWereEnabled = disable();
@@ -1245,7 +1276,7 @@ serialSetLines (SerialDevice *serial, SerialLines high, SerialLines low) {
     status |= high;
     status &= ~low;
     if (ioctl(serial->fileDescriptor, TIOCMSET, &status) != -1) return 1;
-    LogError("TIOCMSET");
+    logSystemError("TIOCMSET");
   }
 #else /* set lines */
 #warning setting modem lines not supported on this platform
@@ -1299,7 +1330,7 @@ serialDefineWaitLines (SerialDevice *serial, SerialLines lines) {
     if (lines & SERIAL_LINE_CAR) eventMask |= EV_RLSD;
 
     if (!SetCommMask(serial->fileHandle, eventMask)) {
-      LogWindowsError("SetCommMask");
+      logWindowsSystemError("SetCommMask");
       return 0;
     }
 #endif /* __MINGW32__ */
@@ -1315,10 +1346,10 @@ serialMonitorWaitLines (SerialDevice *serial) {
 #if defined(__MINGW32__)
   DWORD event;
   if (WaitCommEvent(serial->fileHandle, &event, NULL)) return 1;
-  LogWindowsError("WaitCommEvent");
+  logWindowsSystemError("WaitCommEvent");
 #elif defined(TIOCMIWAIT)
   if (ioctl(serial->fileDescriptor, TIOCMIWAIT, serial->waitLines) != -1) return 1;
-  LogError("TIOCMIWAIT");
+  logSystemError("TIOCMIWAIT");
 #else
   SerialLines old = serial->linesState & serial->waitLines;
   SerialLines new;
@@ -1420,14 +1451,14 @@ serialOpenDevice (const char *path) {
             serial->pending = -1;
 #endif /* __MINGW32__ */
 
-            LogPrint(LOG_DEBUG, "serial device opened: %s: fd=%d",
-                     device,
+            logMessage(LOG_DEBUG, "serial device opened: %s: fd=%d",
+                       device,
 #ifdef __MINGW32__
-                     (int)serial->fileHandle
+                       (int)serial->fileHandle
 #else /* __MINGW32__ */
-                     serial->fileDescriptor
+                       serial->fileDescriptor
 #endif /* __MINGW32__ */
-                     );
+                       );
             free(device);
             return serial;
           }
@@ -1437,13 +1468,13 @@ serialOpenDevice (const char *path) {
 #else /* __MINGW32__ */
 #ifdef __MSDOS__
         } else {
-          LogPrint(LOG_ERR, "could not determine serial device port number: %s", device);
+          logMessage(LOG_ERR, "could not determine serial device port number: %s", device);
         }
 
         if (truePath) free(truePath);
 #else /* __MSDOS__ */
         } else {
-          LogPrint(LOG_ERR, "not a serial device: %s", device);
+          logMessage(LOG_ERR, "not a serial device: %s", device);
         }
 #endif /* __MSDOS__ */
 
@@ -1451,10 +1482,10 @@ serialOpenDevice (const char *path) {
 #endif /* __MINGW32__ */
       } else {
 #ifdef __MINGW32__
-        LogWindowsError("CreateFile");
-        LogPrint(LOG_ERR, "cannot open serial device: %s", device);
+        logWindowsSystemError("CreateFile");
+        logMessage(LOG_ERR, "cannot open serial device: %s", device);
 #else /* __MINGW32__ */
-        LogPrint(LOG_ERR, "cannot open serial device: %s: %s", device, strerror(errno));
+        logMessage(LOG_ERR, "cannot open serial device: %s: %s", device, strerror(errno));
 #endif /* __MINGW32__ */
       }
 
@@ -1463,7 +1494,7 @@ serialOpenDevice (const char *path) {
 
     free(serial);
   } else {
-    LogError("malloc");
+    logMallocError();
   }
 
   return NULL;
@@ -1495,7 +1526,7 @@ serialCloseDevice (SerialDevice *serial) {
 }
 
 int
-serialRestartDevice (SerialDevice *serial, int baud) {
+serialRestartDevice (SerialDevice *serial, unsigned int baud) {
   SerialLines highLines = 0;
   SerialLines lowLines = 0;
   int usingB0;
@@ -1570,10 +1601,10 @@ serialGetStream (SerialDevice *serial) {
     if (serial->fileDescriptor < 0) {
 #ifdef __CYGWIN32__
       if ((serial->fileDescriptor = cygwin_attach_handle_to_fd("serialdevice", -1, serial->fileHandle, TRUE, GENERIC_READ|GENERIC_WRITE)) < 0) {
-        LogError("cygwin_attach_handle_to_fd");
+        logSystemError("cygwin_attach_handle_to_fd");
 #else /* __CYGWIN32__ */
       if ((serial->fileDescriptor = _open_osfhandle((long)serial->fileHandle, O_RDWR)) < 0) {
-        LogError("open_osfhandle");
+        logSystemError("open_osfhandle");
 #endif /* __CYGWIN32__ */
         return NULL;
       }
@@ -1581,7 +1612,7 @@ serialGetStream (SerialDevice *serial) {
 #endif /* __MINGW32__ */
 
     if (!(serial->stream = fdopen(serial->fileDescriptor, "ab+"))) {
-      LogError("fdopen");
+      logSystemError("fdopen");
       return NULL;
     }
   }
@@ -1590,15 +1621,15 @@ serialGetStream (SerialDevice *serial) {
 }
 
 int
-isSerialDevice (const char **path) {
+isSerialDevice (const char **identifier) {
 #ifdef ALLOW_DOS_DEVICE_NAMES
-  if (isDosDevice(*path, "COM")) return 1;
+  if (isDosDevice(*identifier, "COM")) return 1;
 #endif /* ALLOW_DOS_DEVICE_NAMES */
 
-  if (!isQualifiedDevice(path, "serial"))
-    if (isQualifiedDevice(path, NULL))
+  if (!isQualifiedDevice(identifier, "serial"))
+    if (isQualifiedDevice(identifier, NULL))
       return 0;
 
-  if (!**path) *path = FIRST_SERIAL_DEVICE;
+  if (!**identifier) *identifier = FIRST_SERIAL_DEVICE;
   return 1;
 }
