@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2010 by The BRLTTY Developers.
+ * Copyright (C) 1995-2011 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -130,10 +130,17 @@ sayString (SpeechSynthesizer *spk, const char *string, int mute) {
 }
 
 static void
-saySpeechSetting (SpeechSynthesizer *spk, int setting, const char *name) {
-  char phrase[0X40];
-  snprintf(phrase, sizeof(phrase), "%s %d", name, setting);
-  sayString(spk, phrase, 1);
+sayStringSetting (SpeechSynthesizer *spk, const char *name, const char *string) {
+  char statement[0X40];
+  snprintf(statement, sizeof(statement), "%s %s", name, string);
+  sayString(spk, statement, 1);
+}
+
+static void
+sayIntegerSetting (SpeechSynthesizer *spk, const char *name, int integer) {
+  char string[0X10];
+  snprintf(string, sizeof(string), "%d", integer);
+  sayStringSetting(spk, name, string);
 }
 
 static unsigned int
@@ -141,11 +148,28 @@ getIntegerSetting (unsigned char setting, unsigned char internal, unsigned int e
   return rescaleInteger(setting, internal, external);
 }
 
+void
+setSpeechVolume (SpeechSynthesizer *spk, int setting, int say) {
+  logMessage(LOG_DEBUG, "setting speech volume: %d", setting);
+  speech->volume(spk, setting);
+  if (say) sayIntegerSetting(spk, gettext("volume"), setting);
+}
+
+unsigned int
+getIntegerSpeechVolume (unsigned char setting, unsigned int normal) {
+  return getIntegerSetting(setting, SPK_VOLUME_DEFAULT, normal);
+}
+
+float
+getFloatSpeechVolume (unsigned char setting) {
+  return (float)setting / (float)SPK_VOLUME_DEFAULT;
+}
+
 void
 setSpeechRate (SpeechSynthesizer *spk, int setting, int say) {
-  LogPrint(LOG_DEBUG, "setting speech rate: %d", setting);
+  logMessage(LOG_DEBUG, "setting speech rate: %d", setting);
   speech->rate(spk, setting);
-  if (say) saySpeechSetting(spk, setting, "rate");
+  if (say) sayIntegerSetting(spk, gettext("rate"), setting);
 }
 
 unsigned int
@@ -182,28 +206,11 @@ getFloatSpeechRate (unsigned char setting) {
   return spkRateTable[setting];
 }
 
-void
-setSpeechVolume (SpeechSynthesizer *spk, int setting, int say) {
-  LogPrint(LOG_DEBUG, "setting speech volume: %d", setting);
-  speech->volume(spk, setting);
-  if (say) saySpeechSetting(spk, setting, "volume");
-}
-
-unsigned int
-getIntegerSpeechVolume (unsigned char setting, unsigned int normal) {
-  return getIntegerSetting(setting, SPK_VOLUME_DEFAULT, normal);
-}
-
-float
-getFloatSpeechVolume (unsigned char setting) {
-  return (float)setting / (float)SPK_VOLUME_DEFAULT;
-}
-
 void
 setSpeechPitch (SpeechSynthesizer *spk, int setting, int say) {
-  LogPrint(LOG_DEBUG, "setting speech pitch: %d", setting);
+  logMessage(LOG_DEBUG, "setting speech pitch: %d", setting);
   speech->pitch(spk, setting);
-  if (say) saySpeechSetting(spk, setting, "pitch");
+  if (say) sayIntegerSetting(spk, gettext("pitch"), setting);
 }
 
 unsigned int
@@ -218,126 +225,133 @@ getFloatSpeechPitch (unsigned char setting) {
 
 void
 setSpeechPunctuation (SpeechSynthesizer *spk, SpeechPunctuation setting, int say) {
-  LogPrint(LOG_DEBUG, "setting speech punctuation: %d", setting);
+  logMessage(LOG_DEBUG, "setting speech punctuation: %d", setting);
   speech->punctuation(spk, setting);
 }
 
-static char *speechFifoPath = NULL;
+static char *speechInputPath = NULL;
 #ifdef __MINGW32__
-static HANDLE speechFifoHandle = INVALID_HANDLE_VALUE;
-static int speechFifoConnected;
+static HANDLE speechInputHandle = INVALID_HANDLE_VALUE;
+static int speechInputConnected;
 #else /* __MINGW32__ */
-static int speechFifoDescriptor = -1;
+static int speechInputDescriptor = -1;
 #endif /* __MINGW32__ */
 
 static void
-exitSpeechFifo (void) {
-  if (speechFifoPath) {
+exitSpeechInput (void) {
+  if (speechInputPath) {
 #ifndef __MINGW32__
-    unlink(speechFifoPath);
+    unlink(speechInputPath);
 #endif /* __MINGW32__ */
-    free(speechFifoPath);
-    speechFifoPath = NULL;
+
+    free(speechInputPath);
+    speechInputPath = NULL;
   }
 
 #ifdef __MINGW32__
-  if (speechFifoHandle != INVALID_HANDLE_VALUE) {
-    if (speechFifoConnected) {
-      DisconnectNamedPipe(speechFifoHandle);
-      speechFifoConnected = 0;
+  if (speechInputHandle != INVALID_HANDLE_VALUE) {
+    if (speechInputConnected) {
+      DisconnectNamedPipe(speechInputHandle);
+      speechInputConnected = 0;
     }
-    CloseHandle(speechFifoHandle);
-    speechFifoHandle = INVALID_HANDLE_VALUE;
+
+    CloseHandle(speechInputHandle);
+    speechInputHandle = INVALID_HANDLE_VALUE;
   }
 #else /* __MINGW32__ */
-  if (speechFifoDescriptor != -1) {
-    close(speechFifoDescriptor);
-    speechFifoDescriptor = -1;
+  if (speechInputDescriptor != -1) {
+    close(speechInputDescriptor);
+    speechInputDescriptor = -1;
   }
 #endif /* __MINGW32__ */
 }
 
 int
-openSpeechFifo (const char *directory, const char *path) {
-  atexit(exitSpeechFifo);
+enableSpeechInput (const char *name) {
+  const char *directory;
+  atexit(exitSpeechInput);
 
 #ifdef __MINGW32__
   directory = "//./pipe";
+#else /* __MINGW32__ */
+  directory = ".";
 #endif /* __MINGW32__ */
 
-  if ((speechFifoPath = makePath(directory, path))) {
+  if ((speechInputPath = makePath(directory, name))) {
 #ifdef __MINGW32__
-    if ((speechFifoHandle = CreateNamedPipe(speechFifoPath, PIPE_ACCESS_INBOUND,
-                                            PIPE_TYPE_MESSAGE|PIPE_READMODE_MESSAGE|PIPE_NOWAIT,
-                                            1, 0, 0, 0, NULL)) != INVALID_HANDLE_VALUE) {
-      LogPrint(LOG_DEBUG, "Speech FIFO created: %s: handle=%u",
-               speechFifoPath, (unsigned)speechFifoHandle);
+    if ((speechInputHandle = CreateNamedPipe(speechInputPath, PIPE_ACCESS_INBOUND,
+                                             PIPE_TYPE_MESSAGE|PIPE_READMODE_MESSAGE|PIPE_NOWAIT,
+                                             1, 0, 0, 0, NULL)) != INVALID_HANDLE_VALUE) {
+      logMessage(LOG_DEBUG, "speech input pipe created: %s: handle=%u",
+                 speechInputPath, (unsigned)speechInputHandle);
       return 1;
     } else {
-      LogWindowsError("CreateNamedPipe");
+      logWindowsSystemError("CreateNamedPipe");
     }
 #else /* __MINGW32__ */
-    int ret = mkfifo(speechFifoPath, 0);
+    int result = mkfifo(speechInputPath, 0);
 
-    if ((ret == -1) && (errno == EEXIST)) {
+    if ((result == -1) && (errno == EEXIST)) {
       struct stat fifo;
-      if ((lstat(speechFifoPath, &fifo) != -1) && S_ISFIFO(fifo.st_mode)) ret = 0;
+
+      if ((lstat(speechInputPath, &fifo) != -1) && S_ISFIFO(fifo.st_mode)) result = 0;
     }
 
-    if (ret != -1) {
-      chmod(speechFifoPath, S_IRUSR|S_IWUSR|S_IWGRP|S_IWOTH);
-      if ((speechFifoDescriptor = open(speechFifoPath, O_RDONLY|O_NONBLOCK)) != -1) {
-        LogPrint(LOG_DEBUG, "Speech FIFO created: %s: fd=%d",
-                 speechFifoPath, speechFifoDescriptor);
+    if (result != -1) {
+      chmod(speechInputPath, S_IRUSR|S_IWUSR|S_IWGRP|S_IWOTH);
+
+      if ((speechInputDescriptor = open(speechInputPath, O_RDONLY|O_NONBLOCK)) != -1) {
+        logMessage(LOG_DEBUG, "speech input FIFO created: %s: fd=%d",
+                   speechInputPath, speechInputDescriptor);
         return 1;
       } else {
-        LogPrint(LOG_ERR, "cannot open speech FIFO: %s: %s",
-                 speechFifoPath, strerror(errno));
+        logMessage(LOG_ERR, "cannot open speech input FIFO: %s: %s",
+                   speechInputPath, strerror(errno));
       }
 
-      unlink(speechFifoPath);
+      unlink(speechInputPath);
     } else {
-      LogPrint(LOG_ERR, "cannot create speech FIFO: %s: %s",
-               speechFifoPath, strerror(errno));
+      logMessage(LOG_ERR, "cannot create speech input FIFO: %s: %s",
+                 speechInputPath, strerror(errno));
     }
 #endif /* __MINGW32__ */
 
-    free(speechFifoPath);
-    speechFifoPath = NULL;
+    free(speechInputPath);
+    speechInputPath = NULL;
   }
 
   return 0;
 }
 
 void
-processSpeechFifo (SpeechSynthesizer *spk) {
+processSpeechInput (SpeechSynthesizer *spk) {
 #ifdef __MINGW32__
-  if (speechFifoHandle != INVALID_HANDLE_VALUE) {
-    if (speechFifoConnected ||
-        (speechFifoConnected = ConnectNamedPipe(speechFifoHandle, NULL)) ||
-        (speechFifoConnected = (GetLastError() == ERROR_PIPE_CONNECTED))) {
+  if (speechInputHandle != INVALID_HANDLE_VALUE) {
+    if (speechInputConnected ||
+        (speechInputConnected = ConnectNamedPipe(speechInputHandle, NULL)) ||
+        (speechInputConnected = (GetLastError() == ERROR_PIPE_CONNECTED))) {
       char buffer[0X1000];
       DWORD count;
 
-      if (ReadFile(speechFifoHandle, buffer, sizeof(buffer), &count, NULL)) {
+      if (ReadFile(speechInputHandle, buffer, sizeof(buffer), &count, NULL)) {
         if (count) sayCharacters(spk, buffer, count, 0);
       } else {
         DWORD error = GetLastError();
 
         if (error != ERROR_NO_DATA) {
-          speechFifoConnected = 0;
-          DisconnectNamedPipe(speechFifoHandle);
+          speechInputConnected = 0;
+          DisconnectNamedPipe(speechInputHandle);
 
           if (error != ERROR_BROKEN_PIPE)
-            LogWindowsError("Speech FIFO read");
+            logWindowsSystemError("speech input FIFO read");
         }
       }
     }
   }
 #else /* __MINGW32__ */
-  if (speechFifoDescriptor != -1) {
+  if (speechInputDescriptor != -1) {
     char buffer[0X1000];
-    int count = read(speechFifoDescriptor, buffer, sizeof(buffer));
+    int count = read(speechInputDescriptor, buffer, sizeof(buffer));
     if (count > 0) sayCharacters(spk, buffer, count, 0);
   }
 #endif /* __MINGW32__ */
