@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2009 by The BRLTTY Developers.
+ * Copyright (C) 1995-2011 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -19,6 +19,8 @@
 #include <signal.h>
 #include <sys/wait.h>
 
+#include "log.h"
+
 int
 executeHostCommand (const char *const *arguments) {
   int result = 0XFF;
@@ -31,32 +33,32 @@ executeHostCommand (const char *const *arguments) {
 
   switch ((pid = fork())) {
     case -1: /* error */
-      LogError("fork");
+      logSystemError("fork");
       break;
 
     case 0: /* child */
       sigprocmask(SIG_SETMASK, &oldMask, NULL);
       execvp(arguments[0], (char *const*)arguments);
-      LogError("execvp");
+      logSystemError("execvp");
       _exit(1);
 
     default: { /* parent */
       int status;
       if (waitpid(pid, &status, 0) == -1) {
-        LogError("waitpid");
+        logSystemError("waitpid");
       } else if (WIFEXITED(status)) {
         result = WEXITSTATUS(status);
-        LogPrint(LOG_DEBUG, "exit status: %d", result);
+        logMessage(LOG_DEBUG, "exit status: %d", result);
       } else if (WIFSIGNALED(status)) {
         result = WTERMSIG(status);
-        LogPrint(LOG_DEBUG, "termination signal: %d", result);
+        logMessage(LOG_DEBUG, "termination signal: %d", result);
         result += 0X80;
       } else if (WIFSTOPPED(status)) {
         result = WSTOPSIG(status);
-        LogPrint(LOG_DEBUG, "stop signal: %d", result);
+        logMessage(LOG_DEBUG, "stop signal: %d", result);
         result += 0X80;
       } else {
-        LogPrint(LOG_DEBUG, "unknown status: 0X%X", status);
+        logMessage(LOG_DEBUG, "unknown status: 0X%X", status);
       }
     }
   }
