@@ -2,13 +2,14 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2007 by The BRLTTY Developers.
+ * Copyright (C) 1995-2011 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
  * This is free software, placed under the terms of the
  * GNU General Public License, as published by the Free Software
- * Foundation.  Please see the file COPYING for details.
+ * Foundation; either version 2 of the License, or (at your option) any
+ * later version. Please see the file LICENSE-GPL for details.
  *
  * Web Page: http://mielke.cc/brltty/
  *
@@ -23,8 +24,13 @@ extern "C" {
 #endif /* __cplusplus */
 
 extern int installKernelModule (const char *name, int *status);
+
 extern int openCharacterDevice (const char *name, int flags, int major, int minor);
+
 extern int getUinputDevice (void);
+extern int hasInputEvent (int device, uint16_t type, uint16_t code, uint16_t max);
+extern int writeInputEvent (uint16_t type, uint16_t code, int32_t value);
+
 extern int writeKeyEvent (int key, int press);
 extern void releaseAllKeys (void);
 
