@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2010 by The BRLTTY Developers.
+ * Copyright (C) 1995-2011 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -22,7 +22,7 @@
 
 MidiDevice *
 openMidiDevice (int errorLevel, const char *device) {
-  LogPrint(errorLevel, "MIDI device not supported.");
+  logMessage(errorLevel, "MIDI device not supported.");
   return NULL;
 }
 
