@@ -2,24 +2,29 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2008 by The BRLTTY Developers.
+ * Copyright (C) 1995-2011 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
  * This is free software, placed under the terms of the
  * GNU General Public License, as published by the Free Software
- * Foundation.  Please see the file COPYING for details.
+ * Foundation; either version 2 of the License, or (at your option) any
+ * later version. Please see the file LICENSE-GPL for details.
  *
  * Web Page: http://mielke.cc/brltty/
  *
  * This software is maintained by Dave Mielke <dave@mielke.cc>.
  */
 
+#include "log.h"
+#include "parse.h"
+#include "timing.h"
+
 struct MidiDeviceStruct {
   HMIDIOUT handle;
   unsigned char note;
   int count;
-  unsigned char buffer[0X80];
+  char buffer[0X80];
 };
 
 typedef enum {
@@ -37,11 +42,11 @@ static void
 logMidiOutError (MMRESULT error, int errorLevel, const char *action) {
   char text[MAXERRORLENGTH];
   midiOutGetErrorText(error, text, sizeof(text));
-  LogPrint(errorLevel, "%s error %d: %s", action, error, text);
+  logMessage(errorLevel, "%s error %d: %s", action, error, text);
 }
 
 static int
-addMidiMessage (MidiDevice *midi, const char *message, int length) {
+addMidiMessage (MidiDevice *midi, const unsigned char *message, int length) {
   if ((midi->count + length) > sizeof(midi->buffer))
     if (!flushMidiDevice(midi))
       return 0;
@@ -52,7 +57,7 @@ addMidiMessage (MidiDevice *midi, const char *message, int length) {
 }
 
 static int
-writeMidiMessage (MidiDevice *midi, const char *message, int length) {
+writeMidiMessage (MidiDevice *midi, const unsigned char *message, int length) {
   if (!addMidiMessage(midi, message, length)) return 0;
   if (!flushMidiDevice(midi)) return 0;
   return 1;
@@ -79,7 +84,7 @@ openMidiDevice (int errorLevel, const char *device) {
     }
 
     if (id == count) {
-      LogPrint(errorLevel, "invalid MIDI device number: %s", device);
+      logMessage(errorLevel, "invalid MIDI device number: %s", device);
       return NULL;
     }
   }
@@ -95,7 +100,7 @@ openMidiDevice (int errorLevel, const char *device) {
 
     free(midi);
   } else {
-    LogError("MIDI device allocation");
+    logSystemError("MIDI device allocation");
   }
   return NULL;
 }
