@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2009 by The BRLTTY Developers.
+ * Copyright (C) 1995-2011 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -25,7 +25,6 @@
 #include <sys/audioio.h>
 #include <sys/time.h>
 
-#include "misc.h"
 #include "system.h"
 
 #include "sys_prog_none.h"
@@ -53,5 +52,3 @@
 #endif /* ENABLE_MIDI_SUPPORT */
 
 #include "sys_ports_none.h"
-
-#include "sys_kbd_none.h"
