@@ -2,19 +2,23 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2007 by The BRLTTY Developers.
+ * Copyright (C) 1995-2011 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
  * This is free software, placed under the terms of the
  * GNU General Public License, as published by the Free Software
- * Foundation.  Please see the file COPYING for details.
+ * Foundation; either version 2 of the License, or (at your option) any
+ * later version. Please see the file LICENSE-GPL for details.
  *
  * Web Page: http://mielke.cc/brltty/
  *
  * This software is maintained by Dave Mielke <dave@mielke.cc>.
  */
 
+#include "log.h"
+#include "misc.h"
+
 char *
 getProgramPath (void) {
   char *path = NULL;
@@ -31,7 +35,7 @@ getProgramPath (void) {
         DWORD length = GetModuleFileName(handle, buffer, size);
 
         if (!length) {
-          LogWindowsError("GetModuleFileName");
+          logWindowsSystemError("GetModuleFileName");
           break;
         }
 
@@ -50,7 +54,7 @@ getProgramPath (void) {
 
     free(buffer);
   } else {
-    LogWindowsError("GetModuleHandle");
+    logWindowsSystemError("GetModuleHandle");
   }
 
   return path;
