@@ -2,13 +2,14 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2007 by The BRLTTY Developers.
+ * Copyright (C) 1995-2011 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
  * This is free software, placed under the terms of the
  * GNU General Public License, as published by the Free Software
- * Foundation.  Please see the file COPYING for details.
+ * Foundation; either version 2 of the License, or (at your option) any
+ * later version. Please see the file LICENSE-GPL for details.
  *
  * Web Page: http://mielke.cc/brltty/
  *
@@ -21,6 +22,7 @@
 #include <errno.h>
 #include <fcntl.h>
 
+#include "log.h"
 #include "misc.h"
 #include "system.h"
 
@@ -37,7 +39,7 @@ getProgramPath (void) {
       int length = readlink("/proc/self/path/a.out", buffer, size);
 
       if (length == -1) {
-        if (errno != ENOENT) LogError("readlink");
+        if (errno != ENOENT) logSystemError("readlink");
         break;
       }
 
@@ -73,9 +75,9 @@ getKeyboard (void) {
   static int keyboard = -1;
   if (keyboard == -1) {
     if ((keyboard = open("/dev/kbd", O_WRONLY)) != -1) {
-      LogPrint(LOG_DEBUG, "keyboard opened: fd=%d", keyboard);
+      logMessage(LOG_DEBUG, "keyboard opened: fd=%d", keyboard);
     } else {
-      LogError("keyboard open");
+      logSystemError("keyboard open");
     }
   }
   return keyboard;
@@ -102,7 +104,7 @@ startBeep (unsigned short frequency) {
   if (keyboard != -1) {
     int command = KBD_CMD_BELL;
     if (ioctl(keyboard, KIOCCMD, &command) != -1) return 1;
-    LogError("ioctl KIOCCMD KBD_CMD_BELL");
+    logSystemError("ioctl KIOCCMD KBD_CMD_BELL");
   }
   return 0;
 }
@@ -113,7 +115,7 @@ stopBeep (void) {
   if (keyboard != -1) {
     int command = KBD_CMD_NOBELL;
     if (ioctl(keyboard, KIOCCMD, &command) != -1) return 1;
-    LogError("ioctl KIOCCMD KBD_CMD_NOBELL");
+    logSystemError("ioctl KIOCCMD KBD_CMD_NOBELL");
   }
   return 0;
 }
