@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2010 by The BRLTTY Developers.
+ * Copyright (C) 1995-2011 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -53,7 +53,7 @@ BEGIN_OPTION_TABLE(programOptions)
     .flags = OPT_Hidden,
     .argument = "directory",
     .setting.string = &opt_tablesDirectory,
-    .defaultSetting = DATA_DIRECTORY,
+    .defaultSetting = TABLES_DIRECTORY,
     .description = "Path to directory containing text tables."
   },
 
@@ -114,7 +114,7 @@ mapDots (unsigned char input, const BrlDotTable from, const BrlDotTable to) {
 
 typedef TextTableData *TableReader (const char *path, FILE *file, void *data);
 typedef int TableWriter (const char *path, FILE *file, TextTableData *ttd, void *data);
-typedef int CharacterWriter (FILE *file, wchar_t character, unsigned char dots, const unsigned char *byte, void *data);
+typedef int CharacterWriter (FILE *file, wchar_t character, unsigned char dots, const unsigned char *byte, int isPrimary, void *data);
 
 static int
 getDots (TextTableData *ttd, wchar_t character, unsigned char *dots) {
@@ -124,6 +124,17 @@ getDots (TextTableData *ttd, wchar_t character, unsigned char *dots) {
   return 1;
 }
 
+static int
+isPrimaryCharacter (TextTableData *ttd, wchar_t character, unsigned char dots) {
+  const TextTableHeader *header = getTextTableHeader(ttd);
+
+  if (BITMASK_TEST(header->dotsCharacterDefined, dots))
+    if (header->dotsToCharacter[dots] == character)
+      return 1;
+
+  return 0;
+}
+
 static int
 writeCharacters (FILE *file, TextTableData *ttd, CharacterWriter writer, void *data) {
   if (*opt_charset) {
@@ -136,7 +147,8 @@ writeCharacters (FILE *file, TextTableData *ttd, CharacterWriter writer, void *d
         const unsigned char *cell = getUnicodeCellEntry(ttd, character);
 
         if (cell) {
-          if (!writer(file, character, *cell, &byte, data)) return 0;
+          int isPrimary = isPrimaryCharacter(ttd, character, *cell);
+          if (!writer(file, character, *cell, &byte, isPrimary, data)) return 0;
         }
       }
     } while ((byte += 1));
@@ -177,7 +189,8 @@ writeCharacters (FILE *file, TextTableData *ttd, CharacterWriter writer, void *d
 
                     {
                       const unsigned char *cell = &row->cells[cellNumber];
-                      if (!writer(file, character, *cell, NULL, data)) return 0;
+                      int isPrimary = isPrimaryCharacter(ttd, character, *cell);
+                      if (!writer(file, character, *cell, NULL, isPrimary, data)) return 0;
                     }
                   }
                 }
@@ -219,10 +232,10 @@ writeDots_native (FILE *file, unsigned char dots) {
 }
 
 static int
-writeCharacter_native (FILE *file, wchar_t character, unsigned char dots, const unsigned char *byte, void *data) {
+writeCharacter_native (FILE *file, wchar_t character, unsigned char dots, const unsigned char *byte, int isPrimary, void *data) {
   uint32_t value = character;
 
-  if (fprintf(file, "char ") == EOF) goto error;
+  if (fprintf(file, "%s ", (isPrimary? "char": "glyph")) == EOF) goto error;
 
   if (value < 0X100) {
     if (fprintf(file, "\\x%02" PRIX32, value) == EOF) goto error;
@@ -302,9 +315,9 @@ readTable_binary (const char *path, FILE *file, void *data) {
 
       if (dots == EOF) {
         if (ferror(file)) {
-          LogPrint(LOG_ERR, "input error: %s: %s", path, strerror(errno));
+          logMessage(LOG_ERR, "input error: %s: %s", path, strerror(errno));
         } else {
-          LogPrint(LOG_ERR, "table too short: %s", path);
+          logMessage(LOG_ERR, "table too short: %s", path);
         }
 
         break;
@@ -338,7 +351,7 @@ writeTable_binary (const char *path, FILE *file, TextTableData *ttd, void *data)
 
     if (data) dots = mapDots(dots, dotsInternal, data);
     if (fputc(dots, file) == EOF) {
-      LogPrint(LOG_ERR, "output error: %s: %s", path, strerror(errno));
+      logMessage(LOG_ERR, "output error: %s: %s", path, strerror(errno));
       return 0;
     }
   }
@@ -353,7 +366,7 @@ readTable_gnome (const char *path, FILE *file, void *data) {
 }
 
 static int
-writeCharacter_gnome (FILE *file, wchar_t character, unsigned char dots, const unsigned char *byte, void *data) {
+writeCharacter_gnome (FILE *file, wchar_t character, unsigned char dots, const unsigned char *byte, int isPrimary, void *data) {
   wchar_t pattern = UNICODE_BRAILLE_ROW | dots;
 
   if (iswprint(character) && !iswspace(character)) {
@@ -391,7 +404,7 @@ readTable_libLouis (const char *path, FILE *file, void *data) {
 }
 
 static int
-writeCharacter_libLouis (FILE *file, wchar_t character, unsigned char dots, const unsigned char *byte, void *data) {
+writeCharacter_libLouis (FILE *file, wchar_t character, unsigned char dots, const unsigned char *byte, int isPrimary, void *data) {
   int i;
 
   {
@@ -429,7 +442,7 @@ writeCharacter_libLouis (FILE *file, wchar_t character, unsigned char dots, cons
     if (fprintf(file, "\\t") == EOF) return 0;
   } else if (character == WC_C('\v')) {
     if (fprintf(file, "\\v") == EOF) return 0;
-  } else if ((character > 0X20) && (character < 0X7F)) {
+  } else if ((character > 0X20) && (character < 0X7F) && (character != '#')) {
     wint_t value = character;
     if (fprintf(file, "%" PRIwc, value) == EOF) return 0;
   } else {
@@ -479,6 +492,67 @@ writeTable_libLouis (const char *path, FILE *file, TextTableData *ttd, void *dat
 error:
   return 0;
 }
+
+static TextTableData *
+readTable_XCompose (const char *path, FILE *file, void *data) {
+  logMessage(LOG_ERR, "Reading XCompose format not supported");
+  return NULL;
+}
+
+static int
+writeCharacter_XCompose (FILE *file, wchar_t character, unsigned char dots, const unsigned char *byte, int isPrimary, void *_data) {
+  if (isPrimary) {
+    if (fprintf(file, "<braille_") == EOF) return 0;
+
+    if (!dots) {
+      if (fprintf(file, "blank") == EOF) return 0;
+    } else {
+      int i;
+      if (fprintf(file, "dots_") == EOF) return 0;
+
+      for (i = 1; i <= 8; i++) {
+        if (dots & (1 << (i -1))) {
+          if (fprintf(file, "%c", '0' + i) == EOF) return 0;
+        }
+      }
+    }
+
+    if (fprintf(file, "> : \"") == EOF) return 0;
+
+    switch (character) {
+      case WC_C('\n'):
+        if (fprintf(file, "\\n") == EOF) return 0;
+        break;
+
+      case WC_C('\r'):
+        if (fprintf(file, "\\r") == EOF) return 0;
+        break;
+
+      case WC_C('"'):
+        if (fprintf(file, "\\\"") == EOF) return 0;
+        break;
+
+      default:
+        if (fprintf(file, "%lc", (wint_t)character) == EOF) return 0;
+        break;
+    }
+
+    if (fprintf(file, "\"\n") == EOF) return 0;
+  }
+
+  return 1;
+}
+
+static int
+writeTable_XCompose (const char *path, FILE *file, TextTableData *ttd, void *data) {
+  if (fprintf(file, "# generated by %s\n", programName) == EOF) goto error;
+  if (!writeCharacters(file, ttd, writeCharacter_XCompose, NULL)) goto error;
+  return 1;
+
+error:
+  return 0;
+}
+
 typedef struct {
   const char *name;
   TableReader *read;
@@ -496,6 +570,7 @@ static const FormatEntry formatEntries[] = {
 #endif /* HAVE_ICONV_H */
 
   {"ctb", readTable_libLouis, writeTable_libLouis, NULL},
+  {"XCompose", readTable_XCompose, writeTable_XCompose, NULL},
 
   {NULL}
 };
@@ -516,7 +591,7 @@ getFormatEntry (const char *name, const char *path, const char *description) {
     name = locatePathExtension(path);
 
     if (!(name && *++name)) {
-      LogPrint(LOG_ERR, "unspecified %s format.", description);
+      logMessage(LOG_ERR, "unspecified %s format.", description);
       exit(2);
     }
   }
@@ -526,7 +601,7 @@ getFormatEntry (const char *name, const char *path, const char *description) {
     if (format) return format;
   }
 
-  LogPrint(LOG_ERR, "unknown %s format: %s", description, name);
+  logMessage(LOG_ERR, "unknown %s format: %s", description, name);
   exit(2);
 }
 
@@ -552,7 +627,7 @@ openTable (const char **file, const char *mode, const char *directory, FILE *std
 
   {
     FILE *stream = fopen(*file, mode);
-    if (!stream) LogPrint(LOG_ERR, "table open error: %s: %s", *file, strerror(errno));
+    if (!stream) logMessage(LOG_ERR, "table open error: %s: %s", *file, strerror(errno));
     return stream;
   }
 }
@@ -826,7 +901,7 @@ static int
 updateCharacterDescription (EditTableData *etd) {
   int ok = 0;
 
-  wchar_t character;
+  wchar_t character = UNICODE_REPLACEMENT_CHARACTER;
   int gotCharacter = getCharacter(etd, &character);
 
   size_t length;
@@ -1813,7 +1888,7 @@ main (int argc, char *argv[]) {
   }
 
   if (argc == 0) {
-    LogPrint(LOG_ERR, "missing input table.");
+    logMessage(LOG_ERR, "missing input table.");
     exit(2);
   }
   inputPath = *argv++, argc--;
@@ -1831,7 +1906,7 @@ main (int argc, char *argv[]) {
   }
 
   if (argc > 0) {
-    LogPrint(LOG_ERR, "too many parameters.");
+    logMessage(LOG_ERR, "too many parameters.");
     exit(2);
   }
 
@@ -1843,7 +1918,7 @@ main (int argc, char *argv[]) {
   }
 
   if (*opt_charset && !setCharset(opt_charset)) {
-    LogPrint(LOG_ERR, "can't establish character set: %s", opt_charset);
+    logMessage(LOG_ERR, "can't establish character set: %s", opt_charset);
     exit(9);
   }
 
