@@ -2,13 +2,14 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2008 by The BRLTTY Developers.
+ * Copyright (C) 1995-2011 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
  * This is free software, placed under the terms of the
  * GNU General Public License, as published by the Free Software
- * Foundation.  Please see the file COPYING for details.
+ * Foundation; either version 2 of the License, or (at your option) any
+ * later version. Please see the file LICENSE-GPL for details.
  *
  * Web Page: http://mielke.cc/brltty/
  *
@@ -26,14 +27,15 @@
 
 #include "options.h"
 #include "tunes.h"
+#include "log.h"
+#include "parse.h"
 #include "misc.h"
 #include "defaults.h"
-#include "brltty.h"
+#include "prefs.h"
 #include "message.h"
 #include "brl.h"
 
 int updateInterval = DEFAULT_UPDATE_INTERVAL;
-Preferences prefs;
 
 static char *opt_tuneDevice;
 static char *opt_outputVolume;
@@ -132,13 +134,6 @@ validateInstrument (unsigned char *value, const char *string) {
 
 int
 main (int argc, char *argv[]) {
-  TuneDevice tuneDevice;
-  unsigned char outputVolume;
-
-#ifdef ENABLE_MIDI_SUPPORT
-  unsigned char midiInstrument;
-#endif /* ENABLE_MIDI_SUPPORT */
-
   {
     static const OptionsDescriptor descriptor = {
       OPTION_TABLE(programOptions),
@@ -148,25 +143,32 @@ main (int argc, char *argv[]) {
     processOptions(&descriptor, &argc, &argv);
   }
 
+  resetPreferences();
+  prefs.alertMessages = 0;
+  prefs.alertDots = 0;
+  prefs.alertTunes = 1;
+
   if (opt_tuneDevice && *opt_tuneDevice) {
     unsigned int device;
+
     if (!validateChoice(&device, opt_tuneDevice, deviceNames)) {
-      LogPrint(LOG_ERR, "%s: %s", "invalid tune device", opt_tuneDevice);
+      logMessage(LOG_ERR, "%s: %s", "invalid tune device", opt_tuneDevice);
       exit(2);
     }
-    tuneDevice = device;
-  } else {
-    tuneDevice = getDefaultTuneDevice();
+
+    prefs.tuneDevice = device;
   }
 
 #ifdef ENABLE_MIDI_SUPPORT
   if (opt_midiInstrument && *opt_midiInstrument) {
-    if (!validateInstrument(&midiInstrument, opt_midiInstrument)) {
-      LogPrint(LOG_ERR, "%s: %s", "invalid musical instrument", opt_midiInstrument);
+    unsigned char instrument;
+
+    if (!validateInstrument(&instrument, opt_midiInstrument)) {
+      logMessage(LOG_ERR, "%s: %s", "invalid musical instrument", opt_midiInstrument);
       exit(2);
     }
-  } else {
-    midiInstrument = 0;
+
+    prefs.midiInstrument = instrument;
   }
 #endif /* ENABLE_MIDI_SUPPORT */
 
@@ -174,22 +176,37 @@ main (int argc, char *argv[]) {
     static const int minimum = 0;
     static const int maximum = 100;
     int volume;
+
     if (!validateInteger(&volume, opt_outputVolume, &minimum, &maximum)) {
-      LogPrint(LOG_ERR, "%s: %s", "invalid volume percentage", opt_outputVolume);
+      logMessage(LOG_ERR, "%s: %s", "invalid volume percentage", opt_outputVolume);
       exit(2);
     }
-    outputVolume = volume;
-  } else {
-    outputVolume = 50;
+
+    switch (prefs.tuneDevice) {
+      case tdPcm:
+        prefs.pcmVolume = volume;
+        break;
+
+      case tdMidi:
+        prefs.midiVolume = volume;
+        break;
+
+      case tdFm:
+        prefs.fmVolume = volume;
+        break;
+
+      default:
+        break;
+    }
   }
 
   if (!argc) {
-    LogPrint(LOG_ERR, "missing tune.");
+    logMessage(LOG_ERR, "missing tune.");
     exit(2);
   }
 
   if (argc % 2) {
-    LogPrint(LOG_ERR, "missing note duration.");
+    logMessage(LOG_ERR, "missing note duration.");
     exit(2);
   }
 
@@ -207,7 +224,7 @@ main (int argc, char *argv[]) {
         static const int maximum = 0X7F;
         const char *argument = *argv++;
         if (!validateInteger(&note, argument, &minimum, &maximum)) {
-          LogPrint(LOG_ERR, "%s: %s", "invalid note number", argument);
+          logMessage(LOG_ERR, "%s: %s", "invalid note number", argument);
           exit(2);
         }
         --argc;
@@ -218,7 +235,7 @@ main (int argc, char *argv[]) {
         static const int maximum = 255;
         const char *argument = *argv++;
         if (!validateInteger(&duration, argument, &minimum, &maximum)) {
-          LogPrint(LOG_ERR, "%s: %s", "invalid note duration", argument);
+          logMessage(LOG_ERR, "%s: %s", "invalid note duration", argument);
           exit(2);
         }
         --argc;
@@ -235,31 +252,11 @@ main (int argc, char *argv[]) {
       *element = te;
     }
 
-    if (!setTuneDevice(tuneDevice)) {
-      LogPrint(LOG_ERR, "unsupported tune device: %s", deviceNames[tuneDevice]);
+    if (!setTuneDevice(prefs.tuneDevice)) {
+      logMessage(LOG_ERR, "unsupported tune device: %s", deviceNames[prefs.tuneDevice]);
       exit(3);
     }
 
-    memset(&prefs, 0, sizeof(prefs));
-    prefs.alertMessages = 0;
-    prefs.alertDots = 0;
-    prefs.alertTunes = 1;
-    switch (tuneDevice) {
-      default:
-        break;
-      case tdPcm:
-        prefs.pcmVolume = outputVolume;
-        break;
-      case tdMidi:
-        prefs.midiVolume = outputVolume;
-        break;
-      case tdFm:
-        prefs.fmVolume = outputVolume;
-        break;
-    }
-#ifdef ENABLE_MIDI_SUPPORT
-    prefs.midiInstrument = midiInstrument;
-#endif /* ENABLE_MIDI_SUPPORT */
     {
       TuneDefinition tune = {NULL, 0, elements};
       playTune(&tune);
@@ -272,7 +269,7 @@ main (int argc, char *argv[]) {
 }
 
 int
-message (const char *string, short flags) {
+message (const char *mode, const char *text, short flags) {
   return 1;
 }
 int
