@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2009 by The BRLTTY Developers.
+ * Copyright (C) 1995-2011 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -28,43 +28,45 @@
 #include <regex.h>
 #endif /* HAVE_REGEX_H */
 
-#include "misc.h"
+#include "log.h"
+#include "device.h"
+#include "timing.h"
 #include "io_usb.h"
 #include "usb_internal.h"
 
-int
+ssize_t
 usbControlRead (
   UsbDevice *device,
-  unsigned char recipient,
-  unsigned char type,
-  unsigned char request,
-  unsigned short value,
-  unsigned short index,
+  uint8_t recipient,
+  uint8_t type,
+  uint8_t request,
+  uint16_t value,
+  uint16_t index,
   void *buffer,
-  int length,
+  uint16_t length,
   int timeout
 ) {
   return usbControlTransfer(device, UsbControlDirection_Input, recipient, type,
                             request, value, index, buffer, length, timeout);
 }
 
-int
+ssize_t
 usbControlWrite (
   UsbDevice *device,
-  unsigned char recipient,
-  unsigned char type,
-  unsigned char request,
-  unsigned short value,
-  unsigned short index,
+  uint8_t recipient,
+  uint8_t type,
+  uint8_t request,
+  uint16_t value,
+  uint16_t index,
   const void *buffer,
-  int length,
+  uint16_t length,
   int timeout
 ) {
   return usbControlTransfer(device, UsbControlDirection_Output, recipient, type,
                             request, value, index, (void *)buffer, length, timeout);
 }
 
-int
+ssize_t
 usbGetDescriptor (
   UsbDevice *device,
   unsigned char type,
@@ -105,7 +107,7 @@ usbGetLanguage (
   if (size != -1) {
     if (size >= 4) {
       *language = getLittleEndian(descriptor.string.wData[0]);
-      LogPrint(LOG_DEBUG, "USB Language: %02X", *language);
+      logMessage(LOG_DEBUG, "USB Language: %02X", *language);
       return 1;
     }
     errno = EIO;
@@ -125,7 +127,7 @@ usbDecodeString (const UsbStringDescriptor *descriptor) {
       string[count] = character;
     }
   } else {
-    LogError("USB string allocate");
+    logSystemError("USB string allocate");
   }
   return string;
 }
@@ -174,7 +176,7 @@ usbLogString (
   if (number) {
     char *string = usbGetString(device, number, 1000);
     if (string) {
-      LogPrint(LOG_INFO, "USB: %s: %s", description, string);
+      logMessage(LOG_INFO, "USB: %s: %s", description, string);
       free(string);
     }
   }
@@ -250,11 +252,11 @@ usbGetConfiguration (
   UsbDevice *device,
   unsigned char *configuration
 ) {
-  int size = usbControlRead(device, UsbControlRecipient_Device, UsbControlType_Standard,
-                            UsbStandardRequest_GetConfiguration, 0, 0,
-                            configuration, sizeof(*configuration), 1000);
+  ssize_t size = usbControlRead(device, UsbControlRecipient_Device, UsbControlType_Standard,
+                                UsbStandardRequest_GetConfiguration, 0, 0,
+                                configuration, sizeof(*configuration), 1000);
   if (size != -1) return 1;
-  LogPrint(LOG_WARNING, "USB standard request not supported: get configuration");
+  logMessage(LOG_WARNING, "USB standard request not supported: get configuration");
   return 0;
 }
 
@@ -287,7 +289,7 @@ usbConfigurationDescriptor (
         int size = usbGetDescriptor(device, UsbDescriptorType_Configuration,
                                     number, 0, &descriptor, 1000);
         if (size == -1) {
-          LogPrint(LOG_WARNING, "USB configuration descriptor not readable: %d", number);
+          logMessage(LOG_WARNING, "USB configuration descriptor not readable: %d", number);
         } else if (descriptor.configuration.bConfigurationValue == current) {
           break;
         }
@@ -298,7 +300,7 @@ usbConfigurationDescriptor (
         UsbDescriptor *descriptors;
 
         if ((descriptors = malloc(length))) {
-          int size;
+          ssize_t size;
 
           if (length > sizeof(descriptor)) {
             size = usbControlRead(device, UsbControlRecipient_Device, UsbControlType_Standard,
@@ -315,10 +317,10 @@ usbConfigurationDescriptor (
             free(descriptors);
           }
         } else {
-          LogError("USB configuration descriptor allocate");
+          logSystemError("USB configuration descriptor allocate");
         }
       } else {
-        LogPrint(LOG_ERR, "USB configuration descriptor not found: %d", current);
+        logMessage(LOG_ERR, "USB configuration descriptor not found: %d", current);
       }
     }
   }
@@ -384,7 +386,7 @@ usbInterfaceDescriptor (
           return &descriptor->interface;
   }
 
-  LogPrint(LOG_WARNING, "USB: interface descriptor not found: %d.%d", interface, alternative);
+  logMessage(LOG_WARNING, "USB: interface descriptor not found: %d.%d", interface, alternative);
   errno = ENOENT;
   return NULL;
 }
@@ -419,7 +421,7 @@ usbEndpointDescriptor (
         return &descriptor->endpoint;
   }
 
-  LogPrint(LOG_WARNING, "USB: endpoint descriptor not found: %02X", endpointAddress);
+  logMessage(LOG_WARNING, "USB: endpoint descriptor not found: %02X", endpointAddress);
   errno = ENOENT;
   return NULL;
 }
@@ -452,9 +454,9 @@ usbDeallocateEndpoint (void *item, void *data) {
 }
 
 static int
-usbTestEndpoint (void *item, void *data) {
-  UsbEndpoint *endpoint = item;
-  unsigned char *endpointAddress = data;
+usbTestEndpoint (const void *item, const void *data) {
+  const UsbEndpoint *endpoint = item;
+  const unsigned char *endpointAddress = data;
   return endpoint->descriptor->bEndpointAddress == *endpointAddress;
 }
 
@@ -484,10 +486,10 @@ usbGetEndpoint (UsbDevice *device, unsigned char endpointAddress) {
         case UsbEndpointTransfer_Interrupt:   transfer = "int"; break;
       }
 
-      LogPrint(LOG_DEBUG, "USB: ept=%02X dir=%s xfr=%s pkt=%d ivl=%dms",
-               descriptor->bEndpointAddress, direction, transfer,
-               getLittleEndian(descriptor->wMaxPacketSize),
-               descriptor->bInterval);
+      logMessage(LOG_DEBUG, "USB: ept=%02X dir=%s xfr=%s pkt=%d ivl=%dms",
+                 descriptor->bEndpointAddress, direction, transfer,
+                 getLittleEndian(descriptor->wMaxPacketSize),
+                 descriptor->bInterval);
     }
 
     if ((endpoint = malloc(sizeof(*endpoint)))) {
@@ -551,12 +553,18 @@ usbApplyInputFilter (void *item, void *data) {
 }
 
 int
-usbApplyInputFilters (UsbDevice *device, void *buffer, int size, int *length) {
-  UsbInputFilterData data = {buffer, size, *length};
+usbApplyInputFilters (UsbDevice *device, void *buffer, size_t size, ssize_t *length) {
+  UsbInputFilterData data = {
+    .buffer = buffer,
+    .size = size,
+    .length = *length
+  };
+
   if (processQueue(device->inputFilters, usbApplyInputFilter, &data)) {
     errno = EIO;
     return 0;
   }
+
   *length = data.length;
   return 1;
 }
@@ -595,14 +603,14 @@ usbOpenInterface (
 
   {
     unsigned char response[1];
-    int size = usbControlRead(device, UsbControlRecipient_Interface, UsbControlType_Standard,
-                              UsbStandardRequest_GetInterface, 0, interface,
-                              response, sizeof(response), 1000);
+    ssize_t size = usbControlRead(device, UsbControlRecipient_Interface, UsbControlType_Standard,
+                                  UsbStandardRequest_GetInterface, 0, interface,
+                                  response, sizeof(response), 1000);
 
     if (size != -1) {
       if (response[0] == alternative) goto done;
     } else {
-      LogPrint(LOG_WARNING, "USB standard request not supported: get interface");
+      logMessage(LOG_WARNING, "USB standard request not supported: get interface");
     }
   }
 
@@ -659,7 +667,7 @@ usbOpenDevice (UsbDeviceExtension *extension) {
     free(device);
   }
 
-  LogError("USB device open");
+  logSystemError("USB device open");
   return NULL;
 }
 
@@ -667,9 +675,9 @@ UsbDevice *
 usbTestDevice (UsbDeviceExtension *extension, UsbDeviceChooser chooser, void *data) {
   UsbDevice *device;
   if ((device = usbOpenDevice(extension))) {
-    LogPrint(LOG_DEBUG, "USB: testing: vendor=%04X product=%04X",
-             device->descriptor.idVendor,
-             device->descriptor.idProduct);
+    logMessage(LOG_DEBUG, "USB: testing: vendor=%04X product=%04X",
+               device->descriptor.idVendor,
+               device->descriptor.idProduct);
     if (chooser(device, data)) {
       usbLogString(device, device->descriptor.iManufacturer, "Manufacturer Name");
       usbLogString(device, device->descriptor.iProduct, "Product Description");
@@ -751,7 +759,7 @@ usbAwaitInput (
     int size = getLittleEndian(endpoint->descriptor->wMaxPacketSize);
     unsigned char *buffer = malloc(size);
     if (buffer) {
-      int count;
+      ssize_t count;
 
       if (timeout) hasTimedOut(0);
       while (1) {
@@ -810,12 +818,12 @@ usbAwaitInput (
   }
 }
 
-int
+ssize_t
 usbReapInput (
   UsbDevice *device,
   unsigned char endpointNumber,
   void *buffer,
-  int length,
+  size_t length,
   int initialTimeout,
   int subsequentTimeout
 ) {
@@ -825,10 +833,13 @@ usbReapInput (
     unsigned char *target = bytes;
     while (length > 0) {
       if (!usbAwaitInput(device, endpointNumber,
-                         (target == bytes)? initialTimeout: subsequentTimeout)) return -1;
+                         (target == bytes)? initialTimeout: subsequentTimeout)) {
+        if (errno == EAGAIN) break;
+        return -1;
+      }
 
       {
-        int count = endpoint->direction.input.length;
+        size_t count = endpoint->direction.input.length;
         if (length < count) count = length;
         memcpy(target, endpoint->direction.input.buffer, count);
 
@@ -849,13 +860,196 @@ usbReapInput (
   return -1;
 }
 
+const unsigned char usbHidItemLengths[] = {0, 1, 2, 4};
+
+const UsbHidDescriptor *
+usbHidDescriptor (UsbDevice *device) {
+  const UsbDescriptor *descriptor = NULL;
+
+  while (usbNextDescriptor(device, &descriptor))
+    if (descriptor->endpoint.bDescriptorType == UsbDescriptorType_HID)
+      return &descriptor->hid;
+
+  logMessage(LOG_WARNING, "USB: HID descriptor not found");
+  errno = ENOENT;
+  return NULL;
+}
+
+ssize_t
+usbHidGetItems (
+  UsbDevice *device,
+  unsigned char interface,
+  unsigned char number,
+  unsigned char **items,
+  int timeout
+) {
+  const UsbHidDescriptor *hid = usbHidDescriptor(device);
+
+  if (hid) {
+    if (number < hid->bNumDescriptors) {
+      const UsbClassDescriptor *descriptor = &hid->descriptors[number];
+      uint16_t length = getLittleEndian(descriptor->wDescriptorLength);
+      void *buffer = malloc(length);
+
+      if (buffer) {
+        ssize_t result = usbControlRead(device,
+                                        UsbControlRecipient_Interface, UsbControlType_Standard,
+                                        UsbStandardRequest_GetDescriptor,
+                                        (descriptor->bDescriptorType << 8) | interface,
+                                        number, buffer, length, timeout);
+
+        if (result != -1) {
+          *items = buffer;
+          return result;
+        }
+
+        free(buffer);
+      } else {
+        logMallocError();
+      }
+    } else {
+      logMessage(LOG_WARNING, "USB report descriptor not found: %u[%u]",
+                 interface, number);
+    }
+  }
+
+  return -1;
+}
+
 int
+usbHidFillReportDescription (
+  const unsigned char *items,
+  size_t size,
+  unsigned char identifier,
+  UsbHidReportDescription *description
+) {
+  int found = 0;
+  int index = 0;
+
+  while (index < size) {
+    unsigned char item = items[index++];
+    UsbHidItemType type = USB_HID_ITEM_TYPE(item);
+    unsigned char length = usbHidItemLengths[USB_HID_ITEM_LENGTH(item)];
+    uint32_t value = 0;
+
+    if (length) {
+      unsigned char shift = 0;
+
+      do {
+        if (index == size) return 0;
+        value |= items[index++] << shift;
+        shift += 8;
+      } while (--length);
+    }
+
+    switch (type) {
+      case UsbHidItemType_ReportID:
+        if (!found && (value == identifier)) {
+          memset(description, 0, sizeof(*description));
+          description->reportIdentifier = identifier;
+
+          found = 1;
+          continue;
+        }
+        break;
+
+      case UsbHidItemType_ReportCount:
+        if (found) {
+          description->reportCount = value;
+          goto defined;
+        }
+        break;
+
+      case UsbHidItemType_ReportSize:
+        if (found) {
+          description->reportSize = value;
+          goto defined;
+        }
+        break;
+
+      case UsbHidItemType_LogicalMinimum:
+        if (found) {
+          description->logicalMinimum = value;
+          goto defined;
+        }
+        break;
+
+      case UsbHidItemType_LogicalMaximum:
+        if (found) {
+          description->logicalMaximum = value;
+          goto defined;
+        }
+        break;
+
+      defined:
+        description->defined |= USB_HID_ITEM_BIT(type);
+        continue;
+
+      default:
+        break;
+    }
+
+    if (found) break;
+  }
+
+  return found;
+}
+
+int
+usbHidGetReportSize (
+  const unsigned char *items,
+  size_t length,
+  unsigned char identifier,
+  size_t *size
+) {
+  UsbHidReportDescription description;
+
+  *size = 0;
+
+  if (usbHidFillReportDescription(items, length, identifier, &description)) {
+    if (description.defined & USB_HID_ITEM_BIT(UsbHidItemType_ReportCount)) {
+      if (description.defined & USB_HID_ITEM_BIT(UsbHidItemType_ReportSize)) {
+        uint32_t bytes = ((description.reportCount * description.reportSize) + 7) / 8;
+
+        logMessage(LOG_DEBUG, "HID Report Size: %02X = %"PRIu32, identifier, bytes);
+        *size = 1 + bytes;
+        return 1;
+      } else {
+        logMessage(LOG_WARNING, "HID report size not defined: %02X", identifier);
+      }
+    } else {
+      logMessage(LOG_WARNING, "HID report count not defined: %02X", identifier);
+    }
+  } else {
+    logMessage(LOG_WARNING, "HID report not found: %02X", identifier);
+  }
+
+  return 0;
+}
+
+ssize_t
+usbHidGetReport (
+  UsbDevice *device,
+  unsigned char interface,
+  unsigned char report,
+  void *buffer,
+  uint16_t length,
+  int timeout
+) {
+  return usbControlRead(device,
+                        UsbControlRecipient_Interface, UsbControlType_Class,
+                        UsbHidRequest_GetReport,
+                        (UsbHidReportType_Input << 8) | report, interface,
+                        buffer, length, timeout);
+}
+
+ssize_t
 usbHidSetReport (
   UsbDevice *device,
   unsigned char interface,
   unsigned char report,
   const void *buffer,
-  int length,
+  uint16_t length,
   int timeout
 ) {
   return usbControlWrite(device,
@@ -865,13 +1059,13 @@ usbHidSetReport (
                          buffer, length, timeout);
 }
 
-int
+ssize_t
 usbHidGetFeature (
   UsbDevice *device,
   unsigned char interface,
   unsigned char report,
   void *buffer,
-  int length,
+  uint16_t length,
   int timeout
 ) {
   return usbControlRead(device,
@@ -881,13 +1075,13 @@ usbHidGetFeature (
                         buffer, length, timeout);
 }
 
-int
+ssize_t
 usbHidSetFeature (
   UsbDevice *device,
   unsigned char interface,
   unsigned char report,
   const void *buffer,
-  int length,
+  uint16_t length,
   int timeout
 ) {
   return usbControlWrite(device,
@@ -898,16 +1092,16 @@ usbHidSetFeature (
 }
 
 static int
-usbSetAttribute_Belkin (UsbDevice *device, unsigned char request, int value, int index) {
-  LogPrint(LOG_DEBUG, "Belkin Request: %02X %04X %04X", request, value, index);
+usbSetAttribute_Belkin (UsbDevice *device, unsigned char request, unsigned int value, unsigned int index) {
+  logMessage(LOG_DEBUG, "Belkin Request: %02X %04X %04X", request, value, index);
   return usbControlWrite(device, UsbControlRecipient_Device, UsbControlType_Vendor,
                          request, value, index, NULL, 0, 1000) != -1;
 }
 static int
-usbSetBaud_Belkin (UsbDevice *device, int rate) {
-  const int base = 230400;
+usbSetBaud_Belkin (UsbDevice *device, unsigned int rate) {
+  const unsigned int base = 230400;
   if (base % rate) {
-    LogPrint(LOG_WARNING, "Unsupported Belkin baud: %d", rate);
+    logMessage(LOG_WARNING, "unsupported Belkin baud: %u", rate);
     errno = EINVAL;
     return 0;
   }
@@ -915,7 +1109,7 @@ usbSetBaud_Belkin (UsbDevice *device, int rate) {
 }
 static int
 usbSetFlowControl_Belkin (UsbDevice *device, SerialFlowControl flow) {
-  int value = 0;
+  unsigned int value = 0;
 #define BELKIN_FLOW(from,to) if ((flow & (from)) == (from)) flow &= ~(from), value |= (to)
   BELKIN_FLOW(SERIAL_FLOW_OUTPUT_CTS, 0X0001);
   BELKIN_FLOW(SERIAL_FLOW_OUTPUT_DSR, 0X0002);
@@ -927,23 +1121,23 @@ usbSetFlowControl_Belkin (UsbDevice *device, SerialFlowControl flow) {
   BELKIN_FLOW(SERIAL_FLOW_INPUT_XON , 0X0100);
 #undef BELKIN_FLOW
   if (flow) {
-    LogPrint(LOG_WARNING, "Unsupported Belkin flow control: %02X", flow);
+    logMessage(LOG_WARNING, "unsupported Belkin flow control: %02X", flow);
   }
   return usbSetAttribute_Belkin(device, 16, value, 0);
 }
 static int
-usbSetDataBits_Belkin (UsbDevice *device, int bits) {
+usbSetDataBits_Belkin (UsbDevice *device, unsigned int bits) {
   if ((bits < 5) || (bits > 8)) {
-    LogPrint(LOG_WARNING, "Unsupported Belkin data bits: %d", bits);
+    logMessage(LOG_WARNING, "unsupported Belkin data bits: %u", bits);
     errno = EINVAL;
     return 0;
   }
   return usbSetAttribute_Belkin(device, 2, bits-5, 0);
 }
 static int
-usbSetStopBits_Belkin (UsbDevice *device, int bits) {
+usbSetStopBits_Belkin (UsbDevice *device, unsigned int bits) {
   if ((bits < 1) || (bits > 2)) {
-    LogPrint(LOG_WARNING, "Unsupported Belkin stop bits: %d", bits);
+    logMessage(LOG_WARNING, "unsupported Belkin stop bits: %u", bits);
     errno = EINVAL;
     return 0;
   }
@@ -951,7 +1145,7 @@ usbSetStopBits_Belkin (UsbDevice *device, int bits) {
 }
 static int
 usbSetParity_Belkin (UsbDevice *device, SerialParity parity) {
-  int value;
+  unsigned int value;
   switch (parity) {
     case SERIAL_PARITY_SPACE: value = 4; break;
     case SERIAL_PARITY_ODD:   value = 2; break;
@@ -959,14 +1153,14 @@ usbSetParity_Belkin (UsbDevice *device, SerialParity parity) {
     case SERIAL_PARITY_MARK:  value = 3; break;
     case SERIAL_PARITY_NONE:  value = 0; break;
     default:
-      LogPrint(LOG_WARNING, "Unsupported Belkin parity: %d", parity);
+      logMessage(LOG_WARNING, "unsupported Belkin parity: %u", parity);
       errno = EINVAL;
       return 0;
   }
   return usbSetAttribute_Belkin(device, 3, value, 0);
 }
 static int
-usbSetDataFormat_Belkin (UsbDevice *device, int dataBits, int stopBits, SerialParity parity) {
+usbSetDataFormat_Belkin (UsbDevice *device, unsigned int dataBits, unsigned int stopBits, SerialParity parity) {
   if (usbSetDataBits_Belkin(device, dataBits))
     if (usbSetStopBits_Belkin(device, stopBits))
       if (usbSetParity_Belkin(device, parity))
@@ -976,7 +1170,7 @@ usbSetDataFormat_Belkin (UsbDevice *device, int dataBits, int stopBits, SerialPa
 static int
 usbSetDtrState_Belkin (UsbDevice *device, int state) {
   if ((state < 0) || (state > 1)) {
-    LogPrint(LOG_WARNING, "Unsupported Belkin DTR state: %d", state);
+    logMessage(LOG_WARNING, "Unsupported Belkin DTR state: %d", state);
     errno = EINVAL;
     return 0;
   }
@@ -985,7 +1179,7 @@ usbSetDtrState_Belkin (UsbDevice *device, int state) {
 static int
 usbSetRtsState_Belkin (UsbDevice *device, int state) {
   if ((state < 0) || (state > 1)) {
-    LogPrint(LOG_WARNING, "Unsupported Belkin RTS state: %d", state);
+    logMessage(LOG_WARNING, "Unsupported Belkin RTS state: %d", state);
     errno = EINVAL;
     return 0;
   }
@@ -1012,18 +1206,18 @@ usbInputFilter_FTDI (UsbInputFilterData *data) {
   return 1;
 }
 static int
-usbSetAttribute_FTDI (UsbDevice *device, unsigned char request, int value, int index) {
-  LogPrint(LOG_DEBUG, "FTDI Request: %02X %04X %04X", request, value, index);
+usbSetAttribute_FTDI (UsbDevice *device, unsigned char request, unsigned int value, unsigned int index) {
+  logMessage(LOG_DEBUG, "FTDI Request: %02X %04X %04X", request, value, index);
   return usbControlWrite(device, UsbControlRecipient_Device, UsbControlType_Vendor,
                          request, value, index, NULL, 0, 1000) != -1;
 }
 static int
-usbSetBaud_FTDI (UsbDevice *device, int divisor) {
+usbSetBaud_FTDI (UsbDevice *device, unsigned int divisor) {
   return usbSetAttribute_FTDI(device, 3, divisor&0XFFFF, divisor>>0X10);
 }
 static int
-usbSetBaud_FTDI_SIO (UsbDevice *device, int rate) {
-  int divisor;
+usbSetBaud_FTDI_SIO (UsbDevice *device, unsigned int rate) {
+  unsigned int divisor;
   switch (rate) {
     case    300: divisor = 0; break;
     case    600: divisor = 1; break;
@@ -1036,23 +1230,23 @@ usbSetBaud_FTDI_SIO (UsbDevice *device, int rate) {
     case  57600: divisor = 8; break;
     case 115200: divisor = 9; break;
     default:
-      LogPrint(LOG_WARNING, "Unsupported FTDI SIO baud: %d", rate);
+      logMessage(LOG_WARNING, "unsupported FTDI SIO baud: %u", rate);
       errno = EINVAL;
       return 0;
   }
   return usbSetBaud_FTDI(device, divisor);
 }
 static int
-usbSetBaud_FTDI_FT8U232AM (UsbDevice *device, int rate) {
+usbSetBaud_FTDI_FT8U232AM (UsbDevice *device, unsigned int rate) {
   if (rate > 3000000) {
-    LogPrint(LOG_WARNING, "Unsupported FTDI FT8U232AM baud: %d", rate);
+    logMessage(LOG_WARNING, "unsupported FTDI FT8U232AM baud: %u", rate);
     errno = EINVAL;
     return 0;
   }
   {
-    const int base = 48000000;
-    int eighths = base / 2 / rate;
-    int divisor;
+    const unsigned int base = 48000000;
+    unsigned int eighths = base / 2 / rate;
+    unsigned int divisor;
     if ((eighths & 07) == 7) eighths++;
     divisor = eighths >> 3;
     divisor |= (eighths & 04)? 0X4000:
@@ -1064,17 +1258,17 @@ usbSetBaud_FTDI_FT8U232AM (UsbDevice *device, int rate) {
   }
 }
 static int
-usbSetBaud_FTDI_FT232BM (UsbDevice *device, int rate) {
+usbSetBaud_FTDI_FT232BM (UsbDevice *device, unsigned int rate) {
   if (rate > 3000000) {
-    LogPrint(LOG_WARNING, "Unsupported FTDI FT232BM baud: %d", rate);
+    logMessage(LOG_WARNING, "unsupported FTDI FT232BM baud: %u", rate);
     errno = EINVAL;
     return 0;
   }
   {
     static const unsigned char mask[8] = {00, 03, 02, 04, 01, 05, 06, 07};
-    const int base = 48000000;
-    const int eighths = base / 2 / rate;
-    int divisor = (eighths >> 3) | (mask[eighths & 07] << 14);
+    const unsigned int base = 48000000;
+    const unsigned int eighths = base / 2 / rate;
+    unsigned int divisor = (eighths >> 3) | (mask[eighths & 07] << 14);
     if (divisor == 1) {
       divisor = 0;
     } else if (divisor == 0X4001) {
@@ -1085,23 +1279,23 @@ usbSetBaud_FTDI_FT232BM (UsbDevice *device, int rate) {
 }
 static int
 usbSetFlowControl_FTDI (UsbDevice *device, SerialFlowControl flow) {
-  int index = 0;
+  unsigned int index = 0;
 #define FTDI_FLOW(from,to) if ((flow & (from)) == (from)) flow &= ~(from), index |= (to)
   FTDI_FLOW(SERIAL_FLOW_OUTPUT_CTS|SERIAL_FLOW_INPUT_RTS, 0X0100);
   FTDI_FLOW(SERIAL_FLOW_OUTPUT_DSR|SERIAL_FLOW_INPUT_DTR, 0X0200);
   FTDI_FLOW(SERIAL_FLOW_OUTPUT_XON|SERIAL_FLOW_INPUT_XON, 0X0400);
 #undef FTDI_FLOW
   if (flow) {
-    LogPrint(LOG_WARNING, "Unsupported FTDI flow control: %02X", flow);
+    logMessage(LOG_WARNING, "unsupported FTDI flow control: %02X", flow);
   }
   return usbSetAttribute_FTDI(device, 2, ((index & 0X0400)? 0X1311: 0), index);
 }
 static int
-usbSetDataFormat_FTDI (UsbDevice *device, int dataBits, int stopBits, SerialParity parity) {
+usbSetDataFormat_FTDI (UsbDevice *device, unsigned int dataBits, unsigned int stopBits, SerialParity parity) {
   int ok = 1;
-  int value = dataBits & 0XFF;
+  unsigned int value = dataBits & 0XFF;
   if (dataBits != value) {
-    LogPrint(LOG_WARNING, "Unsupported FTDI data bits: %d", dataBits);
+    logMessage(LOG_WARNING, "unsupported FTDI data bits: %u", dataBits);
     ok = 0;
   }
   switch (parity) {
@@ -1111,7 +1305,7 @@ usbSetDataFormat_FTDI (UsbDevice *device, int dataBits, int stopBits, SerialPari
     case SERIAL_PARITY_MARK:  value |= 0X300; break;
     case SERIAL_PARITY_SPACE: value |= 0X400; break;
     default:
-      LogPrint(LOG_WARNING, "Unsupported FTDI parity: %d", parity);
+      logMessage(LOG_WARNING, "unsupported FTDI parity: %u", parity);
       ok = 0;
       break;
   }
@@ -1119,7 +1313,7 @@ usbSetDataFormat_FTDI (UsbDevice *device, int dataBits, int stopBits, SerialPari
     case 1: value |= 0X0000; break;
     case 2: value |= 0X1000; break;
     default:
-      LogPrint(LOG_WARNING, "Unsupported FTDI stop bits: %d", stopBits);
+      logMessage(LOG_WARNING, "unsupported FTDI stop bits: %u", stopBits);
       ok = 0;
       break;
   }
@@ -1132,7 +1326,7 @@ usbSetDataFormat_FTDI (UsbDevice *device, int dataBits, int stopBits, SerialPari
 static int
 usbSetModemState_FTDI (UsbDevice *device, int state, int shift, const char *name) {
   if ((state < 0) || (state > 1)) {
-    LogPrint(LOG_WARNING, "Unsupported FTDI %s state: %d", name, state);
+    logMessage(LOG_WARNING, "Unsupported FTDI %s state: %d", name, state);
     errno = EINVAL;
     return 0;
   }
@@ -1171,10 +1365,10 @@ static const UsbSerialOperations usbSerialOperations_FTDI_FT232BM = {
   NULL
 };
 
-static int
-usbGetAttributes_CP2101 (UsbDevice *device, unsigned char request, void *data, int length) {
-  int result = usbControlRead(device, UsbControlRecipient_Interface, UsbControlType_Vendor,
-                              request, 0, 0, data, length, 1000);
+static ssize_t
+usbGetAttributes_CP2101 (UsbDevice *device, unsigned char request, void *data, size_t length) {
+  ssize_t result = usbControlRead(device, UsbControlRecipient_Interface, UsbControlType_Vendor,
+                                  request, 0, 0, data, length, 1000);
   if (result == -1) return 0;
 
   if (result < length) {
@@ -1182,27 +1376,27 @@ usbGetAttributes_CP2101 (UsbDevice *device, unsigned char request, void *data, i
     memset(&bytes[result], 0, length-result);
   }
 
-  LogBytes(LOG_DEBUG, "CP2101 Attributes", data, result);
+  logBytes(LOG_DEBUG, "CP2101 Attributes", data, result);
   return result;
 }
-static int
-usbSetAttributes_CP2101 (UsbDevice *device, unsigned char request, const void *data, int length) {
-  LogBytes(LOG_DEBUG, "CP2101 Attributes", data, length);
+static ssize_t
+usbSetAttributes_CP2101 (UsbDevice *device, unsigned char request, const void *data, size_t length) {
+  logBytes(LOG_DEBUG, "CP2101 Attributes", data, length);
   return usbControlWrite(device, UsbControlRecipient_Interface, UsbControlType_Vendor,
                          request, 0, 0, data, length, 1000) != -1;
 }
 static int
-usbSetAttribute_CP2101 (UsbDevice *device, unsigned char request, int value, int index) {
-  LogPrint(LOG_DEBUG, "CP2101 Request: %02X %04X %04X", request, value, index);
+usbSetAttribute_CP2101 (UsbDevice *device, unsigned char request, unsigned int value, unsigned int index) {
+  logMessage(LOG_DEBUG, "CP2101 Request: %02X %04X %04X", request, value, index);
   return usbControlWrite(device, UsbControlRecipient_Interface, UsbControlType_Vendor,
                          request, value, index, NULL, 0, 1000) != -1;
 }
 static int
-usbSetBaud_CP2101 (UsbDevice *device, int rate) {
-  const int base = 0X384000;
-  int divisor = base / rate;
+usbSetBaud_CP2101 (UsbDevice *device, unsigned int rate) {
+  const unsigned int base = 0X384000;
+  unsigned int divisor = base / rate;
   if ((rate * divisor) != base) {
-    LogPrint(LOG_WARNING, "Unsupported CP2101 baud: %d", rate);
+    logMessage(LOG_WARNING, "unsupported CP2101 baud: %u", rate);
     errno = EINVAL;
     return 0;
   }
@@ -1211,21 +1405,21 @@ usbSetBaud_CP2101 (UsbDevice *device, int rate) {
 static int
 usbSetFlowControl_CP2101 (UsbDevice *device, SerialFlowControl flow) {
   unsigned char bytes[16];
-  int count = usbGetAttributes_CP2101(device, 20, bytes, sizeof(bytes));
+  ssize_t count = usbGetAttributes_CP2101(device, 20, bytes, sizeof(bytes));
   if (!count) return 0;
 
   if (flow) {
-    LogPrint(LOG_WARNING, "Unsupported CP2101 flow control: %02X", flow);
+    logMessage(LOG_WARNING, "unsupported CP2101 flow control: %02X", flow);
   }
 
-  return usbSetAttributes_CP2101(device, 19, bytes, count);
+  return usbSetAttributes_CP2101(device, 19, bytes, count) != -1;
 }
 static int
-usbSetDataFormat_CP2101 (UsbDevice *device, int dataBits, int stopBits, SerialParity parity) {
+usbSetDataFormat_CP2101 (UsbDevice *device, unsigned int dataBits, unsigned int stopBits, SerialParity parity) {
   int ok = 1;
-  int value = dataBits & 0XF;
+  unsigned int value = dataBits & 0XF;
   if (dataBits != value) {
-    LogPrint(LOG_WARNING, "Unsupported CP2101 data bits: %d", dataBits);
+    logMessage(LOG_WARNING, "unsupported CP2101 data bits: %u", dataBits);
     ok = 0;
   }
   value <<= 8;
@@ -1236,7 +1430,7 @@ usbSetDataFormat_CP2101 (UsbDevice *device, int dataBits, int stopBits, SerialPa
     case SERIAL_PARITY_MARK:  value |= 0X30; break;
     case SERIAL_PARITY_SPACE: value |= 0X40; break;
     default:
-      LogPrint(LOG_WARNING, "Unsupported CP2101 parity: %d", parity);
+      logMessage(LOG_WARNING, "unsupported CP2101 parity: %u", parity);
       ok = 0;
       break;
   }
@@ -1244,7 +1438,7 @@ usbSetDataFormat_CP2101 (UsbDevice *device, int dataBits, int stopBits, SerialPa
     case 1: value |= 0X0; break;
     case 2: value |= 0X2; break;
     default:
-      LogPrint(LOG_WARNING, "Unsupported CP2101 stop bits: %d", stopBits);
+      logMessage(LOG_WARNING, "unsupported CP2101 stop bits: %u", stopBits);
       ok = 0;
       break;
   }
@@ -1257,7 +1451,7 @@ usbSetDataFormat_CP2101 (UsbDevice *device, int dataBits, int stopBits, SerialPa
 static int
 usbSetModemState_CP2101 (UsbDevice *device, int state, int shift, const char *name) {
   if ((state < 0) || (state > 1)) {
-    LogPrint(LOG_WARNING, "Unsupported CP2101 %s state: %d", name, state);
+    logMessage(LOG_WARNING, "Unsupported CP2101 %s state: %d", name, state);
     errno = EINVAL;
     return 0;
   }
@@ -1467,10 +1661,6 @@ usbSetSerialOperations (UsbDevice *device) {
 
       sa += 1;
     }
-
-    if (!device->serial)
-      LogPrint(LOG_DEBUG, "USB: no serial operations: vendor=%04X product=%04X",
-               device->descriptor.idVendor, device->descriptor.idProduct);
   }
 
   return 1;
@@ -1479,15 +1669,21 @@ usbSetSerialOperations (UsbDevice *device) {
 int
 usbSetSerialParameters (UsbDevice *device, const SerialParameters *parameters) {
   const UsbSerialOperations *serial = usbGetSerialOperations(device);
-  if (!serial) return 0;
+
+  if (!serial) {
+    logMessage(LOG_DEBUG, "USB: no serial operations: vendor=%04X product=%04X",
+               device->descriptor.idVendor, device->descriptor.idProduct);
+    errno = ENOSYS;
+    return 0;
+  }
 
   if (serial->enableAdapter)
     if (!serial->enableAdapter(device))
       return 0;
 
   if (!serial->setBaud(device, parameters->baud)) return 0;
-  if (!serial->setFlowControl(device, parameters->flow)) return 0;
-  if (!serial->setDataFormat(device, parameters->data, parameters->stop, parameters->parity)) return 0;
+  if (!serial->setFlowControl(device, parameters->flowControl)) return 0;
+  if (!serial->setDataFormat(device, parameters->dataBits, parameters->stopBits, parameters->parity)) return 0;
 
   return 1;
 }
@@ -1559,27 +1755,31 @@ usbChooseChannel (UsbDevice *device, void *data) {
 }
 
 UsbChannel *
-usbFindChannel (const UsbChannelDefinition *definitions, const char *device) {
-  UsbChooseChannelData choose;
-  UsbChannel *channel;
+usbFindChannel (const UsbChannelDefinition *definitions, const char *serialNumber) {
+  UsbChooseChannelData choose = {
+    .definition = definitions,
+    .serialNumber = serialNumber
+  };
+  UsbDevice *device = usbFindDevice(usbChooseChannel, &choose);
 
-  choose.definition = definitions;
-  choose.serialNumber = device;
+  if (device) {
+    UsbChannel *channel = malloc(sizeof(*channel));
 
-  if ((channel = malloc(sizeof(*channel)))) {
-    memset(channel, 0, sizeof(*channel));
-
-    if ((channel->device = usbFindDevice(usbChooseChannel, &choose))) {
+    if (channel) {
+      memset(channel, 0, sizeof(*channel));
+      channel->device = device;
       channel->definition = *choose.definition;
       return channel;
     } else {
-      LogPrint(LOG_DEBUG, "USB device not found%s%s",
-               (*device? ": ": ""),
-               device);
+      logMallocError();
     }
 
-    free(channel);
+    usbCloseDevice(device);
+  } else {
+    logMessage(LOG_DEBUG, "USB device not found%s%s",
+               (*serialNumber? ": ": ""), serialNumber);
   }
+
   return NULL;
 }
 
@@ -1590,6 +1790,6 @@ usbCloseChannel (UsbChannel *channel) {
 }
 
 int
-isUsbDevice (const char **path) {
-  return isQualifiedDevice(path, "usb");
+isUsbDevice (const char **identifier) {
+  return isQualifiedDevice(identifier, "usb");
 }
