@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2009 by The BRLTTY Developers.
+ * Copyright (C) 1995-2011 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -16,6 +16,10 @@
  * This software is maintained by Dave Mielke <dave@mielke.cc>.
  */
 
+#include <limits.h>
+
+#include "log.h"
+
 struct UsbDeviceExtensionStruct {
   char *path;
   int file;
@@ -32,7 +36,7 @@ usbSetTimeout (int file, int new, int *old) {
   if (!old || (new != *old)) {
     int arg = new;
     if (ioctl(file, USB_SET_TIMEOUT, &arg) == -1) {
-      LogError("USB timeout set");
+      logSystemError("USB timeout set");
       return 0;
     }
     if (old) *old = new;
@@ -43,21 +47,21 @@ usbSetTimeout (int file, int new, int *old) {
 static int
 usbSetShortTransfers (int file, int arg) {
   if (ioctl(file, USB_SET_SHORT_XFER, &arg) != -1) return 1;
-  LogError("USB set short transfers");
+  logSystemError("USB set short transfers");
   return 0;
 }
 
 int
 usbResetDevice (UsbDevice *device) {
   errno = ENOSYS;
-  LogError("USB device reset");
+  logSystemError("USB device reset");
   return 0;
 }
 
 int
 usbDisableAutosuspend (UsbDevice *device) {
   errno = ENOSYS;
-  LogError("USB device autosuspend disable");
+  logSystemError("USB device autosuspend disable");
   return 0;
 }
 
@@ -69,7 +73,7 @@ usbSetConfiguration (
   UsbDeviceExtension *devx = device->extension;
   int arg = configuration;
   if (ioctl(devx->file, USB_SET_CONFIG, &arg) != -1) return 1;
-  LogError("USB configuration set");
+  logSystemError("USB configuration set");
   return 0;
 }
 
@@ -81,7 +85,7 @@ usbClaimInterface (
   return 1;
 /*
   errno = ENOSYS;
-  LogError("USB interface claim");
+  logSystemError("USB interface claim");
   return 0;
 */
 }
@@ -94,7 +98,7 @@ usbReleaseInterface (
   return 1;
 /*
   errno = ENOSYS;
-  LogError("USB interface release");
+  logSystemError("USB interface release");
   return 0;
 */
 }
@@ -111,7 +115,7 @@ usbSetAlternative (
   arg.uai_interface_index = interface;
   arg.uai_alt_no = alternative;
   if (ioctl(devx->file, USB_SET_ALTINTERFACE, &arg) != -1) return 1;
-  LogError("USB alternative set");
+  logSystemError("USB alternative set");
   return 0;
 }
 
@@ -121,21 +125,21 @@ usbClearEndpoint (
   unsigned char endpointAddress
 ) {
   errno = ENOSYS;
-  LogError("USB endpoint clear");
+  logSystemError("USB endpoint clear");
   return 0;
 }
 
-int
+ssize_t
 usbControlTransfer (
   UsbDevice *device,
-  unsigned char direction,
-  unsigned char recipient,
-  unsigned char type,
-  unsigned char request,
-  unsigned short value,
-  unsigned short index,
+  uint8_t direction,
+  uint8_t recipient,
+  uint8_t type,
+  uint8_t request,
+  uint16_t value,
+  uint16_t index,
   void *buffer,
-  int length,
+  uint16_t length,
   int timeout
 ) {
   UsbDeviceExtension *devx = device->extension;
@@ -150,7 +154,7 @@ usbControlTransfer (
   arg.ucr_flags = USBD_SHORT_XFER_OK;
   if (usbSetTimeout(devx->file, timeout, &devx->timeout)) {
     if (ioctl(devx->file, USB_DO_REQUEST, &arg) != -1) return arg.ucr_actlen;
-    LogError("USB control transfer");
+    logSystemError("USB control transfer");
   }
   return -1;
 }
@@ -160,11 +164,11 @@ usbSubmitRequest (
   UsbDevice *device,
   unsigned char endpointAddress,
   void *buffer,
-  int length,
+  size_t length,
   void *context
 ) {
   errno = ENOSYS;
-  LogError("USB request submit");
+  logSystemError("USB request submit");
   return NULL;
 }
 
@@ -174,7 +178,7 @@ usbCancelRequest (
   void *request
 ) {
   errno = ENOSYS;
-  LogError("USB request cancel");
+  logSystemError("USB request cancel");
   return 0;
 }
 
@@ -186,19 +190,19 @@ usbReapResponse (
   int wait
 ) {
   errno = ENOSYS;
-  LogError("USB request reap");
+  logSystemError("USB request reap");
   return NULL;
 }
 
-int
+ssize_t
 usbReadEndpoint (
   UsbDevice *device,
   unsigned char endpointNumber,
   void *buffer,
-  int length,
+  size_t length,
   int timeout
 ) {
-  int count = -1;
+  ssize_t count = -1;
   UsbEndpoint *endpoint = usbGetInputEndpoint(device, endpointNumber);
   if (endpoint) {
     UsbEndpointExtension *eptx = endpoint->extension;
@@ -209,28 +213,28 @@ usbReadEndpoint (
           count = -1;
         }
       } else if (errno != ETIMEDOUT) {
-        LogError("USB endpoint read");
+        logSystemError("USB endpoint read");
       }
     }
   }
   return count;
 }
 
-int
+ssize_t
 usbWriteEndpoint (
   UsbDevice *device,
   unsigned char endpointNumber,
   const void *buffer,
-  int length,
+  size_t length,
   int timeout
 ) {
   UsbEndpoint *endpoint = usbGetOutputEndpoint(device, endpointNumber);
   if (endpoint) {
     UsbEndpointExtension *eptx = endpoint->extension;
     if (usbSetTimeout(eptx->file, timeout, &eptx->timeout)) {
-      int count = write(eptx->file, buffer, length);
+      ssize_t count = write(eptx->file, buffer, length);
       if (count != -1) return count;
-      LogError("USB endpoint write");
+      logSystemError("USB endpoint write");
     }
   }
   return -1;
@@ -246,7 +250,7 @@ usbReadDeviceDescriptor (UsbDevice *device) {
     device->descriptor.bcdDevice = getLittleEndian(device->descriptor.bcdDevice);
     return 1;
   }
-  LogError("USB device descriptor read");
+  logSystemError("USB device descriptor read");
   return 0;
 }
 
@@ -327,14 +331,14 @@ usbFindDevice (UsbDeviceChooser chooser, void *data) {
           static const char *driver = "ugen";
           const char *deviceName = info.udi_devnames[0];
 
-          LogPrint(LOG_DEBUG, "USB device [%d,%d]: vendor=%s product=%s",
-                   busNumber, deviceNumber, info.udi_vendor, info.udi_product);
+          logMessage(LOG_DEBUG, "USB device [%d,%d]: vendor=%s product=%s",
+                     busNumber, deviceNumber, info.udi_vendor, info.udi_product);
           {
             int nameNumber;
             for (nameNumber=0; nameNumber<USB_MAX_DEVNAMES; nameNumber++) {
               const char *name = info.udi_devnames[nameNumber];
               if (*name)
-                LogPrint(LOG_DEBUG, "USB name %d: %s", nameNumber, name);
+                logMessage(LOG_DEBUG, "USB name %d: %s", nameNumber, name);
             }
           }
 
@@ -361,16 +365,20 @@ usbFindDevice (UsbDeviceChooser chooser, void *data) {
             }
           }
         } else if (errno != ENXIO) {
-          LogError("USB device query");
+          logSystemError("USB device query");
         }
       }
       close(bus);
     } else if (errno == ENOENT) {
       break;
     } else if (errno != ENXIO) {
-      LogError("USB bus open");
+      logSystemError("USB bus open");
     }
     busNumber++;
   }
   return device;
 }
+
+void
+usbForgetDevices (void) {
+}
