@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2009 by The BRLTTY Developers.
+ * Copyright (C) 1995-2011 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -25,7 +25,7 @@
 #include <IOKit/IOCFPlugIn.h>
 #include <IOKit/usb/IOUSBLib.h>
 
-#include "misc.h"
+#include "log.h"
 #include "io_usb.h"
 #include "usb_internal.h"
 
@@ -33,10 +33,10 @@ typedef struct {
   UsbEndpoint *endpoint;
   void *context;
   void *buffer;
-  int length;
+  size_t length;
 
   IOReturn result;
-  int count;
+  ssize_t count;
 } UsbAsynchronousRequest;
 
 struct UsbDeviceExtensionStruct {
@@ -207,8 +207,8 @@ setUnixError (long int result, const char *action) {
 #undef SET
 
   if (action) {
-    LogPrint(LOG_WARNING, "Darwin error 0X%lX.", result);
-    LogError(action);
+    logMessage(LOG_WARNING, "Darwin error 0X%lX.", result);
+    logSystemError(action);
   }
 }
 
@@ -236,8 +236,8 @@ openDevice (UsbDevice *device, int seize) {
       level = LOG_NOTICE;
     }
 
-    LogPrint(level, "USB device %s: vendor=%04X product=%04X",
-             action, device->descriptor.idVendor, device->descriptor.idProduct);
+    logMessage(level, "USB device %s: vendor=%04X product=%04X",
+               action, device->descriptor.idVendor, device->descriptor.idProduct);
     devx->deviceOpened = 1;
   }
 
@@ -360,7 +360,7 @@ setInterface (UsbDeviceExtension *devx, UInt8 number) {
         setUnixError(result, "USB interface service plugin create");
       }
     }
-    if (!found) LogPrint(LOG_ERR, "USB interface not found: %d", number);
+    if (!found) logMessage(LOG_ERR, "USB interface not found: %d", number);
 
     IOObjectRelease(iterator);
     iterator = 0;
@@ -381,7 +381,7 @@ usbAsynchronousRequestCallback (void *context, IOReturn result, void *arg) {
   request->count = (UInt32)arg;
 
   if (!enqueueItem(eptx->completedRequests, request)) {
-    LogError("USB completed request enqueue");
+    logSystemError("USB completed request enqueue");
     free(request);
   }
 }
@@ -399,7 +399,7 @@ usbResetDevice (UsbDevice *device) {
 int
 usbDisableAutosuspend (UsbDevice *device) {
   errno = ENOSYS;
-  LogError("USB device autosuspend disable");
+  logSystemError("USB device autosuspend disable");
   return 0;
 }
 
@@ -507,17 +507,17 @@ usbClearEndpoint (
   return 0;
 }
 
-int
+ssize_t
 usbControlTransfer (
   UsbDevice *device,
-  unsigned char direction,
-  unsigned char recipient,
-  unsigned char type,
-  unsigned char request,
-  unsigned short value,
-  unsigned short index,
+  uint8_t direction,
+  uint8_t recipient,
+  uint8_t type,
+  uint8_t request,
+  uint16_t value,
+  uint16_t index,
   void *buffer,
-  int length,
+  uint16_t length,
   int timeout
 ) {
   UsbDeviceExtension *devx = device->extension;
@@ -545,7 +545,7 @@ usbSubmitRequest (
   UsbDevice *device,
   unsigned char endpointAddress,
   void *buffer,
-  int length,
+  size_t length,
   void *context
 ) {
   UsbDeviceExtension *devx = device->extension;
@@ -600,15 +600,15 @@ usbSubmitRequest (
           break;
 
         default:
-          LogPrint(LOG_ERR, "USB endpoint direction not suppported: %d",
-                   eptx->transferDirection);
+          logMessage(LOG_ERR, "USB endpoint direction not suppported: %d",
+                     eptx->transferDirection);
           errno = ENOSYS;
           break;
       }
 
       free(request);
     } else {
-      LogError("USB asynchronous request allocate");
+      logSystemError("USB asynchronous request allocate");
     }
   }
 
@@ -678,12 +678,12 @@ none:
   return NULL;
 }
 
-int
+ssize_t
 usbReadEndpoint (
   UsbDevice *device,
   unsigned char endpointNumber,
   void *buffer,
-  int length,
+  size_t length,
   int timeout
 ) {
   UsbDeviceExtension *devx = device->extension;
@@ -733,12 +733,12 @@ usbReadEndpoint (
   return -1;
 }
 
-int
+ssize_t
 usbWriteEndpoint (
   UsbDevice *device,
   unsigned char endpointNumber,
   const void *buffer,
-  int length,
+  size_t length,
   int timeout
 ) {
   UsbDeviceExtension *devx = device->extension;
@@ -817,10 +817,10 @@ usbAllocateEndpointExtension (UsbEndpoint *endpoint) {
           if ((eptx->endpointNumber == number) &&
               (((eptx->transferDirection == kUSBIn) && (direction == UsbEndpointDirection_Input)) ||
                ((eptx->transferDirection == kUSBOut) && (direction == UsbEndpointDirection_Output)))) {
-            LogPrint(LOG_DEBUG, "USB: ept=%02X -> pip=%d (num=%d dir=%d xfr=%d int=%d pkt=%d)",
-                     endpoint->descriptor->bEndpointAddress, eptx->pipeNumber,
-                     eptx->endpointNumber, eptx->transferDirection, eptx->transferMode,
-                     eptx->pollInterval, eptx->packetSize);
+            logMessage(LOG_DEBUG, "USB: ept=%02X -> pip=%d (num=%d dir=%d xfr=%d int=%d pkt=%d)",
+                       endpoint->descriptor->bEndpointAddress, eptx->pipeNumber,
+                       eptx->endpointNumber, eptx->transferDirection, eptx->transferMode,
+                       eptx->pollInterval, eptx->packetSize);
 
             eptx->endpoint = endpoint;
             endpoint->extension = eptx;
@@ -832,17 +832,17 @@ usbAllocateEndpointExtension (UsbEndpoint *endpoint) {
       }
 
       errno = EIO;
-      LogPrint(LOG_ERR, "USB pipe not found: ept=%02X",
-               endpoint->descriptor->bEndpointAddress);
+      logMessage(LOG_ERR, "USB pipe not found: ept=%02X",
+                 endpoint->descriptor->bEndpointAddress);
 
       deallocateQueue(eptx->completedRequests);
     } else {
-      LogError("USB completed request queue allocate");
+      logSystemError("USB completed request queue allocate");
     }
 
     free(eptx);
   } else {
-    LogError("USB endpoint extension allocate");
+    logSystemError("USB endpoint extension allocate");
   }
 
   return 0;
@@ -934,7 +934,7 @@ usbFindDevice (UsbDeviceChooser chooser, void *data) {
                 free(devx);
                 devx = NULL;
               } else {
-                LogError("USB device extension allocate");
+                logSystemError("USB device extension allocate");
               }
 
               (*interface)->Release(interface);
@@ -953,7 +953,7 @@ usbFindDevice (UsbDeviceChooser chooser, void *data) {
         setUnixError(kernelResult, "USB device iterator create");
       }
     } else {
-      LogPrint(LOG_ERR, "USB device matching dictionary create error.");
+      logMessage(LOG_ERR, "USB device matching dictionary create error.");
     }
 
     mach_port_deallocate(mach_task_self(), port);
@@ -963,3 +963,7 @@ usbFindDevice (UsbDeviceChooser chooser, void *data) {
 
   return device;
 }
+
+void
+usbForgetDevices (void) {
+}
