@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2009 by The BRLTTY Developers.
+ * Copyright (C) 1995-2011 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -19,12 +19,13 @@
 #ifndef BRLTTY_INCLUDED_USB_INTERNAL
 #define BRLTTY_INCLUDED_USB_INTERNAL
 
+#include "bitfield.h"
+#include "queue.h"
+
 #ifdef __cplusplus
 extern "C" {
 #endif /* __cplusplus */
 
-#include "queue.h"
-
 typedef struct {
   UsbInputFilter filter;
 } UsbInputFilterEntry;
@@ -41,7 +42,7 @@ typedef struct {
       Queue *pending;
       void *completed;
       unsigned char *buffer;
-      int length;
+      size_t length;
     } input;
 
     struct {
@@ -71,7 +72,7 @@ extern UsbDevice *usbTestDevice (
 extern UsbEndpoint *usbGetEndpoint (UsbDevice *device, unsigned char endpointAddress);
 extern UsbEndpoint *usbGetInputEndpoint (UsbDevice *device, unsigned char endpointNumber);
 extern UsbEndpoint *usbGetOutputEndpoint (UsbDevice *device, unsigned char endpointNumber);
-extern int usbApplyInputFilters (UsbDevice *device, void *buffer, int size, int *length);
+extern int usbApplyInputFilters (UsbDevice *device, void *buffer, size_t size, ssize_t *length);
 
 extern int usbSetConfiguration (
   UsbDevice *device,
@@ -90,16 +91,16 @@ extern int usbSetAlternative (
   unsigned char interface,
   unsigned char alternative
 );
-extern int usbControlTransfer (
+extern ssize_t usbControlTransfer (
   UsbDevice *device,
-  unsigned char direction,
-  unsigned char recipient,
-  unsigned char type,
-  unsigned char request,
-  unsigned short value,
-  unsigned short index,
+  uint8_t direction,
+  uint8_t recipient,
+  uint8_t type,
+  uint8_t request,
+  uint16_t value,
+  uint16_t index,
   void *buffer,
-  int length,
+  uint16_t length,
   int timeout
 );
 extern int usbReadDeviceDescriptor (UsbDevice *device);
