@@ -2,13 +2,14 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2007 by The BRLTTY Developers.
+ * Copyright (C) 1995-2011 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
  * This is free software, placed under the terms of the
  * GNU General Public License, as published by the Free Software
- * Foundation.  Please see the file COPYING for details.
+ * Foundation; either version 2 of the License, or (at your option) any
+ * later version. Please see the file LICENSE-GPL for details.
  *
  * Web Page: http://mielke.cc/brltty/
  *
@@ -23,9 +24,13 @@
 #include <fcntl.h>
 #include <sys/ioctl.h>
 #include <sys/param.h>
-#include <dev/usb/usb.h>
 
-#include "misc.h"
+#ifdef HAVE_LEGACY_DEV_USB_USB_H
+#include <legacy/dev/usb/usb.h>
+#else /* HAVE_LEGACY_DEV_USB_USB_H */
+#include <dev/usb/usb.h>
+#endif /* HAVE_LEGACY_DEV_USB_USB_H */
+
 #include "io_usb.h"
 #include "usb_internal.h"
 
