@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2010 by The BRLTTY Developers.
+ * Copyright (C) 1995-2011 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -67,8 +67,8 @@ static int
 usbOpenUsbfsFile (UsbDeviceExtension *devx) {
   if (devx->usbfsFile == -1) {
     if ((devx->usbfsFile = open(devx->host->usbfsPath, O_RDWR)) == -1) {
-      LogPrint(LOG_ERR, "USBFS open error: %s: %s",
-               devx->host->usbfsPath, strerror(errno));
+      logMessage(LOG_ERR, "USBFS open error: %s: %s",
+                 devx->host->usbfsPath, strerror(errno));
       return 0;
     }
   }
@@ -90,7 +90,7 @@ usbResetDevice (UsbDevice *device) {
 
   if (usbOpenUsbfsFile(devx)) {
     if (ioctl(devx->usbfsFile, USBDEVFS_RESET, NULL) != -1) return 1;
-    LogError("USB device reset");
+    logSystemError("USB device reset");
   }
 
   return 0;
@@ -121,7 +121,7 @@ usbDisableAutosuspend (UsbDevice *device) {
           }
 
           if (errno != EINVAL) {
-            LogPrint(LOG_ERR, "write error: %s: %s", path, strerror(errno));
+            logMessage(LOG_ERR, "write error: %s: %s", path, strerror(errno));
             break;
           }
 
@@ -130,8 +130,8 @@ usbDisableAutosuspend (UsbDevice *device) {
 
         close(file);
       } else {
-        LogPrint((errno == ENOENT)? LOG_DEBUG: LOG_ERR,
-                 "open error: %s: %s", path, strerror(errno));
+        logMessage((errno == ENOENT)? LOG_DEBUG: LOG_ERR,
+                   "open error: %s: %s", path, strerror(errno));
       }
 
       free(path);
@@ -178,7 +178,7 @@ usbControlDriver (
     arg.data = data;
 
     if (ioctl(devx->usbfsFile, USBDEVFS_IOCTL, &arg) != -1) return 1;
-    LogError("USB driver control");
+    logSystemError("USB driver control");
   }
 
   return 0;
@@ -189,7 +189,7 @@ usbDisconnectDriver (UsbDevice *device, unsigned char interface) {
 #ifdef USBDEVFS_DISCONNECT
   if (usbControlDriver(device, interface, USBDEVFS_DISCONNECT, NULL)) return 1;
 #else /* USBDEVFS_DISCONNECT */
-  LogPrint(LOG_WARNING, "USB driver disconnection not available.");
+  logMessage(LOG_WARNING, "USB driver disconnection not available.");
 #endif /* USBDEVFS_DISCONNECT */
   return 0;
 }
@@ -199,7 +199,7 @@ usbDisconnectInterface (UsbDevice *device, unsigned char interface) {
   char *driver = usbGetDriver(device, interface);
 
   if (driver) {
-    LogPrint(LOG_WARNING, "USB interface in use: %u (%s)", interface, driver);
+    logMessage(LOG_WARNING, "USB interface in use: %u (%s)", interface, driver);
     free(driver);
 
     if (usbDisconnectDriver(device, interface)) return 1;
@@ -219,7 +219,7 @@ usbSetConfiguration (
     unsigned int arg = configuration;
 
     if (ioctl(devx->usbfsFile, USBDEVFS_SETCONFIGURATION, &arg) != -1) return 1;
-    LogError("USB configuration set");
+    logSystemError("USB configuration set");
   }
 
   return 0;
@@ -249,7 +249,7 @@ usbClaimInterface (
       disconnected = 1;
     }
 
-    LogError("USB interface claim");
+    logSystemError("USB interface claim");
   }
 
   return 0;
@@ -266,7 +266,7 @@ usbReleaseInterface (
     unsigned int arg = interface;
     if (ioctl(devx->usbfsFile, USBDEVFS_RELEASEINTERFACE, &arg) != -1) return 1;
     if (errno == ENODEV) return 1;
-    LogError("USB interface release");
+    logSystemError("USB interface release");
   }
 
   return 0;
@@ -288,7 +288,7 @@ usbSetAlternative (
     arg.altsetting = alternative;
 
     if (ioctl(devx->usbfsFile, USBDEVFS_SETINTERFACE, &arg) != -1) return 1;
-    LogError("USB alternative set");
+    logSystemError("USB alternative set");
   }
 
   return 0;
@@ -305,23 +305,23 @@ usbClearEndpoint (
     unsigned int arg = endpointAddress;
 
     if (ioctl(devx->usbfsFile, USBDEVFS_CLEAR_HALT, &arg) != -1) return 1;
-    LogError("USB endpoint clear");
+    logSystemError("USB endpoint clear");
   }
 
   return 0;
 }
 
-int
+ssize_t
 usbControlTransfer (
   UsbDevice *device,
-  unsigned char direction,
-  unsigned char recipient,
-  unsigned char type,
-  unsigned char request,
-  unsigned short value,
-  unsigned short index,
+  uint8_t direction,
+  uint8_t recipient,
+  uint8_t type,
+  uint8_t request,
+  uint16_t value,
+  uint16_t index,
   void *buffer,
-  int length,
+  uint16_t length,
   int timeout
 ) {
   UsbDeviceExtension *devx = device->extension;
@@ -344,7 +344,7 @@ usbControlTransfer (
     {
       int count = ioctl(devx->usbfsFile, USBDEVFS_CONTROL, &arg);
       if (count != -1) return count;
-      LogError("USB control transfer");
+      logSystemError("USB control transfer");
     }
   }
 
@@ -371,14 +371,14 @@ usbReapUrb (
           UsbEndpointExtension *eptx = endpoint->extension;
 
           if (enqueueItem(eptx->completedRequests, urb)) return 1;
-          LogError("USB completed request enqueue");
+          logSystemError("USB completed request enqueue");
           free(urb);
         }
       } else {
         errno = EAGAIN;
       }
     } else {
-      if (wait || (errno != EAGAIN)) LogError("USB URB reap");
+      if (wait || (errno != EAGAIN)) logSystemError("USB URB reap");
     }
   }
 
@@ -390,7 +390,7 @@ usbSubmitRequest (
   UsbDevice *device,
   unsigned char endpointAddress,
   void *buffer,
-  int length,
+  size_t length,
   void *context
 ) {
   UsbDeviceExtension *devx = device->extension;
@@ -429,9 +429,9 @@ usbSubmitRequest (
         }
 
       /*
-        LogPrint(LOG_DEBUG, "USB submit: urb=%p typ=%02X ept=%02X flg=%X sig=%d buf=%p len=%d ctx=%p",
-                 urb, urb->type, urb->endpoint, urb->flags, urb->signr,
-                 urb->buffer, urb->buffer_length, urb->usercontext);
+        logMessage(LOG_DEBUG, "USB submit: urb=%p typ=%02X ept=%02X flg=%X sig=%d buf=%p len=%d ctx=%p",
+                   urb, urb->type, urb->endpoint, urb->flags, urb->signr,
+                   urb->buffer, urb->buffer_length, urb->usercontext);
       */
       submit:
         if (ioctl(devx->usbfsFile, USBDEVFS_SUBMITURB, urb) != -1) return urb;
@@ -443,11 +443,11 @@ usbSubmitRequest (
         }
 
         /* UHCI support returns ENXIO if a URB is already submitted. */
-        if (errno != ENXIO) LogError("USB URB submit");
+        if (errno != ENXIO) logSystemError("USB URB submit");
 
         free(urb);
       } else {
-        LogError("USB URB allocate");
+        logSystemError("USB URB allocate");
       }
     }
   }
@@ -469,7 +469,7 @@ usbCancelRequest (
       if (errno == ENODEV)  {
         reap = 0;
       } else if (errno != EINVAL) {
-        LogError("USB URB discard");
+        logSystemError("USB URB discard");
       }
     }
     
@@ -495,8 +495,8 @@ usbCancelRequest (
           return 1;
         }
 
-        LogPrint(LOG_ERR, "USB request not found: urb=%p ept=%02X",
-                 urb, urb->endpoint);
+        logMessage(LOG_ERR, "USB request not found: urb=%p ept=%02X",
+                   urb, urb->endpoint);
       }
     }
   }
@@ -528,7 +528,7 @@ usbReapResponse (
     if ((response->error = urb->status)) {
       if (response->error < 0) response->error = -response->error;
       errno = response->error;
-      LogError("USB URB status");
+      logSystemError("USB URB status");
       response->count = -1;
     } else {
       response->count = urb->actual_length;
@@ -549,11 +549,11 @@ usbReapResponse (
   return NULL;
 }
 
-static int
+static ssize_t
 usbBulkTransfer (
   UsbEndpoint *endpoint,
   void *buffer,
-  int length,
+  size_t length,
   int timeout
 ) {
   UsbDeviceExtension *devx = endpoint->device->extension;
@@ -573,7 +573,7 @@ usbBulkTransfer (
       if (USB_ENDPOINT_DIRECTION(endpoint->descriptor) == UsbEndpointDirection_Input)
         if (errno == ETIMEDOUT)
           errno = EAGAIN;
-      if (errno != EAGAIN) LogError("USB bulk transfer");
+      if (errno != EAGAIN) logSystemError("USB bulk transfer");
     }
   }
 
@@ -619,15 +619,15 @@ usbInterruptTransfer (
   return NULL;
 }
 
-int
+ssize_t
 usbReadEndpoint (
   UsbDevice *device,
   unsigned char endpointNumber,
   void *buffer,
-  int length,
+  size_t length,
   int timeout
 ) {
-  int count = -1;
+  ssize_t count = -1;
   UsbEndpoint *endpoint;
 
   if ((endpoint = usbGetInputEndpoint(device, endpointNumber))) {
@@ -650,7 +650,7 @@ usbReadEndpoint (
       }
 
       default:
-        LogPrint(LOG_ERR, "USB input transfer not supported: %d", transfer);
+        logMessage(LOG_ERR, "USB input transfer not supported: %d", transfer);
         errno = ENOSYS;
         break;
     }
@@ -666,12 +666,12 @@ usbReadEndpoint (
   return count;
 }
 
-int
+ssize_t
 usbWriteEndpoint (
   UsbDevice *device,
   unsigned char endpointNumber,
   const void *buffer,
-  int length,
+  size_t length,
   int timeout
 ) {
   UsbEndpoint *endpoint;
@@ -695,7 +695,7 @@ usbWriteEndpoint (
       }
 */
       default:
-        LogPrint(LOG_ERR, "USB output transfer not supported: %d", transfer);
+        logMessage(LOG_ERR, "USB output transfer not supported: %d", transfer);
         errno = ENOSYS;
         break;
     }
@@ -718,12 +718,12 @@ usbAllocateEndpointExtension (UsbEndpoint *endpoint) {
       endpoint->extension = eptx;
       return 1;
     } else {
-      LogError("USB endpoint completed request queue allocate");
+      logSystemError("USB endpoint completed request queue allocate");
     }
 
     free(eptx);
   } else {
-    LogError("USB endpoint extension allocate");
+    logSystemError("USB endpoint extension allocate");
   }
 
   return 0;
@@ -775,7 +775,7 @@ usbTestHostDevice (void *item, void *data) {
 
     usbDeallocateDeviceExtension(devx);
   } else {
-    LogError("malloc");
+    logMallocError();
   }
 
   return 0;
@@ -820,7 +820,7 @@ usbMakeSysfsPath (const char *usbfsPath) {
 
         if (access(path, F_OK) != -1) {
           char *sysfsPath = strdup(path);
-          if (!sysfsPath) LogError("strdup");
+          if (!sysfsPath) logSystemError("strdup");
           return sysfsPath;
         }
 
@@ -860,9 +860,9 @@ usbReadHostDeviceDescriptor (UsbHostDevice *host) {
     int count = read(file, &host->usbDescriptor, UsbDescriptorSize_Device);
 
     if (count == -1) {
-      LogError("USB device descriptor read");
+      logSystemError("USB device descriptor read");
     } else if (count != UsbDescriptorSize_Device) {
-      LogPrint(LOG_ERR, "USB short device descriptor: %d", count);
+      logMessage(LOG_ERR, "USB short device descriptor: %d", count);
     } else {
       ok = 1;
 
@@ -898,12 +898,12 @@ usbAddHostDevice (const char *path) {
       if (host->sysfsPath) free(host->sysfsPath);
       free(host->usbfsPath);
     } else {
-      LogError("strdup");
+      logSystemError("strdup");
     }
 
     free(host);
   } else {
-    LogError("malloc");
+    logMallocError();
   }
 
   return ok;
@@ -965,7 +965,7 @@ usbGetFileSystem (const char *type, const FileSystemCandidate *candidates, Mount
     const FileSystemCandidate *candidate = candidates;
 
     while (candidate->path) {
-      LogPrint(LOG_DEBUG, "verifying file system path: %s: %s", type, candidate->path);
+      logMessage(LOG_DEBUG, "verifying file system path: %s: %s", type, candidate->path);
       if (candidate->verify(candidate->path)) return strdupWrapper(candidate->path);
       candidate += 1;
     }
@@ -1039,12 +1039,12 @@ usbFindDevice (UsbDeviceChooser chooser, void *data) {
       char *root;
 
       if ((root = usbGetUsbfs())) {
-        LogPrint(LOG_DEBUG, "USBFS Root: %s", root);
+        logMessage(LOG_DEBUG, "USBFS Root: %s", root);
         if (usbAddHostDevices(root)) ok = 1;
 
         free(root);
       } else {
-        LogPrint(LOG_DEBUG, "USBFS not mounted");
+        logMessage(LOG_DEBUG, "USBFS not mounted");
       }
 
       if (!ok) {
