@@ -1,13 +1,14 @@
 /*
  * XBrlAPI - A background process tinkering with X for proper BrlAPI behavior
  *
- * Copyright (C) 2003-2008 by Samuel Thibault <Samuel.Thibault@ens-lyon.org>
+ * Copyright (C) 2003-2011 by Samuel Thibault <Samuel.Thibault@ens-lyon.org>
  *
  * XBrlAPI comes with ABSOLUTELY NO WARRANTY.
  *
  * This is free software, placed under the terms of the
  * GNU General Public License, as published by the Free Software
- * Foundation.  Please see the file COPYING for details.
+ * Foundation; either version 2 of the License, or (at your option) any
+ * later version. Please see the file LICENSE-GPL for details.
  *
  * Web Page: http://mielke.cc/brltty/
  *
@@ -47,13 +48,12 @@
 #include <X11/extensions/XTest.h>
 #define CAN_SIMULATE_KEY_PRESSES
 #else /* HAVE_X11_EXTENSIONS_XTEST_H && HAVE_X11_EXTENSIONS_XKB_H */
-#warning key press simulation not supported on this installation
+#warning key press simulation not supported by this build - check that libxtst has been installed
 #endif /* HAVE_X11_EXTENSIONS_XTEST_H && HAVE_X11_EXTENSIONS_XKB_H */
 
 #define BRLAPI_NO_DEPRECATED
 #include "brlapi.h"
 
-#include "misc.h"
 #include "options.h"
 
 //#define DEBUG
@@ -70,6 +70,7 @@
 static char *auth;
 static char *host;
 static char *xDisplay;
+static int quiet;
 
 BEGIN_OPTION_TABLE(programOptions)
   { .letter = 'b',
@@ -92,6 +93,12 @@ BEGIN_OPTION_TABLE(programOptions)
     .setting.string = &xDisplay,
     .description = strtext("X display to connect to")
   },
+
+  { .letter = 'q',
+    .word = "quiet",
+    .setting.flag = &quiet,
+    .description = strtext("Do not write any text to the braille device")
+  },
 END_OPTION_TABLE
 
 /******************************************************************************
@@ -174,14 +181,20 @@ void tobrltty_init(char *auth, char *host) {
 static int getXVTnb(void);
 
 void getVT(void) {
-  if (getenv("WINDOWPATH")) {
+  char *path = getenv("WINDOWPATH");
+  int vtno = -1;
+  if (!path)
+    /* Workaround for old xinit/xdm/gdm/kdm */
+    vtno = getXVTnb();
+
+  if (path || vtno == -1) {
     if (brlapi_enterTtyModeWithPath(NULL,0,NULL)<0)
       fatal_brlapi_errno("geTtyPath",gettext("cannot get tty\n"));
   } else {
-    int vtno = getXVTnb();
     if (brlapi_enterTtyMode(vtno,NULL)<0)
       fatal_brlapi_errno("enterTtyMode",gettext("cannot get tty %d\n"),vtno);
   }
+
   if (brlapi_ignoreAllKeys()<0)
     fatal_brlapi_errno("ignoreAllKeys",gettext("cannot ignore keys\n"));
 #ifdef CAN_SIMULATE_KEY_PRESSES
@@ -297,15 +310,21 @@ static int getXVTnb(void) {
 
   root=DefaultRootWindow(dpy);
 
-  if ((property=XInternAtom(dpy,"XFree86_VT",False))==None)
-    fatal(gettext("no XFree86_VT atom\n"));
-  
-  if (XGetWindowProperty(dpy,root,property,0,1,False,AnyPropertyType,
-    &actual_type, &actual_format, &nitems, &bytes_after, &buf))
-    fatal(gettext("cannot get root window XFree86_VT property\n"));
+  if ((property=XInternAtom(dpy,"XFree86_VT",False))==None) {
+    fprintf(stderr,gettext("no XFree86_VT atom\n"));
+    return -1;
+  }
 
-  if (nitems<1)
-    fatal(gettext("no items for VT number\n"));
+  if (XGetWindowProperty(dpy,root,property,0,1,False,AnyPropertyType,
+    &actual_type, &actual_format, &nitems, &bytes_after, &buf)) {
+    fprintf(stderr,gettext("cannot get root window XFree86_VT property\n"));
+    return -1;
+  }
+
+  if (nitems<1) {
+    fprintf(stderr, gettext("no items for VT number\n"));
+    goto out;
+  }
   if (nitems>1)
     fprintf(stderr,gettext("more than one item for VT number\n"));
   switch (actual_type) {
@@ -316,11 +335,12 @@ static int getXVTnb(void) {
     case 8:  vt = (*(uint8_t *)buf); break;
     case 16: vt = (*(uint16_t *)buf); break;
     case 32: vt = (*(uint32_t *)buf); break;
-    default: fatal(gettext("bad format for VT number\n"));
+    default: fprintf(stderr, gettext("bad format for VT number\n")); goto out;
     }
     break;
-  default: fatal(gettext("bad type for VT number\n"));
+  default: fprintf(stderr, gettext("bad type for VT number\n")); goto out;
   }
+out:
   if (!XFree(buf)) fatal("XFree(VTnobuf)");
   return vt;
 }
@@ -438,10 +458,12 @@ static void setFocus(Window win) {
   curWindow=win;
   api_setFocus((uint32_t)win);
 
-  if (!(window=window_of_Window(win))) {
-    fprintf(stderr,gettext("didn't grab window %#010lx but got focus\n"),win);
-    api_setName("unknown");
-  } else setName(window);
+  if (!quiet) {
+    if (!(window=window_of_Window(win))) {
+      fprintf(stderr,gettext("didn't grab window %#010lx but got focus\n"),win);
+      api_setName("unknown");
+    } else setName(window);
+  }
 }
 
 #ifdef CAN_SIMULATE_KEY_PRESSES
@@ -634,7 +656,8 @@ void toX_f(const char *display) {
     if (haveXTest && FD_ISSET(brlapi_fd,&readfds)) {
       while ((res = brlapi_readKey(0, &code)==1)) {
 	if (((code & BRLAPI_KEY_TYPE_MASK) != BRLAPI_KEY_TYPE_SYM)) {
-	  fprintf(stderr,gettext("unexpected block type %" BRLAPI_PRIxKEYCODE "\n"),code);
+	  fprintf(stderr, "%s: %" BRLAPI_PRIxKEYCODE "\n",
+                  gettext("unexpected block type"), code);
 	  continue;
 	}
 
